/* valaccodeconditionalexpression.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaccodeconditionalexpression.h>
#include <string.h>
#include <valaccodenode.h>

struct _ValaCCodeConditionalExpressionPrivate {
	ValaCCodeExpression* _condition;
	ValaCCodeExpression* _true_expression;
	ValaCCodeExpression* _false_expression;
};
#define VALA_CCODE_CONDITIONAL_EXPRESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_CONDITIONAL_EXPRESSION, ValaCCodeConditionalExpressionPrivate))
enum  {
	VALA_CCODE_CONDITIONAL_EXPRESSION_DUMMY_PROPERTY,
	VALA_CCODE_CONDITIONAL_EXPRESSION_CONDITION,
	VALA_CCODE_CONDITIONAL_EXPRESSION_TRUE_EXPRESSION,
	VALA_CCODE_CONDITIONAL_EXPRESSION_FALSE_EXPRESSION,
};
static void vala_ccode_conditional_expression_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer);
static void vala_ccode_conditional_expression_dispose (GObject * obj);

static void vala_ccode_conditional_expression_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer)
{
	ValaCCodeConditionalExpression * self = VALA_CCODE_CONDITIONAL_EXPRESSION (base);
	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
	vala_ccode_writer_write_string (writer, "(");
	vala_ccode_node_write (VALA_CCODE_NODE (vala_ccode_conditional_expression_get_condition (self)), writer);
	vala_ccode_writer_write_string (writer, " ? ");
	vala_ccode_node_write (VALA_CCODE_NODE (vala_ccode_conditional_expression_get_true_expression (self)), writer);
	vala_ccode_writer_write_string (writer, " : ");
	vala_ccode_node_write (VALA_CCODE_NODE (vala_ccode_conditional_expression_get_false_expression (self)), writer);
	vala_ccode_writer_write_string (writer, ")");
}


ValaCCodeExpression* vala_ccode_conditional_expression_get_condition (ValaCCodeConditionalExpression* self)
{
	g_return_val_if_fail (VALA_IS_CCODE_CONDITIONAL_EXPRESSION (self), NULL);
	return self->priv->_condition;
}


void vala_ccode_conditional_expression_set_condition (ValaCCodeConditionalExpression* self, ValaCCodeExpression* value)
{
	g_return_if_fail (VALA_IS_CCODE_CONDITIONAL_EXPRESSION (self));
	ValaCCodeExpression* __temp1 = NULL;
	self->priv->_condition = (__temp1 = g_object_ref (value), (self->priv->_condition == NULL ? NULL : (self->priv->_condition = (g_object_unref (self->priv->_condition), NULL))), __temp1);
}


ValaCCodeExpression* vala_ccode_conditional_expression_get_true_expression (ValaCCodeConditionalExpression* self)
{
	g_return_val_if_fail (VALA_IS_CCODE_CONDITIONAL_EXPRESSION (self), NULL);
	return self->priv->_true_expression;
}


void vala_ccode_conditional_expression_set_true_expression (ValaCCodeConditionalExpression* self, ValaCCodeExpression* value)
{
	g_return_if_fail (VALA_IS_CCODE_CONDITIONAL_EXPRESSION (self));
	ValaCCodeExpression* __temp3 = NULL;
	self->priv->_true_expression = (__temp3 = g_object_ref (value), (self->priv->_true_expression == NULL ? NULL : (self->priv->_true_expression = (g_object_unref (self->priv->_true_expression), NULL))), __temp3);
}


ValaCCodeExpression* vala_ccode_conditional_expression_get_false_expression (ValaCCodeConditionalExpression* self)
{
	g_return_val_if_fail (VALA_IS_CCODE_CONDITIONAL_EXPRESSION (self), NULL);
	return self->priv->_false_expression;
}


void vala_ccode_conditional_expression_set_false_expression (ValaCCodeConditionalExpression* self, ValaCCodeExpression* value)
{
	g_return_if_fail (VALA_IS_CCODE_CONDITIONAL_EXPRESSION (self));
	ValaCCodeExpression* __temp5 = NULL;
	self->priv->_false_expression = (__temp5 = g_object_ref (value), (self->priv->_false_expression == NULL ? NULL : (self->priv->_false_expression = (g_object_unref (self->priv->_false_expression), NULL))), __temp5);
}


void vala_ccode_conditional_expression_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaCCodeConditionalExpression * self = VALA_CCODE_CONDITIONAL_EXPRESSION (object);
	switch (property_id) {
		case VALA_CCODE_CONDITIONAL_EXPRESSION_CONDITION:
		g_value_set_object (value, vala_ccode_conditional_expression_get_condition (self));
		break;
		case VALA_CCODE_CONDITIONAL_EXPRESSION_TRUE_EXPRESSION:
		g_value_set_object (value, vala_ccode_conditional_expression_get_true_expression (self));
		break;
		case VALA_CCODE_CONDITIONAL_EXPRESSION_FALSE_EXPRESSION:
		g_value_set_object (value, vala_ccode_conditional_expression_get_false_expression (self));
		break;
	}
}


void vala_ccode_conditional_expression_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaCCodeConditionalExpression * self = VALA_CCODE_CONDITIONAL_EXPRESSION (object);
	switch (property_id) {
		case VALA_CCODE_CONDITIONAL_EXPRESSION_CONDITION:
		vala_ccode_conditional_expression_set_condition (self, g_value_get_object (value));
		break;
		case VALA_CCODE_CONDITIONAL_EXPRESSION_TRUE_EXPRESSION:
		vala_ccode_conditional_expression_set_true_expression (self, g_value_get_object (value));
		break;
		case VALA_CCODE_CONDITIONAL_EXPRESSION_FALSE_EXPRESSION:
		vala_ccode_conditional_expression_set_false_expression (self, g_value_get_object (value));
		break;
	}
}


static void vala_ccode_conditional_expression_class_init (ValaCCodeConditionalExpressionClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaCCodeConditionalExpressionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_ccode_conditional_expression_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_ccode_conditional_expression_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_ccode_conditional_expression_dispose;
	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_conditional_expression_real_write;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_CONDITIONAL_EXPRESSION_CONDITION, g_param_spec_object ("condition", "foo", "bar", VALA_TYPE_CCODE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_CONDITIONAL_EXPRESSION_TRUE_EXPRESSION, g_param_spec_object ("true-expression", "foo", "bar", VALA_TYPE_CCODE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_CONDITIONAL_EXPRESSION_FALSE_EXPRESSION, g_param_spec_object ("false-expression", "foo", "bar", VALA_TYPE_CCODE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_ccode_conditional_expression_init (ValaCCodeConditionalExpression * self)
{
	self->priv = VALA_CCODE_CONDITIONAL_EXPRESSION_GET_PRIVATE (self);
}


static void vala_ccode_conditional_expression_dispose (GObject * obj)
{
	ValaCCodeConditionalExpression * self = VALA_CCODE_CONDITIONAL_EXPRESSION (obj);
	(self->priv->_condition == NULL ? NULL : (self->priv->_condition = (g_object_unref (self->priv->_condition), NULL)));
	(self->priv->_true_expression == NULL ? NULL : (self->priv->_true_expression = (g_object_unref (self->priv->_true_expression), NULL)));
	(self->priv->_false_expression == NULL ? NULL : (self->priv->_false_expression = (g_object_unref (self->priv->_false_expression), NULL)));
	ValaCCodeConditionalExpressionClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CCODE_CONDITIONAL_EXPRESSION_CLASS (g_type_class_peek (VALA_TYPE_CCODE_CONDITIONAL_EXPRESSION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_ccode_conditional_expression_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeConditionalExpressionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_conditional_expression_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeConditionalExpression), 0, (GInstanceInitFunc) vala_ccode_conditional_expression_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CCODE_EXPRESSION, "ValaCCodeConditionalExpression", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



