/* valaccodeincludedirective.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaccodeincludedirective.h>

struct _ValaCCodeIncludeDirectivePrivate {
	char* _filename;
};
#define VALA_CCODE_INCLUDE_DIRECTIVE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_INCLUDE_DIRECTIVE, ValaCCodeIncludeDirectivePrivate))
enum  {
	VALA_CCODE_INCLUDE_DIRECTIVE_DUMMY_PROPERTY,
	VALA_CCODE_INCLUDE_DIRECTIVE_FILENAME,
};
static void vala_ccode_include_directive_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer);
static void vala_ccode_include_directive_dispose (GObject * obj);

static void vala_ccode_include_directive_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer)
{
	ValaCCodeIncludeDirective * self = VALA_CCODE_INCLUDE_DIRECTIVE (base);
	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
	vala_ccode_writer_write_indent (writer);
	vala_ccode_writer_write_string (writer, "#include <");
	vala_ccode_writer_write_string (writer, vala_ccode_include_directive_get_filename (self));
	vala_ccode_writer_write_string (writer, ">");
	vala_ccode_writer_write_newline (writer);
}


char* vala_ccode_include_directive_get_filename (ValaCCodeIncludeDirective* self)
{
	g_return_val_if_fail (VALA_IS_CCODE_INCLUDE_DIRECTIVE (self), NULL);
	return self->priv->_filename;
}


void vala_ccode_include_directive_set_filename (ValaCCodeIncludeDirective* self, char* value)
{
	g_return_if_fail (VALA_IS_CCODE_INCLUDE_DIRECTIVE (self));
	char* __temp1 = NULL;
	self->priv->_filename = (__temp1 = g_strdup (value), (self->priv->_filename == NULL ? NULL : (self->priv->_filename = (g_free (self->priv->_filename), NULL))), __temp1);
}


void vala_ccode_include_directive_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaCCodeIncludeDirective * self = VALA_CCODE_INCLUDE_DIRECTIVE (object);
	switch (property_id) {
		case VALA_CCODE_INCLUDE_DIRECTIVE_FILENAME:
		g_value_set_string (value, vala_ccode_include_directive_get_filename (self));
		break;
	}
}


void vala_ccode_include_directive_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaCCodeIncludeDirective * self = VALA_CCODE_INCLUDE_DIRECTIVE (object);
	switch (property_id) {
		case VALA_CCODE_INCLUDE_DIRECTIVE_FILENAME:
		vala_ccode_include_directive_set_filename (self, g_value_get_string (value));
		break;
	}
}


static void vala_ccode_include_directive_class_init (ValaCCodeIncludeDirectiveClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaCCodeIncludeDirectivePrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_ccode_include_directive_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_ccode_include_directive_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_ccode_include_directive_dispose;
	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_include_directive_real_write;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_INCLUDE_DIRECTIVE_FILENAME, g_param_spec_string ("filename", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_ccode_include_directive_init (ValaCCodeIncludeDirective * self)
{
	self->priv = VALA_CCODE_INCLUDE_DIRECTIVE_GET_PRIVATE (self);
}


static void vala_ccode_include_directive_dispose (GObject * obj)
{
	ValaCCodeIncludeDirective * self = VALA_CCODE_INCLUDE_DIRECTIVE (obj);
	(self->priv->_filename == NULL ? NULL : (self->priv->_filename = (g_free (self->priv->_filename), NULL)));
	ValaCCodeIncludeDirectiveClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CCODE_INCLUDE_DIRECTIVE_CLASS (g_type_class_peek (VALA_TYPE_CCODE_INCLUDE_DIRECTIVE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_ccode_include_directive_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeIncludeDirectiveClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_include_directive_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeIncludeDirective), 0, (GInstanceInitFunc) vala_ccode_include_directive_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CCODE_NODE, "ValaCCodeIncludeDirective", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



