/* valaccodeparenthesizedexpression.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaccodeparenthesizedexpression.h>
#include <string.h>
#include <valaccodenode.h>

struct _ValaCCodeParenthesizedExpressionPrivate {
	ValaCCodeExpression* _inner;
};
#define VALA_CCODE_PARENTHESIZED_EXPRESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_PARENTHESIZED_EXPRESSION, ValaCCodeParenthesizedExpressionPrivate))
enum  {
	VALA_CCODE_PARENTHESIZED_EXPRESSION_DUMMY_PROPERTY,
	VALA_CCODE_PARENTHESIZED_EXPRESSION_INNER,
};
static void vala_ccode_parenthesized_expression_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer);
static void vala_ccode_parenthesized_expression_dispose (GObject * obj);

static void vala_ccode_parenthesized_expression_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer)
{
	ValaCCodeParenthesizedExpression * self = VALA_CCODE_PARENTHESIZED_EXPRESSION (base);
	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
	vala_ccode_writer_write_string (writer, "(");
	vala_ccode_node_write (VALA_CCODE_NODE (vala_ccode_parenthesized_expression_get_inner (self)), writer);
	vala_ccode_writer_write_string (writer, ")");
}


ValaCCodeExpression* vala_ccode_parenthesized_expression_get_inner (ValaCCodeParenthesizedExpression* self)
{
	g_return_val_if_fail (VALA_IS_CCODE_PARENTHESIZED_EXPRESSION (self), NULL);
	return self->priv->_inner;
}


void vala_ccode_parenthesized_expression_set_inner (ValaCCodeParenthesizedExpression* self, ValaCCodeExpression* value)
{
	g_return_if_fail (VALA_IS_CCODE_PARENTHESIZED_EXPRESSION (self));
	ValaCCodeExpression* __temp1 = NULL;
	self->priv->_inner = (__temp1 = g_object_ref (value), (self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL))), __temp1);
}


void vala_ccode_parenthesized_expression_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaCCodeParenthesizedExpression * self = VALA_CCODE_PARENTHESIZED_EXPRESSION (object);
	switch (property_id) {
		case VALA_CCODE_PARENTHESIZED_EXPRESSION_INNER:
		g_value_set_object (value, vala_ccode_parenthesized_expression_get_inner (self));
		break;
	}
}


void vala_ccode_parenthesized_expression_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaCCodeParenthesizedExpression * self = VALA_CCODE_PARENTHESIZED_EXPRESSION (object);
	switch (property_id) {
		case VALA_CCODE_PARENTHESIZED_EXPRESSION_INNER:
		vala_ccode_parenthesized_expression_set_inner (self, g_value_get_object (value));
		break;
	}
}


static void vala_ccode_parenthesized_expression_class_init (ValaCCodeParenthesizedExpressionClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaCCodeParenthesizedExpressionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_ccode_parenthesized_expression_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_ccode_parenthesized_expression_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_ccode_parenthesized_expression_dispose;
	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_parenthesized_expression_real_write;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_PARENTHESIZED_EXPRESSION_INNER, g_param_spec_object ("inner", "foo", "bar", VALA_TYPE_CCODE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_ccode_parenthesized_expression_init (ValaCCodeParenthesizedExpression * self)
{
	self->priv = VALA_CCODE_PARENTHESIZED_EXPRESSION_GET_PRIVATE (self);
}


static void vala_ccode_parenthesized_expression_dispose (GObject * obj)
{
	ValaCCodeParenthesizedExpression * self = VALA_CCODE_PARENTHESIZED_EXPRESSION (obj);
	(self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL)));
	ValaCCodeParenthesizedExpressionClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CCODE_PARENTHESIZED_EXPRESSION_CLASS (g_type_class_peek (VALA_TYPE_CCODE_PARENTHESIZED_EXPRESSION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_ccode_parenthesized_expression_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeParenthesizedExpressionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_parenthesized_expression_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeParenthesizedExpression), 0, (GInstanceInitFunc) vala_ccode_parenthesized_expression_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CCODE_EXPRESSION, "ValaCCodeParenthesizedExpression", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



