/* valaccodetypedefinition.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaccodetypedefinition.h>

struct _ValaCCodeTypeDefinitionPrivate {
	char* _type_name;
	ValaCCodeDeclarator* _declarator;
};
#define VALA_CCODE_TYPE_DEFINITION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_TYPE_DEFINITION, ValaCCodeTypeDefinitionPrivate))
enum  {
	VALA_CCODE_TYPE_DEFINITION_DUMMY_PROPERTY,
	VALA_CCODE_TYPE_DEFINITION_TYPE_NAME,
	VALA_CCODE_TYPE_DEFINITION_DECLARATOR,
};
static void vala_ccode_type_definition_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer);
static void vala_ccode_type_definition_dispose (GObject * obj);

static void vala_ccode_type_definition_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer)
{
	ValaCCodeTypeDefinition * self = VALA_CCODE_TYPE_DEFINITION (base);
	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
	vala_ccode_writer_write_indent (writer);
	vala_ccode_writer_write_string (writer, "typedef ");
	vala_ccode_writer_write_string (writer, vala_ccode_type_definition_get_type_name (self));
	vala_ccode_writer_write_string (writer, " ");
	vala_ccode_node_write (VALA_CCODE_NODE (vala_ccode_type_definition_get_declarator (self)), writer);
	vala_ccode_writer_write_string (writer, ";");
	vala_ccode_writer_write_newline (writer);
}


char* vala_ccode_type_definition_get_type_name (ValaCCodeTypeDefinition* self)
{
	g_return_val_if_fail (VALA_IS_CCODE_TYPE_DEFINITION (self), NULL);
	return self->priv->_type_name;
}


void vala_ccode_type_definition_set_type_name (ValaCCodeTypeDefinition* self, char* value)
{
	g_return_if_fail (VALA_IS_CCODE_TYPE_DEFINITION (self));
	char* __temp2 = NULL;
	char* __temp1 = NULL;
	self->priv->_type_name = (__temp2 = (__temp1 = value, (__temp1 == NULL ? __temp1 : g_strdup (__temp1))), (self->priv->_type_name == NULL ? NULL : (self->priv->_type_name = (g_free (self->priv->_type_name), NULL))), __temp2);
}


ValaCCodeDeclarator* vala_ccode_type_definition_get_declarator (ValaCCodeTypeDefinition* self)
{
	g_return_val_if_fail (VALA_IS_CCODE_TYPE_DEFINITION (self), NULL);
	return self->priv->_declarator;
}


void vala_ccode_type_definition_set_declarator (ValaCCodeTypeDefinition* self, ValaCCodeDeclarator* value)
{
	g_return_if_fail (VALA_IS_CCODE_TYPE_DEFINITION (self));
	ValaCCodeDeclarator* __temp5 = NULL;
	ValaCCodeDeclarator* __temp4 = NULL;
	self->priv->_declarator = (__temp5 = (__temp4 = value, (__temp4 == NULL ? __temp4 : g_object_ref (__temp4))), (self->priv->_declarator == NULL ? NULL : (self->priv->_declarator = (g_object_unref (self->priv->_declarator), NULL))), __temp5);
}


void vala_ccode_type_definition_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaCCodeTypeDefinition * self = VALA_CCODE_TYPE_DEFINITION (object);
	switch (property_id) {
		case VALA_CCODE_TYPE_DEFINITION_TYPE_NAME:
		g_value_set_string (value, vala_ccode_type_definition_get_type_name (self));
		break;
		case VALA_CCODE_TYPE_DEFINITION_DECLARATOR:
		g_value_set_object (value, vala_ccode_type_definition_get_declarator (self));
		break;
	}
}


void vala_ccode_type_definition_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaCCodeTypeDefinition * self = VALA_CCODE_TYPE_DEFINITION (object);
	switch (property_id) {
		case VALA_CCODE_TYPE_DEFINITION_TYPE_NAME:
		vala_ccode_type_definition_set_type_name (self, g_value_get_string (value));
		break;
		case VALA_CCODE_TYPE_DEFINITION_DECLARATOR:
		vala_ccode_type_definition_set_declarator (self, g_value_get_object (value));
		break;
	}
}


static void vala_ccode_type_definition_class_init (ValaCCodeTypeDefinitionClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaCCodeTypeDefinitionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_ccode_type_definition_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_ccode_type_definition_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_ccode_type_definition_dispose;
	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_type_definition_real_write;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_TYPE_DEFINITION_TYPE_NAME, g_param_spec_string ("type-name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_TYPE_DEFINITION_DECLARATOR, g_param_spec_object ("declarator", "foo", "bar", VALA_TYPE_CCODE_DECLARATOR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_ccode_type_definition_init (ValaCCodeTypeDefinition * self)
{
	self->priv = VALA_CCODE_TYPE_DEFINITION_GET_PRIVATE (self);
}


static void vala_ccode_type_definition_dispose (GObject * obj)
{
	ValaCCodeTypeDefinition * self = VALA_CCODE_TYPE_DEFINITION (obj);
	(self->priv->_type_name == NULL ? NULL : (self->priv->_type_name = (g_free (self->priv->_type_name), NULL)));
	(self->priv->_declarator == NULL ? NULL : (self->priv->_declarator = (g_object_unref (self->priv->_declarator), NULL)));
	ValaCCodeTypeDefinitionClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CCODE_TYPE_DEFINITION_CLASS (g_type_class_peek (VALA_TYPE_CCODE_TYPE_DEFINITION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_ccode_type_definition_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeTypeDefinitionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_type_definition_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeTypeDefinition), 0, (GInstanceInitFunc) vala_ccode_type_definition_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CCODE_NODE, "ValaCCodeTypeDefinition", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



