/* valaattribute.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaattribute.h>
#include <valasourcereference.h>
#include <valanamedargument.h>

struct _ValaAttributePrivate {
	char* _name;
};
#define VALA_ATTRIBUTE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_ATTRIBUTE, ValaAttributePrivate))
enum  {
	VALA_ATTRIBUTE_DUMMY_PROPERTY,
	VALA_ATTRIBUTE_NAME,
};
static void vala_attribute_dispose (GObject * obj);

/**
 * Creates a new attribute.
 *
 * @param name attribute type name
 * @param source reference to source code
 * @return newly created attribute
 */
ValaAttribute* vala_attribute_new (char* name, ValaSourceReference* source)
{
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_ATTRIBUTE, "name", name, "source_reference", source, NULL));
}


/**
 * Adds an attribute argument.
 *
 * @param arg named argument
 */
void vala_attribute_add_argument (ValaAttribute* self, ValaNamedArgument* arg)
{
	g_return_if_fail (VALA_IS_ATTRIBUTE (self));
	g_return_if_fail (VALA_IS_NAMED_ARGUMENT (arg));
	self->args = g_list_append (self->args, g_object_ref (arg));
}


char* vala_attribute_get_name (ValaAttribute* self)
{
	g_return_val_if_fail (VALA_IS_ATTRIBUTE (self), NULL);
	return self->priv->_name;
}


void vala_attribute_set_name (ValaAttribute* self, char* value)
{
	g_return_if_fail (VALA_IS_ATTRIBUTE (self));
	char* __temp2 = NULL;
	self->priv->_name = (__temp2 = g_strdup (value), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp2);
}


void vala_attribute_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaAttribute * self = VALA_ATTRIBUTE (object);
	switch (property_id) {
		case VALA_ATTRIBUTE_NAME:
		g_value_set_string (value, vala_attribute_get_name (self));
		break;
	}
}


void vala_attribute_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaAttribute * self = VALA_ATTRIBUTE (object);
	switch (property_id) {
		case VALA_ATTRIBUTE_NAME:
		vala_attribute_set_name (self, g_value_get_string (value));
		break;
	}
}


static void vala_attribute_class_init (ValaAttributeClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaAttributePrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_attribute_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_attribute_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_attribute_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ATTRIBUTE_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_attribute_init (ValaAttribute * self)
{
	self->priv = VALA_ATTRIBUTE_GET_PRIVATE (self);
}


static void vala_attribute_dispose (GObject * obj)
{
	ValaAttribute * self = VALA_ATTRIBUTE (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	(self->args == NULL ? NULL : (self->args = (g_list_foreach (self->args, (GFunc) g_object_unref, NULL), g_list_free (self->args), NULL)));
	ValaAttributeClass * klass;
	GObjectClass * parent_class;
	klass = VALA_ATTRIBUTE_CLASS (g_type_class_peek (VALA_TYPE_ATTRIBUTE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_attribute_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaAttributeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_attribute_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaAttribute), 0, (GInstanceInitFunc) vala_attribute_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaAttribute", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



