/* valacharacterliteral.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valacharacterliteral.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaCharacterLiteralPrivate {
	char* _value;
};
#define VALA_CHARACTER_LITERAL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CHARACTER_LITERAL, ValaCharacterLiteralPrivate))
enum  {
	VALA_CHARACTER_LITERAL_DUMMY_PROPERTY,
	VALA_CHARACTER_LITERAL_VALUE,
};
static void vala_character_literal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_character_literal_dispose (GObject * obj);

/**
 * Creates a new character literal.
 *
 * @param c      character
 * @param source reference to source code
 * @return       newly created character literal
 */
ValaCharacterLiteral* vala_character_literal_new (char* c, ValaSourceReference* source)
{
	g_return_val_if_fail (c != NULL, NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_CHARACTER_LITERAL, "value", c, "source_reference", source, NULL));
}


static void vala_character_literal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaCharacterLiteral * self = VALA_CHARACTER_LITERAL (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_character_literal (visitor, self);
}


char* vala_character_literal_get_value (ValaCharacterLiteral* self)
{
	g_return_val_if_fail (VALA_IS_CHARACTER_LITERAL (self), NULL);
	return self->priv->_value;
}


void vala_character_literal_set_value (ValaCharacterLiteral* self, char* value)
{
	g_return_if_fail (VALA_IS_CHARACTER_LITERAL (self));
	char* __temp2 = NULL;
	self->priv->_value = (__temp2 = g_strdup (value), (self->priv->_value == NULL ? NULL : (self->priv->_value = (g_free (self->priv->_value), NULL))), __temp2);
}


void vala_character_literal_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaCharacterLiteral * self = VALA_CHARACTER_LITERAL (object);
	switch (property_id) {
		case VALA_CHARACTER_LITERAL_VALUE:
		g_value_set_string (value, vala_character_literal_get_value (self));
		break;
	}
}


void vala_character_literal_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaCharacterLiteral * self = VALA_CHARACTER_LITERAL (object);
	switch (property_id) {
		case VALA_CHARACTER_LITERAL_VALUE:
		vala_character_literal_set_value (self, g_value_get_string (value));
		break;
	}
}


static void vala_character_literal_class_init (ValaCharacterLiteralClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaCharacterLiteralPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_character_literal_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_character_literal_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_character_literal_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_character_literal_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CHARACTER_LITERAL_VALUE, g_param_spec_string ("value", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_character_literal_init (ValaCharacterLiteral * self)
{
	self->priv = VALA_CHARACTER_LITERAL_GET_PRIVATE (self);
}


static void vala_character_literal_dispose (GObject * obj)
{
	ValaCharacterLiteral * self = VALA_CHARACTER_LITERAL (obj);
	(self->priv->_value == NULL ? NULL : (self->priv->_value = (g_free (self->priv->_value), NULL)));
	ValaCharacterLiteralClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CHARACTER_LITERAL_CLASS (g_type_class_peek (VALA_TYPE_CHARACTER_LITERAL));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_character_literal_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCharacterLiteralClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_character_literal_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCharacterLiteral), 0, (GInstanceInitFunc) vala_character_literal_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_LITERAL, "ValaCharacterLiteral", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



