/* valacodenode.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valacodenode.h>
#include <valasymbol.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaCodeNodePrivate {
	ValaSymbol* _symbol;
	ValaSourceReference* _source_reference;
	ValaCCodeNode* _ccodenode;
	gboolean _error;
};
#define VALA_CODE_NODE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CODE_NODE, ValaCodeNodePrivate))
enum  {
	VALA_CODE_NODE_DUMMY_PROPERTY,
	VALA_CODE_NODE_SYMBOL,
	VALA_CODE_NODE_SOURCE_REFERENCE,
	VALA_CODE_NODE_CCODENODE,
	VALA_CODE_NODE_ERROR,
};
static void vala_code_node_dispose (GObject * obj);

void vala_code_node_accept (ValaCodeNode* self, ValaCodeVisitor* visitor)
{
	VALA_CODE_NODE_GET_CLASS (self)->accept (self, visitor);
}


ValaSymbol* vala_code_node_get_symbol (ValaCodeNode* self)
{
	g_return_val_if_fail (VALA_IS_CODE_NODE (self), NULL);
	return self->priv->_symbol;
}


void vala_code_node_set_symbol (ValaCodeNode* self, ValaSymbol* value)
{
	g_return_if_fail (VALA_IS_CODE_NODE (self));
	ValaSymbol* __temp2 = NULL;
	ValaSymbol* __temp1 = NULL;
	self->priv->_symbol = (__temp2 = (__temp1 = value, (__temp1 == NULL ? __temp1 : g_object_ref (__temp1))), (self->priv->_symbol == NULL ? NULL : (self->priv->_symbol = (g_object_unref (self->priv->_symbol), NULL))), __temp2);
}


ValaSourceReference* vala_code_node_get_source_reference (ValaCodeNode* self)
{
	g_return_val_if_fail (VALA_IS_CODE_NODE (self), NULL);
	return self->priv->_source_reference;
}


void vala_code_node_set_source_reference (ValaCodeNode* self, ValaSourceReference* value)
{
	g_return_if_fail (VALA_IS_CODE_NODE (self));
	ValaSourceReference* __temp5 = NULL;
	ValaSourceReference* __temp4 = NULL;
	self->priv->_source_reference = (__temp5 = (__temp4 = value, (__temp4 == NULL ? __temp4 : g_object_ref (__temp4))), (self->priv->_source_reference == NULL ? NULL : (self->priv->_source_reference = (g_object_unref (self->priv->_source_reference), NULL))), __temp5);
}


ValaCCodeNode* vala_code_node_get_ccodenode (ValaCodeNode* self)
{
	g_return_val_if_fail (VALA_IS_CODE_NODE (self), NULL);
	return self->priv->_ccodenode;
}


void vala_code_node_set_ccodenode (ValaCodeNode* self, ValaCCodeNode* value)
{
	g_return_if_fail (VALA_IS_CODE_NODE (self));
	ValaCCodeNode* __temp8 = NULL;
	ValaCCodeNode* __temp7 = NULL;
	self->priv->_ccodenode = (__temp8 = (__temp7 = value, (__temp7 == NULL ? __temp7 : g_object_ref (__temp7))), (self->priv->_ccodenode == NULL ? NULL : (self->priv->_ccodenode = (g_object_unref (self->priv->_ccodenode), NULL))), __temp8);
}


gboolean vala_code_node_get_error (ValaCodeNode* self)
{
	g_return_val_if_fail (VALA_IS_CODE_NODE (self), FALSE);
	return self->priv->_error;
}


void vala_code_node_set_error (ValaCodeNode* self, gboolean value)
{
	g_return_if_fail (VALA_IS_CODE_NODE (self));
	self->priv->_error = value;
}


void vala_code_node_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaCodeNode * self = VALA_CODE_NODE (object);
	switch (property_id) {
		case VALA_CODE_NODE_SYMBOL:
		g_value_set_object (value, vala_code_node_get_symbol (self));
		break;
		case VALA_CODE_NODE_SOURCE_REFERENCE:
		g_value_set_object (value, vala_code_node_get_source_reference (self));
		break;
		case VALA_CODE_NODE_CCODENODE:
		g_value_set_object (value, vala_code_node_get_ccodenode (self));
		break;
		case VALA_CODE_NODE_ERROR:
		g_value_set_boolean (value, vala_code_node_get_error (self));
		break;
	}
}


void vala_code_node_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaCodeNode * self = VALA_CODE_NODE (object);
	switch (property_id) {
		case VALA_CODE_NODE_SYMBOL:
		vala_code_node_set_symbol (self, g_value_get_object (value));
		break;
		case VALA_CODE_NODE_SOURCE_REFERENCE:
		vala_code_node_set_source_reference (self, g_value_get_object (value));
		break;
		case VALA_CODE_NODE_CCODENODE:
		vala_code_node_set_ccodenode (self, g_value_get_object (value));
		break;
		case VALA_CODE_NODE_ERROR:
		vala_code_node_set_error (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_code_node_class_init (ValaCodeNodeClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaCodeNodePrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_code_node_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_code_node_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_code_node_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CODE_NODE_SYMBOL, g_param_spec_object ("symbol", "foo", "bar", VALA_TYPE_SYMBOL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CODE_NODE_SOURCE_REFERENCE, g_param_spec_object ("source-reference", "foo", "bar", VALA_TYPE_SOURCE_REFERENCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CODE_NODE_CCODENODE, g_param_spec_object ("ccodenode", "foo", "bar", VALA_TYPE_CCODE_NODE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CODE_NODE_ERROR, g_param_spec_boolean ("error", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_code_node_init (ValaCodeNode * self)
{
	self->priv = VALA_CODE_NODE_GET_PRIVATE (self);
}


static void vala_code_node_dispose (GObject * obj)
{
	ValaCodeNode * self = VALA_CODE_NODE (obj);
	(self->priv->_symbol == NULL ? NULL : (self->priv->_symbol = (g_object_unref (self->priv->_symbol), NULL)));
	(self->priv->_source_reference == NULL ? NULL : (self->priv->_source_reference = (g_object_unref (self->priv->_source_reference), NULL)));
	(self->attributes == NULL ? NULL : (self->attributes = (g_list_foreach (self->attributes, (GFunc) g_object_unref, NULL), g_list_free (self->attributes), NULL)));
	(self->priv->_ccodenode == NULL ? NULL : (self->priv->_ccodenode = (g_object_unref (self->priv->_ccodenode), NULL)));
	ValaCodeNodeClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CODE_NODE_CLASS (g_type_class_peek (VALA_TYPE_CODE_NODE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_code_node_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCodeNodeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_code_node_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCodeNode), 0, (GInstanceInitFunc) vala_code_node_init };
		g_define_type_id = g_type_register_static (G_TYPE_OBJECT, "ValaCodeNode", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return g_define_type_id;
}



