/* valadeclarationstatement.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valadeclarationstatement.h>
#include <valalocalvariabledeclaration.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaDeclarationStatementPrivate {
	ValaLocalVariableDeclaration* _declaration;
};
#define VALA_DECLARATION_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_DECLARATION_STATEMENT, ValaDeclarationStatementPrivate))
enum  {
	VALA_DECLARATION_STATEMENT_DUMMY_PROPERTY,
	VALA_DECLARATION_STATEMENT_DECLARATION,
};
static void vala_declaration_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_declaration_statement_dispose (GObject * obj);

/**
 * Creates a new declaration statement.
 *
 * @param decl   local variable declaration
 * @param source reference to source code
 * @return       newly created declaration statement
 */
ValaDeclarationStatement* vala_declaration_statement_new (ValaLocalVariableDeclaration* decl, ValaSourceReference* source)
{
	g_return_val_if_fail (VALA_IS_LOCAL_VARIABLE_DECLARATION (decl), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_DECLARATION_STATEMENT, "declaration", decl, "source_reference", source, NULL));
}


static void vala_declaration_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaDeclarationStatement * self = VALA_DECLARATION_STATEMENT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (vala_declaration_statement_get_declaration (self)), visitor);
	vala_code_visitor_visit_declaration_statement (visitor, self);
}


ValaLocalVariableDeclaration* vala_declaration_statement_get_declaration (ValaDeclarationStatement* self)
{
	g_return_val_if_fail (VALA_IS_DECLARATION_STATEMENT (self), NULL);
	return self->priv->_declaration;
}


void vala_declaration_statement_set_declaration (ValaDeclarationStatement* self, ValaLocalVariableDeclaration* value)
{
	g_return_if_fail (VALA_IS_DECLARATION_STATEMENT (self));
	ValaLocalVariableDeclaration* __temp2 = NULL;
	self->priv->_declaration = (__temp2 = g_object_ref (value), (self->priv->_declaration == NULL ? NULL : (self->priv->_declaration = (g_object_unref (self->priv->_declaration), NULL))), __temp2);
}


void vala_declaration_statement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaDeclarationStatement * self = VALA_DECLARATION_STATEMENT (object);
	switch (property_id) {
		case VALA_DECLARATION_STATEMENT_DECLARATION:
		g_value_set_object (value, vala_declaration_statement_get_declaration (self));
		break;
	}
}


void vala_declaration_statement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaDeclarationStatement * self = VALA_DECLARATION_STATEMENT (object);
	switch (property_id) {
		case VALA_DECLARATION_STATEMENT_DECLARATION:
		vala_declaration_statement_set_declaration (self, g_value_get_object (value));
		break;
	}
}


static void vala_declaration_statement_class_init (ValaDeclarationStatementClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaDeclarationStatementPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_declaration_statement_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_declaration_statement_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_declaration_statement_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_declaration_statement_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_DECLARATION_STATEMENT_DECLARATION, g_param_spec_object ("declaration", "foo", "bar", VALA_TYPE_LOCAL_VARIABLE_DECLARATION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_declaration_statement_init (ValaDeclarationStatement * self)
{
	self->priv = VALA_DECLARATION_STATEMENT_GET_PRIVATE (self);
}


static void vala_declaration_statement_dispose (GObject * obj)
{
	ValaDeclarationStatement * self = VALA_DECLARATION_STATEMENT (obj);
	(self->priv->_declaration == NULL ? NULL : (self->priv->_declaration = (g_object_unref (self->priv->_declaration), NULL)));
	ValaDeclarationStatementClass * klass;
	GObjectClass * parent_class;
	klass = VALA_DECLARATION_STATEMENT_CLASS (g_type_class_peek (VALA_TYPE_DECLARATION_STATEMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_declaration_statement_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaDeclarationStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_declaration_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaDeclarationStatement), 0, (GInstanceInitFunc) vala_declaration_statement_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_STATEMENT, "ValaDeclarationStatement", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



