/* valaemptystatement.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaemptystatement.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaEmptyStatementPrivate {
};
#define VALA_EMPTY_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_EMPTY_STATEMENT, ValaEmptyStatementPrivate))
enum  {
	VALA_EMPTY_STATEMENT_DUMMY_PROPERTY,
};
static void vala_empty_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_empty_statement_dispose (GObject * obj);

/**
 * Creates a new empty statement.
 *
 * @param source reference to source code
 * @return       newly created empty statement
 */
ValaEmptyStatement* vala_empty_statement_new (ValaSourceReference* source)
{
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_EMPTY_STATEMENT, "source_reference", source, NULL));
}


static void vala_empty_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaEmptyStatement * self = VALA_EMPTY_STATEMENT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_empty_statement (visitor, self);
}


void vala_empty_statement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaEmptyStatement * self = VALA_EMPTY_STATEMENT (object);
	switch (property_id) {
	}
}


void vala_empty_statement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaEmptyStatement * self = VALA_EMPTY_STATEMENT (object);
	switch (property_id) {
	}
}


static void vala_empty_statement_class_init (ValaEmptyStatementClass * klass)
{
	G_OBJECT_CLASS (klass)->get_property = vala_empty_statement_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_empty_statement_set_property;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_empty_statement_real_accept;
}


static void vala_empty_statement_init (ValaEmptyStatement * self)
{
}


static void vala_empty_statement_dispose (GObject * obj)
{
	ValaEmptyStatement * self = VALA_EMPTY_STATEMENT (obj);
	ValaEmptyStatementClass * klass;
	GObjectClass * parent_class;
	klass = VALA_EMPTY_STATEMENT_CLASS (g_type_class_peek (VALA_TYPE_EMPTY_STATEMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_empty_statement_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaEmptyStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_empty_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaEmptyStatement), 0, (GInstanceInitFunc) vala_empty_statement_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_STATEMENT, "ValaEmptyStatement", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



