/* valaforeachstatement.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaforeachstatement.h>
#include <valatypereference.h>
#include <valaexpression.h>
#include <valavariabledeclarator.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaForeachStatementPrivate {
	ValaTypeReference* _type_reference;
	char* _variable_name;
	ValaExpression* _collection;
	ValaStatement* _body;
	ValaVariableDeclarator* _variable_declarator;
};
#define VALA_FOREACH_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_FOREACH_STATEMENT, ValaForeachStatementPrivate))
enum  {
	VALA_FOREACH_STATEMENT_DUMMY_PROPERTY,
	VALA_FOREACH_STATEMENT_TYPE_REFERENCE,
	VALA_FOREACH_STATEMENT_VARIABLE_NAME,
	VALA_FOREACH_STATEMENT_COLLECTION,
	VALA_FOREACH_STATEMENT_BODY,
	VALA_FOREACH_STATEMENT_VARIABLE_DECLARATOR,
};
static void vala_foreach_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_foreach_statement_dispose (GObject * obj);

/**
 * Creates a new foreach statement.
 *
 * @param type   element type
 * @param id     element variable name
 * @param col    loop body
 * @param source reference to source code
 * @return       newly created foreach statement
 */
ValaForeachStatement* vala_foreach_statement_new (ValaTypeReference* type, char* id, ValaExpression* col, ValaStatement* body, ValaSourceReference* source)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), NULL);
	g_return_val_if_fail (id != NULL, NULL);
	g_return_val_if_fail (VALA_IS_EXPRESSION (col), NULL);
	g_return_val_if_fail (body == NULL || VALA_IS_STATEMENT (body), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_FOREACH_STATEMENT, "type_reference", type, "variable_name", id, "collection", col, "body", body, "source_reference", source, NULL));
}


static void vala_foreach_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaForeachStatement * self = VALA_FOREACH_STATEMENT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_foreach_statement (visitor, self);
	vala_code_node_accept (VALA_CODE_NODE (vala_foreach_statement_get_type_reference (self)), visitor);
	vala_code_node_accept (VALA_CODE_NODE (vala_foreach_statement_get_collection (self)), visitor);
	vala_code_node_accept (VALA_CODE_NODE (vala_foreach_statement_get_body (self)), visitor);
	vala_code_visitor_visit_end_foreach_statement (visitor, self);
}


ValaTypeReference* vala_foreach_statement_get_type_reference (ValaForeachStatement* self)
{
	g_return_val_if_fail (VALA_IS_FOREACH_STATEMENT (self), NULL);
	return self->priv->_type_reference;
}


void vala_foreach_statement_set_type_reference (ValaForeachStatement* self, ValaTypeReference* value)
{
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (self));
	ValaTypeReference* __temp2 = NULL;
	self->priv->_type_reference = (__temp2 = g_object_ref (value), (self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL))), __temp2);
}


char* vala_foreach_statement_get_variable_name (ValaForeachStatement* self)
{
	g_return_val_if_fail (VALA_IS_FOREACH_STATEMENT (self), NULL);
	return self->priv->_variable_name;
}


void vala_foreach_statement_set_variable_name (ValaForeachStatement* self, char* value)
{
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (self));
	char* __temp4 = NULL;
	self->priv->_variable_name = (__temp4 = g_strdup (value), (self->priv->_variable_name == NULL ? NULL : (self->priv->_variable_name = (g_free (self->priv->_variable_name), NULL))), __temp4);
}


ValaExpression* vala_foreach_statement_get_collection (ValaForeachStatement* self)
{
	g_return_val_if_fail (VALA_IS_FOREACH_STATEMENT (self), NULL);
	return self->priv->_collection;
}


void vala_foreach_statement_set_collection (ValaForeachStatement* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (self));
	ValaExpression* __temp6 = NULL;
	self->priv->_collection = (__temp6 = g_object_ref (value), (self->priv->_collection == NULL ? NULL : (self->priv->_collection = (g_object_unref (self->priv->_collection), NULL))), __temp6);
}


ValaStatement* vala_foreach_statement_get_body (ValaForeachStatement* self)
{
	g_return_val_if_fail (VALA_IS_FOREACH_STATEMENT (self), NULL);
	return self->priv->_body;
}


void vala_foreach_statement_set_body (ValaForeachStatement* self, ValaStatement* value)
{
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (self));
	ValaStatement* __temp9 = NULL;
	ValaStatement* __temp8 = NULL;
	self->priv->_body = (__temp9 = (__temp8 = value, (__temp8 == NULL ? __temp8 : g_object_ref (__temp8))), (self->priv->_body == NULL ? NULL : (self->priv->_body = (g_object_unref (self->priv->_body), NULL))), __temp9);
}


ValaVariableDeclarator* vala_foreach_statement_get_variable_declarator (ValaForeachStatement* self)
{
	g_return_val_if_fail (VALA_IS_FOREACH_STATEMENT (self), NULL);
	return self->priv->_variable_declarator;
}


void vala_foreach_statement_set_variable_declarator (ValaForeachStatement* self, ValaVariableDeclarator* value)
{
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (self));
	ValaVariableDeclarator* __temp12 = NULL;
	ValaVariableDeclarator* __temp11 = NULL;
	self->priv->_variable_declarator = (__temp12 = (__temp11 = value, (__temp11 == NULL ? __temp11 : g_object_ref (__temp11))), (self->priv->_variable_declarator == NULL ? NULL : (self->priv->_variable_declarator = (g_object_unref (self->priv->_variable_declarator), NULL))), __temp12);
}


void vala_foreach_statement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaForeachStatement * self = VALA_FOREACH_STATEMENT (object);
	switch (property_id) {
		case VALA_FOREACH_STATEMENT_TYPE_REFERENCE:
		g_value_set_object (value, vala_foreach_statement_get_type_reference (self));
		break;
		case VALA_FOREACH_STATEMENT_VARIABLE_NAME:
		g_value_set_string (value, vala_foreach_statement_get_variable_name (self));
		break;
		case VALA_FOREACH_STATEMENT_COLLECTION:
		g_value_set_object (value, vala_foreach_statement_get_collection (self));
		break;
		case VALA_FOREACH_STATEMENT_BODY:
		g_value_set_object (value, vala_foreach_statement_get_body (self));
		break;
		case VALA_FOREACH_STATEMENT_VARIABLE_DECLARATOR:
		g_value_set_object (value, vala_foreach_statement_get_variable_declarator (self));
		break;
	}
}


void vala_foreach_statement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaForeachStatement * self = VALA_FOREACH_STATEMENT (object);
	switch (property_id) {
		case VALA_FOREACH_STATEMENT_TYPE_REFERENCE:
		vala_foreach_statement_set_type_reference (self, g_value_get_object (value));
		break;
		case VALA_FOREACH_STATEMENT_VARIABLE_NAME:
		vala_foreach_statement_set_variable_name (self, g_value_get_string (value));
		break;
		case VALA_FOREACH_STATEMENT_COLLECTION:
		vala_foreach_statement_set_collection (self, g_value_get_object (value));
		break;
		case VALA_FOREACH_STATEMENT_BODY:
		vala_foreach_statement_set_body (self, g_value_get_object (value));
		break;
		case VALA_FOREACH_STATEMENT_VARIABLE_DECLARATOR:
		vala_foreach_statement_set_variable_declarator (self, g_value_get_object (value));
		break;
	}
}


static void vala_foreach_statement_class_init (ValaForeachStatementClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaForeachStatementPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_foreach_statement_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_foreach_statement_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_foreach_statement_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_foreach_statement_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FOREACH_STATEMENT_TYPE_REFERENCE, g_param_spec_object ("type-reference", "foo", "bar", VALA_TYPE_TYPE_REFERENCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FOREACH_STATEMENT_VARIABLE_NAME, g_param_spec_string ("variable-name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FOREACH_STATEMENT_COLLECTION, g_param_spec_object ("collection", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FOREACH_STATEMENT_BODY, g_param_spec_object ("body", "foo", "bar", VALA_TYPE_STATEMENT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FOREACH_STATEMENT_VARIABLE_DECLARATOR, g_param_spec_object ("variable-declarator", "foo", "bar", VALA_TYPE_VARIABLE_DECLARATOR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_foreach_statement_init (ValaForeachStatement * self)
{
	self->priv = VALA_FOREACH_STATEMENT_GET_PRIVATE (self);
}


static void vala_foreach_statement_dispose (GObject * obj)
{
	ValaForeachStatement * self = VALA_FOREACH_STATEMENT (obj);
	(self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL)));
	(self->priv->_variable_name == NULL ? NULL : (self->priv->_variable_name = (g_free (self->priv->_variable_name), NULL)));
	(self->priv->_collection == NULL ? NULL : (self->priv->_collection = (g_object_unref (self->priv->_collection), NULL)));
	(self->priv->_body == NULL ? NULL : (self->priv->_body = (g_object_unref (self->priv->_body), NULL)));
	(self->priv->_variable_declarator == NULL ? NULL : (self->priv->_variable_declarator = (g_object_unref (self->priv->_variable_declarator), NULL)));
	ValaForeachStatementClass * klass;
	GObjectClass * parent_class;
	klass = VALA_FOREACH_STATEMENT_CLASS (g_type_class_peek (VALA_TYPE_FOREACH_STATEMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_foreach_statement_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaForeachStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_foreach_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaForeachStatement), 0, (GInstanceInitFunc) vala_foreach_statement_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_STATEMENT, "ValaForeachStatement", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



