/* valaintegerliteral.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaintegerliteral.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaIntegerLiteralPrivate {
	char* _value;
};
#define VALA_INTEGER_LITERAL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_INTEGER_LITERAL, ValaIntegerLiteralPrivate))
enum  {
	VALA_INTEGER_LITERAL_DUMMY_PROPERTY,
	VALA_INTEGER_LITERAL_VALUE,
};
static void vala_integer_literal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_integer_literal_dispose (GObject * obj);

/**
 * Creates a new integer literal.
 *
 * @param i      literal value
 * @param source reference to source code
 * @return       newly created integer literal
 */
ValaIntegerLiteral* vala_integer_literal_new (char* i, ValaSourceReference* source)
{
	g_return_val_if_fail (i != NULL, NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_INTEGER_LITERAL, "value", i, "source_reference", source, NULL));
}


static void vala_integer_literal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaIntegerLiteral * self = VALA_INTEGER_LITERAL (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_integer_literal (visitor, self);
}


char* vala_integer_literal_get_value (ValaIntegerLiteral* self)
{
	g_return_val_if_fail (VALA_IS_INTEGER_LITERAL (self), NULL);
	return self->priv->_value;
}


void vala_integer_literal_set_value (ValaIntegerLiteral* self, char* value)
{
	g_return_if_fail (VALA_IS_INTEGER_LITERAL (self));
	char* __temp2 = NULL;
	self->priv->_value = (__temp2 = g_strdup (value), (self->priv->_value == NULL ? NULL : (self->priv->_value = (g_free (self->priv->_value), NULL))), __temp2);
}


void vala_integer_literal_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaIntegerLiteral * self = VALA_INTEGER_LITERAL (object);
	switch (property_id) {
		case VALA_INTEGER_LITERAL_VALUE:
		g_value_set_string (value, vala_integer_literal_get_value (self));
		break;
	}
}


void vala_integer_literal_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaIntegerLiteral * self = VALA_INTEGER_LITERAL (object);
	switch (property_id) {
		case VALA_INTEGER_LITERAL_VALUE:
		vala_integer_literal_set_value (self, g_value_get_string (value));
		break;
	}
}


static void vala_integer_literal_class_init (ValaIntegerLiteralClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaIntegerLiteralPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_integer_literal_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_integer_literal_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_integer_literal_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_integer_literal_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_INTEGER_LITERAL_VALUE, g_param_spec_string ("value", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_integer_literal_init (ValaIntegerLiteral * self)
{
	self->priv = VALA_INTEGER_LITERAL_GET_PRIVATE (self);
}


static void vala_integer_literal_dispose (GObject * obj)
{
	ValaIntegerLiteral * self = VALA_INTEGER_LITERAL (obj);
	(self->priv->_value == NULL ? NULL : (self->priv->_value = (g_free (self->priv->_value), NULL)));
	ValaIntegerLiteralClass * klass;
	GObjectClass * parent_class;
	klass = VALA_INTEGER_LITERAL_CLASS (g_type_class_peek (VALA_TYPE_INTEGER_LITERAL));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_integer_literal_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaIntegerLiteralClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_integer_literal_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaIntegerLiteral), 0, (GInstanceInitFunc) vala_integer_literal_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_LITERAL, "ValaIntegerLiteral", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



