/* valanamedargument.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valanamedargument.h>
#include <valaexpression.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaNamedArgumentPrivate {
	char* _name;
	ValaExpression* _argument;
};
#define VALA_NAMED_ARGUMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_NAMED_ARGUMENT, ValaNamedArgumentPrivate))
enum  {
	VALA_NAMED_ARGUMENT_DUMMY_PROPERTY,
	VALA_NAMED_ARGUMENT_NAME,
	VALA_NAMED_ARGUMENT_ARGUMENT,
};
static void vala_named_argument_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_named_argument_dispose (GObject * obj);

ValaNamedArgument* vala_named_argument_new (char* name, ValaExpression* arg, ValaSourceReference* source)
{
	g_return_val_if_fail (arg == NULL || VALA_IS_EXPRESSION (arg), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_NAMED_ARGUMENT, "name", name, "argument", arg, "source_reference", source, NULL));
}


static void vala_named_argument_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaNamedArgument * self = VALA_NAMED_ARGUMENT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (vala_named_argument_get_argument (self)), visitor);
	vala_code_visitor_visit_named_argument (visitor, self);
}


char* vala_named_argument_get_name (ValaNamedArgument* self)
{
	g_return_val_if_fail (VALA_IS_NAMED_ARGUMENT (self), NULL);
	return self->priv->_name;
}


void vala_named_argument_set_name (ValaNamedArgument* self, char* value)
{
	g_return_if_fail (VALA_IS_NAMED_ARGUMENT (self));
	char* __temp3 = NULL;
	char* __temp2 = NULL;
	self->priv->_name = (__temp3 = (__temp2 = value, (__temp2 == NULL ? __temp2 : g_strdup (__temp2))), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp3);
}


ValaExpression* vala_named_argument_get_argument (ValaNamedArgument* self)
{
	g_return_val_if_fail (VALA_IS_NAMED_ARGUMENT (self), NULL);
	return self->priv->_argument;
}


void vala_named_argument_set_argument (ValaNamedArgument* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_NAMED_ARGUMENT (self));
	ValaExpression* __temp6 = NULL;
	ValaExpression* __temp5 = NULL;
	self->priv->_argument = (__temp6 = (__temp5 = value, (__temp5 == NULL ? __temp5 : g_object_ref (__temp5))), (self->priv->_argument == NULL ? NULL : (self->priv->_argument = (g_object_unref (self->priv->_argument), NULL))), __temp6);
}


void vala_named_argument_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaNamedArgument * self = VALA_NAMED_ARGUMENT (object);
	switch (property_id) {
		case VALA_NAMED_ARGUMENT_NAME:
		g_value_set_string (value, vala_named_argument_get_name (self));
		break;
		case VALA_NAMED_ARGUMENT_ARGUMENT:
		g_value_set_object (value, vala_named_argument_get_argument (self));
		break;
	}
}


void vala_named_argument_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaNamedArgument * self = VALA_NAMED_ARGUMENT (object);
	switch (property_id) {
		case VALA_NAMED_ARGUMENT_NAME:
		vala_named_argument_set_name (self, g_value_get_string (value));
		break;
		case VALA_NAMED_ARGUMENT_ARGUMENT:
		vala_named_argument_set_argument (self, g_value_get_object (value));
		break;
	}
}


static void vala_named_argument_class_init (ValaNamedArgumentClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaNamedArgumentPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_named_argument_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_named_argument_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_named_argument_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_named_argument_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_NAMED_ARGUMENT_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_NAMED_ARGUMENT_ARGUMENT, g_param_spec_object ("argument", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_named_argument_init (ValaNamedArgument * self)
{
	self->priv = VALA_NAMED_ARGUMENT_GET_PRIVATE (self);
}


static void vala_named_argument_dispose (GObject * obj)
{
	ValaNamedArgument * self = VALA_NAMED_ARGUMENT (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	(self->priv->_argument == NULL ? NULL : (self->priv->_argument = (g_object_unref (self->priv->_argument), NULL)));
	ValaNamedArgumentClass * klass;
	GObjectClass * parent_class;
	klass = VALA_NAMED_ARGUMENT_CLASS (g_type_class_peek (VALA_TYPE_NAMED_ARGUMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_named_argument_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaNamedArgumentClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_named_argument_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaNamedArgument), 0, (GInstanceInitFunc) vala_named_argument_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaNamedArgument", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



