/* valanullliteral.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valanullliteral.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaNullLiteralPrivate {
};
#define VALA_NULL_LITERAL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_NULL_LITERAL, ValaNullLiteralPrivate))
enum  {
	VALA_NULL_LITERAL_DUMMY_PROPERTY,
};
static void vala_null_literal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_null_literal_dispose (GObject * obj);

ValaNullLiteral* vala_null_literal_new (ValaSourceReference* source)
{
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_NULL_LITERAL, "source_reference", source, NULL));
}


static void vala_null_literal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaNullLiteral * self = VALA_NULL_LITERAL (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_null_literal (visitor, self);
}


void vala_null_literal_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaNullLiteral * self = VALA_NULL_LITERAL (object);
	switch (property_id) {
	}
}


void vala_null_literal_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaNullLiteral * self = VALA_NULL_LITERAL (object);
	switch (property_id) {
	}
}


static void vala_null_literal_class_init (ValaNullLiteralClass * klass)
{
	G_OBJECT_CLASS (klass)->get_property = vala_null_literal_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_null_literal_set_property;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_null_literal_real_accept;
}


static void vala_null_literal_init (ValaNullLiteral * self)
{
}


static void vala_null_literal_dispose (GObject * obj)
{
	ValaNullLiteral * self = VALA_NULL_LITERAL (obj);
	ValaNullLiteralClass * klass;
	GObjectClass * parent_class;
	klass = VALA_NULL_LITERAL_CLASS (g_type_class_peek (VALA_TYPE_NULL_LITERAL));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_null_literal_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaNullLiteralClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_null_literal_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaNullLiteral), 0, (GInstanceInitFunc) vala_null_literal_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_LITERAL, "ValaNullLiteral", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



