/* valaobjectcreationexpression.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaobjectcreationexpression.h>
#include <valatypereference.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>
#include <valanamedargument.h>

struct _ValaObjectCreationExpressionPrivate {
	ValaTypeReference* _type_reference;
	GList* _named_argument_list;
};
#define VALA_OBJECT_CREATION_EXPRESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_OBJECT_CREATION_EXPRESSION, ValaObjectCreationExpressionPrivate))
enum  {
	VALA_OBJECT_CREATION_EXPRESSION_DUMMY_PROPERTY,
	VALA_OBJECT_CREATION_EXPRESSION_TYPE_REFERENCE,
	VALA_OBJECT_CREATION_EXPRESSION_NAMED_ARGUMENT_LIST,
};
static void vala_object_creation_expression_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_object_creation_expression_dispose (GObject * obj);

ValaObjectCreationExpression* vala_object_creation_expression_new (ValaTypeReference* type, GList* named_argument_list, ValaSourceReference* source)
{
	g_return_val_if_fail (type == NULL || VALA_IS_TYPE_REFERENCE (type), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_OBJECT_CREATION_EXPRESSION, "type_reference", type, "named_argument_list", named_argument_list, "source_reference", source, NULL));
}


static void vala_object_creation_expression_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaObjectCreationExpression * self = VALA_OBJECT_CREATION_EXPRESSION (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (vala_object_creation_expression_get_type_reference (self)), visitor);
	{
		GList * arg_it;
		for (arg_it = vala_object_creation_expression_get_named_argument_list (self); arg_it != NULL; arg_it = arg_it->next) {
			ValaNamedArgument* arg = arg_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (arg), visitor);
			}
		}
	}
	vala_code_visitor_visit_object_creation_expression (visitor, self);
}


ValaTypeReference* vala_object_creation_expression_get_type_reference (ValaObjectCreationExpression* self)
{
	g_return_val_if_fail (VALA_IS_OBJECT_CREATION_EXPRESSION (self), NULL);
	return self->priv->_type_reference;
}


void vala_object_creation_expression_set_type_reference (ValaObjectCreationExpression* self, ValaTypeReference* value)
{
	g_return_if_fail (VALA_IS_OBJECT_CREATION_EXPRESSION (self));
	ValaTypeReference* __temp3 = NULL;
	ValaTypeReference* __temp2 = NULL;
	self->priv->_type_reference = (__temp3 = (__temp2 = value, (__temp2 == NULL ? __temp2 : g_object_ref (__temp2))), (self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL))), __temp3);
}


GList* vala_object_creation_expression_get_named_argument_list (ValaObjectCreationExpression* self)
{
	g_return_val_if_fail (VALA_IS_OBJECT_CREATION_EXPRESSION (self), NULL);
	return self->priv->_named_argument_list;
}


void vala_object_creation_expression_set_named_argument_list (ValaObjectCreationExpression* self, GList* value)
{
	g_return_if_fail (VALA_IS_OBJECT_CREATION_EXPRESSION (self));
	GList* __temp6 = NULL;
	GList* __temp5 = NULL;
	self->priv->_named_argument_list = (__temp6 = (__temp5 = value, (__temp5 == NULL ? __temp5 : g_list_copy (__temp5))), (self->priv->_named_argument_list == NULL ? NULL : (self->priv->_named_argument_list = (g_list_foreach (self->priv->_named_argument_list, (GFunc) g_object_unref, NULL), g_list_free (self->priv->_named_argument_list), NULL))), __temp6);
}


void vala_object_creation_expression_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaObjectCreationExpression * self = VALA_OBJECT_CREATION_EXPRESSION (object);
	switch (property_id) {
		case VALA_OBJECT_CREATION_EXPRESSION_TYPE_REFERENCE:
		g_value_set_object (value, vala_object_creation_expression_get_type_reference (self));
		break;
		case VALA_OBJECT_CREATION_EXPRESSION_NAMED_ARGUMENT_LIST:
		g_value_set_pointer (value, vala_object_creation_expression_get_named_argument_list (self));
		break;
	}
}


void vala_object_creation_expression_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaObjectCreationExpression * self = VALA_OBJECT_CREATION_EXPRESSION (object);
	switch (property_id) {
		case VALA_OBJECT_CREATION_EXPRESSION_TYPE_REFERENCE:
		vala_object_creation_expression_set_type_reference (self, g_value_get_object (value));
		break;
		case VALA_OBJECT_CREATION_EXPRESSION_NAMED_ARGUMENT_LIST:
		vala_object_creation_expression_set_named_argument_list (self, g_value_get_pointer (value));
		break;
	}
}


static void vala_object_creation_expression_class_init (ValaObjectCreationExpressionClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaObjectCreationExpressionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_object_creation_expression_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_object_creation_expression_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_object_creation_expression_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_object_creation_expression_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_OBJECT_CREATION_EXPRESSION_TYPE_REFERENCE, g_param_spec_object ("type-reference", "foo", "bar", VALA_TYPE_TYPE_REFERENCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_OBJECT_CREATION_EXPRESSION_NAMED_ARGUMENT_LIST, g_param_spec_pointer ("named-argument-list", "foo", "bar", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_object_creation_expression_init (ValaObjectCreationExpression * self)
{
	self->priv = VALA_OBJECT_CREATION_EXPRESSION_GET_PRIVATE (self);
}


static void vala_object_creation_expression_dispose (GObject * obj)
{
	ValaObjectCreationExpression * self = VALA_OBJECT_CREATION_EXPRESSION (obj);
	(self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL)));
	(self->priv->_named_argument_list == NULL ? NULL : (self->priv->_named_argument_list = (g_list_foreach (self->priv->_named_argument_list, (GFunc) g_object_unref, NULL), g_list_free (self->priv->_named_argument_list), NULL)));
	ValaObjectCreationExpressionClass * klass;
	GObjectClass * parent_class;
	klass = VALA_OBJECT_CREATION_EXPRESSION_CLASS (g_type_class_peek (VALA_TYPE_OBJECT_CREATION_EXPRESSION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_object_creation_expression_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaObjectCreationExpressionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_object_creation_expression_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaObjectCreationExpression), 0, (GInstanceInitFunc) vala_object_creation_expression_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_EXPRESSION, "ValaObjectCreationExpression", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



