/* valaparenthesizedexpression.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaparenthesizedexpression.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaParenthesizedExpressionPrivate {
	ValaExpression* _inner;
};
#define VALA_PARENTHESIZED_EXPRESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_PARENTHESIZED_EXPRESSION, ValaParenthesizedExpressionPrivate))
enum  {
	VALA_PARENTHESIZED_EXPRESSION_DUMMY_PROPERTY,
	VALA_PARENTHESIZED_EXPRESSION_INNER,
};
static void vala_parenthesized_expression_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_parenthesized_expression_dispose (GObject * obj);

ValaParenthesizedExpression* vala_parenthesized_expression_new (ValaExpression* inner, ValaSourceReference* source)
{
	g_return_val_if_fail (inner == NULL || VALA_IS_EXPRESSION (inner), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_PARENTHESIZED_EXPRESSION, "inner", inner, "source_reference", source, NULL));
}


static void vala_parenthesized_expression_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaParenthesizedExpression * self = VALA_PARENTHESIZED_EXPRESSION (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (vala_parenthesized_expression_get_inner (self)), visitor);
	vala_code_visitor_visit_parenthesized_expression (visitor, self);
}


ValaExpression* vala_parenthesized_expression_get_inner (ValaParenthesizedExpression* self)
{
	g_return_val_if_fail (VALA_IS_PARENTHESIZED_EXPRESSION (self), NULL);
	return self->priv->_inner;
}


void vala_parenthesized_expression_set_inner (ValaParenthesizedExpression* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_PARENTHESIZED_EXPRESSION (self));
	ValaExpression* __temp3 = NULL;
	ValaExpression* __temp2 = NULL;
	self->priv->_inner = (__temp3 = (__temp2 = value, (__temp2 == NULL ? __temp2 : g_object_ref (__temp2))), (self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL))), __temp3);
}


void vala_parenthesized_expression_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaParenthesizedExpression * self = VALA_PARENTHESIZED_EXPRESSION (object);
	switch (property_id) {
		case VALA_PARENTHESIZED_EXPRESSION_INNER:
		g_value_set_object (value, vala_parenthesized_expression_get_inner (self));
		break;
	}
}


void vala_parenthesized_expression_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaParenthesizedExpression * self = VALA_PARENTHESIZED_EXPRESSION (object);
	switch (property_id) {
		case VALA_PARENTHESIZED_EXPRESSION_INNER:
		vala_parenthesized_expression_set_inner (self, g_value_get_object (value));
		break;
	}
}


static void vala_parenthesized_expression_class_init (ValaParenthesizedExpressionClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaParenthesizedExpressionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_parenthesized_expression_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_parenthesized_expression_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_parenthesized_expression_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_parenthesized_expression_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PARENTHESIZED_EXPRESSION_INNER, g_param_spec_object ("inner", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_parenthesized_expression_init (ValaParenthesizedExpression * self)
{
	self->priv = VALA_PARENTHESIZED_EXPRESSION_GET_PRIVATE (self);
}


static void vala_parenthesized_expression_dispose (GObject * obj)
{
	ValaParenthesizedExpression * self = VALA_PARENTHESIZED_EXPRESSION (obj);
	(self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL)));
	ValaParenthesizedExpressionClass * klass;
	GObjectClass * parent_class;
	klass = VALA_PARENTHESIZED_EXPRESSION_CLASS (g_type_class_peek (VALA_TYPE_PARENTHESIZED_EXPRESSION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_parenthesized_expression_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaParenthesizedExpressionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_parenthesized_expression_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaParenthesizedExpression), 0, (GInstanceInitFunc) vala_parenthesized_expression_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_EXPRESSION, "ValaParenthesizedExpression", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



