/* valaparser.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaparser.h>

struct _ValaParserPrivate {
	char* comment;
	char* _file_comment;
};
#define VALA_PARSER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_PARSER, ValaParserPrivate))
enum  {
	VALA_PARSER_DUMMY_PROPERTY,
};
static void vala_parser_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file);
static void vala_parser_dispose (GObject * obj);

void vala_parser_parse (ValaParser* self, ValaCodeContext* context)
{
	g_return_if_fail (VALA_IS_PARSER (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	vala_code_context_accept (context, VALA_CODE_VISITOR (self));
}


static void vala_parser_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file)
{
	ValaParser * self = VALA_PARSER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (source_file));
	vala_parser_parse_file (self, source_file);
	char* __temp1 = NULL;
	char* __temp0 = NULL;
	source_file->comment = (__temp1 = (__temp0 = self->priv->_file_comment, (__temp0 == NULL ? __temp0 : g_strdup (__temp0))), (source_file->comment == NULL ? NULL : (source_file->comment = (g_free (source_file->comment), NULL))), __temp1);
}


void vala_parser_push_comment (ValaParser* self, char* comment_item, gboolean file_comment)
{
	g_return_if_fail (VALA_IS_PARSER (self));
	g_return_if_fail (comment_item != NULL);
	if (self->priv->comment == NULL) {
		char* __temp2 = NULL;
		self->priv->comment = (__temp2 = g_strdup (comment_item), (self->priv->comment == NULL ? NULL : (self->priv->comment = (g_free (self->priv->comment), NULL))), __temp2);
	} else {
		char* __temp3 = NULL;
		self->priv->comment = (__temp3 = g_strdup_printf ("%s\n%s", self->priv->comment, comment_item), (self->priv->comment == NULL ? NULL : (self->priv->comment = (g_free (self->priv->comment), NULL))), __temp3);
	}
	if (file_comment) {
		char* __temp5 = NULL;
		char* __temp4 = NULL;
		self->priv->_file_comment = (__temp5 = (__temp4 = self->priv->comment, (__temp4 == NULL ? __temp4 : g_strdup (__temp4))), (self->priv->_file_comment == NULL ? NULL : (self->priv->_file_comment = (g_free (self->priv->_file_comment), NULL))), __temp5);
		char* __temp6 = NULL;
		self->priv->comment = (__temp6 = NULL, (self->priv->comment == NULL ? NULL : (self->priv->comment = (g_free (self->priv->comment), NULL))), __temp6);
	}
}


char* vala_parser_pop_comment (ValaParser* self)
{
	g_return_val_if_fail (VALA_IS_PARSER (self), NULL);
	if (self->priv->comment == NULL) {
		return NULL;
	}
	GString* result = g_string_new (self->priv->comment);
	char* __temp8 = NULL;
	self->priv->comment = (__temp8 = NULL, (self->priv->comment == NULL ? NULL : (self->priv->comment = (g_free (self->priv->comment), NULL))), __temp8);
	char* index = NULL;
	while ((index = g_utf8_strchr (result->str, -1, '\t')) != NULL) {
		g_string_erase (result, g_utf8_pointer_to_offset (result->str, index), 1);
	}
	char* __temp9 = NULL;
	char* __temp10 = NULL;
	return (__temp10 = (__temp9 = result->str, (__temp9 == NULL ? __temp9 : g_strdup (__temp9))), (result == NULL ? NULL : (result = (g_string_free (result, TRUE), NULL))), __temp10);
	(result == NULL ? NULL : (result = (g_string_free (result, TRUE), NULL)));
}


void vala_parser_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaParser * self = VALA_PARSER (object);
	switch (property_id) {
	}
}


void vala_parser_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaParser * self = VALA_PARSER (object);
	switch (property_id) {
	}
}


static void vala_parser_class_init (ValaParserClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaParserPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_parser_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_parser_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_parser_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_source_file = vala_parser_real_visit_begin_source_file;
}


static void vala_parser_init (ValaParser * self)
{
	self->priv = VALA_PARSER_GET_PRIVATE (self);
}


static void vala_parser_dispose (GObject * obj)
{
	ValaParser * self = VALA_PARSER (obj);
	(self->priv->comment == NULL ? NULL : (self->priv->comment = (g_free (self->priv->comment), NULL)));
	(self->priv->_file_comment == NULL ? NULL : (self->priv->_file_comment = (g_free (self->priv->_file_comment), NULL)));
	ValaParserClass * klass;
	GObjectClass * parent_class;
	klass = VALA_PARSER_CLASS (g_type_class_peek (VALA_TYPE_PARSER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_parser_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaParserClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_parser_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaParser), 0, (GInstanceInitFunc) vala_parser_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaParser", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



