/* valaproperty.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#ifndef __VALAPROPERTY_H__
#define __VALAPROPERTY_H__

#include <glib.h>
#include <string.h>
#include <valaccodeconstant.h>
#include <valacodenode.h>
#include <valamemberaccessibility.h>

G_BEGIN_DECLS


#define VALA_TYPE_PROPERTY (vala_property_get_type ())
#define VALA_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_PROPERTY, ValaProperty))
#define VALA_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_PROPERTY, ValaPropertyClass))
#define VALA_IS_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_PROPERTY))
#define VALA_IS_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_PROPERTY))
#define VALA_PROPERTY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_PROPERTY, ValaPropertyClass))

typedef struct _ValaPropertyPrivate ValaPropertyPrivate;

struct _ValaProperty {
	ValaCodeNode parent;
	ValaPropertyPrivate * priv;
	ValaMemberAccessibility access;
	ValaFormalParameter* this_parameter;
	gboolean no_accessor_method;
};
struct _ValaPropertyClass {
	ValaCodeNodeClass parent;
};

ValaProperty* vala_property_new (char* name, ValaTypeReference* type, ValaPropertyAccessor* get_accessor, ValaPropertyAccessor* set_accessor, ValaSourceReference* source);
char* vala_property_get_upper_case_cname (ValaProperty* self);
ValaCCodeConstant* vala_property_get_canonical_cconstant (ValaProperty* self);
void vala_property_process_attributes (ValaProperty* self);
char* vala_property_get_name (ValaProperty* self);
void vala_property_set_name (ValaProperty* self, char* value);
ValaTypeReference* vala_property_get_type_reference (ValaProperty* self);
void vala_property_set_type_reference (ValaProperty* self, ValaTypeReference* value);
ValaPropertyAccessor* vala_property_get_get_accessor (ValaProperty* self);
void vala_property_set_get_accessor (ValaProperty* self, ValaPropertyAccessor* value);
ValaPropertyAccessor* vala_property_get_set_accessor (ValaProperty* self);
void vala_property_set_set_accessor (ValaProperty* self, ValaPropertyAccessor* value);
GType vala_property_get_type ();

G_END_DECLS

#endif
