/* valapropertyaccessor.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valapropertyaccessor.h>
#include <valastatement.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaPropertyAccessorPrivate {
	gboolean _readable;
	gboolean _writable;
	gboolean _construct_;
	ValaStatement* _body;
};
#define VALA_PROPERTY_ACCESSOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_PROPERTY_ACCESSOR, ValaPropertyAccessorPrivate))
enum  {
	VALA_PROPERTY_ACCESSOR_DUMMY_PROPERTY,
	VALA_PROPERTY_ACCESSOR_READABLE,
	VALA_PROPERTY_ACCESSOR_WRITABLE,
	VALA_PROPERTY_ACCESSOR_CONSTRUCT_,
	VALA_PROPERTY_ACCESSOR_BODY,
};
static void vala_property_accessor_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_property_accessor_dispose (GObject * obj);

ValaPropertyAccessor* vala_property_accessor_new (gboolean readable, gboolean writable, gboolean construct_, ValaStatement* body, ValaSourceReference* source)
{
	g_return_val_if_fail (body == NULL || VALA_IS_STATEMENT (body), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_PROPERTY_ACCESSOR, "readable", readable, "writable", writable, "construct_", construct_, "body", body, "source_reference", source, NULL));
}


static void vala_property_accessor_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaPropertyAccessor * self = VALA_PROPERTY_ACCESSOR (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_property_accessor (visitor, self);
	if (vala_property_accessor_get_body (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_property_accessor_get_body (self)), visitor);
	}
	vala_code_visitor_visit_end_property_accessor (visitor, self);
}


gboolean vala_property_accessor_get_readable (ValaPropertyAccessor* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY_ACCESSOR (self), FALSE);
	return self->priv->_readable;
}


void vala_property_accessor_set_readable (ValaPropertyAccessor* self, gboolean value)
{
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (self));
	self->priv->_readable = value;
}


gboolean vala_property_accessor_get_writable (ValaPropertyAccessor* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY_ACCESSOR (self), FALSE);
	return self->priv->_writable;
}


void vala_property_accessor_set_writable (ValaPropertyAccessor* self, gboolean value)
{
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (self));
	self->priv->_writable = value;
}


gboolean vala_property_accessor_get_construct_ (ValaPropertyAccessor* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY_ACCESSOR (self), FALSE);
	return self->priv->_construct_;
}


void vala_property_accessor_set_construct_ (ValaPropertyAccessor* self, gboolean value)
{
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (self));
	self->priv->_construct_ = value;
}


ValaStatement* vala_property_accessor_get_body (ValaPropertyAccessor* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY_ACCESSOR (self), NULL);
	return self->priv->_body;
}


void vala_property_accessor_set_body (ValaPropertyAccessor* self, ValaStatement* value)
{
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (self));
	ValaStatement* __temp6 = NULL;
	ValaStatement* __temp5 = NULL;
	self->priv->_body = (__temp6 = (__temp5 = value, (__temp5 == NULL ? __temp5 : g_object_ref (__temp5))), (self->priv->_body == NULL ? NULL : (self->priv->_body = (g_object_unref (self->priv->_body), NULL))), __temp6);
}


void vala_property_accessor_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaPropertyAccessor * self = VALA_PROPERTY_ACCESSOR (object);
	switch (property_id) {
		case VALA_PROPERTY_ACCESSOR_READABLE:
		g_value_set_boolean (value, vala_property_accessor_get_readable (self));
		break;
		case VALA_PROPERTY_ACCESSOR_WRITABLE:
		g_value_set_boolean (value, vala_property_accessor_get_writable (self));
		break;
		case VALA_PROPERTY_ACCESSOR_CONSTRUCT_:
		g_value_set_boolean (value, vala_property_accessor_get_construct_ (self));
		break;
		case VALA_PROPERTY_ACCESSOR_BODY:
		g_value_set_object (value, vala_property_accessor_get_body (self));
		break;
	}
}


void vala_property_accessor_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaPropertyAccessor * self = VALA_PROPERTY_ACCESSOR (object);
	switch (property_id) {
		case VALA_PROPERTY_ACCESSOR_READABLE:
		vala_property_accessor_set_readable (self, g_value_get_boolean (value));
		break;
		case VALA_PROPERTY_ACCESSOR_WRITABLE:
		vala_property_accessor_set_writable (self, g_value_get_boolean (value));
		break;
		case VALA_PROPERTY_ACCESSOR_CONSTRUCT_:
		vala_property_accessor_set_construct_ (self, g_value_get_boolean (value));
		break;
		case VALA_PROPERTY_ACCESSOR_BODY:
		vala_property_accessor_set_body (self, g_value_get_object (value));
		break;
	}
}


static void vala_property_accessor_class_init (ValaPropertyAccessorClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaPropertyAccessorPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_property_accessor_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_property_accessor_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_property_accessor_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_property_accessor_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_ACCESSOR_READABLE, g_param_spec_boolean ("readable", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_ACCESSOR_WRITABLE, g_param_spec_boolean ("writable", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_ACCESSOR_CONSTRUCT_, g_param_spec_boolean ("construct-", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_ACCESSOR_BODY, g_param_spec_object ("body", "foo", "bar", VALA_TYPE_STATEMENT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_property_accessor_init (ValaPropertyAccessor * self)
{
	self->priv = VALA_PROPERTY_ACCESSOR_GET_PRIVATE (self);
}


static void vala_property_accessor_dispose (GObject * obj)
{
	ValaPropertyAccessor * self = VALA_PROPERTY_ACCESSOR (obj);
	(self->priv->_body == NULL ? NULL : (self->priv->_body = (g_object_unref (self->priv->_body), NULL)));
	(self->value_parameter == NULL ? NULL : (self->value_parameter = (g_object_unref (self->value_parameter), NULL)));
	ValaPropertyAccessorClass * klass;
	GObjectClass * parent_class;
	klass = VALA_PROPERTY_ACCESSOR_CLASS (g_type_class_peek (VALA_TYPE_PROPERTY_ACCESSOR));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_property_accessor_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaPropertyAccessorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_property_accessor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaPropertyAccessor), 0, (GInstanceInitFunc) vala_property_accessor_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaPropertyAccessor", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



