/* valareturnstatement.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valareturnstatement.h>
#include <valaexpression.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaReturnStatementPrivate {
	ValaExpression* _return_expression;
};
#define VALA_RETURN_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_RETURN_STATEMENT, ValaReturnStatementPrivate))
enum  {
	VALA_RETURN_STATEMENT_DUMMY_PROPERTY,
	VALA_RETURN_STATEMENT_RETURN_EXPRESSION,
};
static void vala_return_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_return_statement_dispose (GObject * obj);

ValaReturnStatement* vala_return_statement_new (ValaExpression* result, ValaSourceReference* source)
{
	g_return_val_if_fail (result == NULL || VALA_IS_EXPRESSION (result), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_RETURN_STATEMENT, "return_expression", result, "source_reference", source, NULL));
}


static void vala_return_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaReturnStatement * self = VALA_RETURN_STATEMENT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	if (vala_return_statement_get_return_expression (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_return_statement_get_return_expression (self)), visitor);
		vala_code_visitor_visit_end_full_expression (visitor, vala_return_statement_get_return_expression (self));
	}
	vala_code_visitor_visit_return_statement (visitor, self);
}


ValaExpression* vala_return_statement_get_return_expression (ValaReturnStatement* self)
{
	g_return_val_if_fail (VALA_IS_RETURN_STATEMENT (self), NULL);
	return self->priv->_return_expression;
}


void vala_return_statement_set_return_expression (ValaReturnStatement* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (self));
	ValaExpression* __temp3 = NULL;
	ValaExpression* __temp2 = NULL;
	self->priv->_return_expression = (__temp3 = (__temp2 = value, (__temp2 == NULL ? __temp2 : g_object_ref (__temp2))), (self->priv->_return_expression == NULL ? NULL : (self->priv->_return_expression = (g_object_unref (self->priv->_return_expression), NULL))), __temp3);
}


void vala_return_statement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaReturnStatement * self = VALA_RETURN_STATEMENT (object);
	switch (property_id) {
		case VALA_RETURN_STATEMENT_RETURN_EXPRESSION:
		g_value_set_object (value, vala_return_statement_get_return_expression (self));
		break;
	}
}


void vala_return_statement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaReturnStatement * self = VALA_RETURN_STATEMENT (object);
	switch (property_id) {
		case VALA_RETURN_STATEMENT_RETURN_EXPRESSION:
		vala_return_statement_set_return_expression (self, g_value_get_object (value));
		break;
	}
}


static void vala_return_statement_class_init (ValaReturnStatementClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaReturnStatementPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_return_statement_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_return_statement_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_return_statement_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_return_statement_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_RETURN_STATEMENT_RETURN_EXPRESSION, g_param_spec_object ("return-expression", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_return_statement_init (ValaReturnStatement * self)
{
	self->priv = VALA_RETURN_STATEMENT_GET_PRIVATE (self);
}


static void vala_return_statement_dispose (GObject * obj)
{
	ValaReturnStatement * self = VALA_RETURN_STATEMENT (obj);
	(self->priv->_return_expression == NULL ? NULL : (self->priv->_return_expression = (g_object_unref (self->priv->_return_expression), NULL)));
	ValaReturnStatementClass * klass;
	GObjectClass * parent_class;
	klass = VALA_RETURN_STATEMENT_CLASS (g_type_class_peek (VALA_TYPE_RETURN_STATEMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_return_statement_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaReturnStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_return_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaReturnStatement), 0, (GInstanceInitFunc) vala_return_statement_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_STATEMENT, "ValaReturnStatement", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



