/* valasemanticanalyzer.vala
 *
 * Copyright (C) 2006  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <rasa@gmx.ch>
 */

#ifndef __VALASEMANTICANALYZER_H__
#define __VALASEMANTICANALYZER_H__

#include <glib.h>
#include <string.h>
#include <valacodevisitor.h>
#include <valacodecontext.h>
#include <valasourcefile.h>
#include <valanamespace.h>
#include <valaclass.h>
#include <valastruct.h>
#include <valafield.h>
#include <valamethod.h>
#include <valaformalparameter.h>
#include <valaproperty.h>
#include <valapropertyaccessor.h>
#include <valaconstructor.h>
#include <valadestructor.h>
#include <valanamedargument.h>
#include <valablock.h>
#include <valavariabledeclarator.h>
#include <valaexpressionstatement.h>
#include <valaifstatement.h>
#include <valawhilestatement.h>
#include <valaforstatement.h>
#include <valaforeachstatement.h>
#include <valareturnstatement.h>
#include <valabooleanliteral.h>
#include <valacharacterliteral.h>
#include <valaintegerliteral.h>
#include <valarealliteral.h>
#include <valastringliteral.h>
#include <valanullliteral.h>
#include <valaliteralexpression.h>
#include <valatypereference.h>
#include <valacodenode.h>
#include <valasymbol.h>
#include <valaparenthesizedexpression.h>
#include <valadatatype.h>
#include <valamemberaccess.h>
#include <valainvocationexpression.h>
#include <valaelementaccess.h>
#include <valapostfixexpression.h>
#include <valaobjectcreationexpression.h>
#include <valaunaryexpression.h>
#include <valacastexpression.h>
#include <valabinaryexpression.h>
#include <valatypecheck.h>
#include <valaconditionalexpression.h>
#include <valalambdaexpression.h>
#include <valaassignment.h>

G_BEGIN_DECLS


#define VALA_TYPE_SEMANTIC_ANALYZER (vala_semantic_analyzer_get_type ())
#define VALA_SEMANTIC_ANALYZER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzer))
#define VALA_SEMANTIC_ANALYZER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzerClass))
#define VALA_IS_SEMANTIC_ANALYZER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SEMANTIC_ANALYZER))
#define VALA_IS_SEMANTIC_ANALYZER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SEMANTIC_ANALYZER))
#define VALA_SEMANTIC_ANALYZER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzerClass))

typedef struct _ValaSemanticAnalyzer ValaSemanticAnalyzer;
typedef struct _ValaSemanticAnalyzerClass ValaSemanticAnalyzerClass;
typedef struct _ValaSemanticAnalyzerPrivate ValaSemanticAnalyzerPrivate;

/**
 * Code visitor analyzing and checking code.
 */
struct _ValaSemanticAnalyzer {
	ValaCodeVisitor parent;
	ValaSemanticAnalyzerPrivate * priv;
};
struct _ValaSemanticAnalyzerClass {
	ValaCodeVisitorClass parent;
};

void vala_semantic_analyzer_analyze (ValaSemanticAnalyzer* self, ValaCodeContext* context);
gboolean vala_semantic_analyzer_get_memory_management (ValaSemanticAnalyzer* self);
void vala_semantic_analyzer_set_memory_management (ValaSemanticAnalyzer* self, gboolean value);
GType vala_semantic_analyzer_get_type ();

G_END_DECLS

#endif
