/* valasourcereference.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valasourcereference.h>
#include <valasourcefile.h>

struct _ValaSourceReferencePrivate {
	ValaSourceFile* _file;
	int _first_line;
	int _first_column;
	int _last_line;
	int _last_column;
	char* _comment;
};
#define VALA_SOURCE_REFERENCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferencePrivate))
enum  {
	VALA_SOURCE_REFERENCE_DUMMY_PROPERTY,
	VALA_SOURCE_REFERENCE_FILE,
	VALA_SOURCE_REFERENCE_FIRST_LINE,
	VALA_SOURCE_REFERENCE_FIRST_COLUMN,
	VALA_SOURCE_REFERENCE_LAST_LINE,
	VALA_SOURCE_REFERENCE_LAST_COLUMN,
	VALA_SOURCE_REFERENCE_COMMENT,
};
static void vala_source_reference_dispose (GObject * obj);

ValaSourceReference* vala_source_reference_new (ValaSourceFile* file, int first_line, int first_column, int last_line, int last_column)
{
	g_return_val_if_fail (file == NULL || VALA_IS_SOURCE_FILE (file), NULL);
	return (g_object_new (VALA_TYPE_SOURCE_REFERENCE, "file", file, "first_line", first_line, "first_column", first_column, "last_line", last_line, "last_column", last_column, NULL));
}


ValaSourceReference* vala_source_reference_new_with_comment (ValaSourceFile* file, int first_line, int first_column, int last_line, int last_column, char* comment)
{
	g_return_val_if_fail (file == NULL || VALA_IS_SOURCE_FILE (file), NULL);
	return (g_object_new (VALA_TYPE_SOURCE_REFERENCE, "file", file, "first_line", first_line, "first_column", first_column, "last_line", last_line, "last_column", last_column, "comment", comment, NULL));
}


char* vala_source_reference_to_string (ValaSourceReference* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_REFERENCE (self), NULL);
	return (g_strdup_printf ("%s:%d.%d-%d.%d", vala_source_file_get_filename (vala_source_reference_get_file (self)), vala_source_reference_get_first_line (self), vala_source_reference_get_first_column (self), vala_source_reference_get_last_line (self), vala_source_reference_get_last_column (self)));
}


ValaSourceFile* vala_source_reference_get_file (ValaSourceReference* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_REFERENCE (self), NULL);
	return self->priv->_file;
}


void vala_source_reference_set_file (ValaSourceReference* self, ValaSourceFile* value)
{
	g_return_if_fail (VALA_IS_SOURCE_REFERENCE (self));
	self->priv->_file = value;
}


int vala_source_reference_get_first_line (ValaSourceReference* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_REFERENCE (self), 0);
	return self->priv->_first_line;
}


void vala_source_reference_set_first_line (ValaSourceReference* self, int value)
{
	g_return_if_fail (VALA_IS_SOURCE_REFERENCE (self));
	self->priv->_first_line = value;
}


int vala_source_reference_get_first_column (ValaSourceReference* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_REFERENCE (self), 0);
	return self->priv->_first_column;
}


void vala_source_reference_set_first_column (ValaSourceReference* self, int value)
{
	g_return_if_fail (VALA_IS_SOURCE_REFERENCE (self));
	self->priv->_first_column = value;
}


int vala_source_reference_get_last_line (ValaSourceReference* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_REFERENCE (self), 0);
	return self->priv->_last_line;
}


void vala_source_reference_set_last_line (ValaSourceReference* self, int value)
{
	g_return_if_fail (VALA_IS_SOURCE_REFERENCE (self));
	self->priv->_last_line = value;
}


int vala_source_reference_get_last_column (ValaSourceReference* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_REFERENCE (self), 0);
	return self->priv->_last_column;
}


void vala_source_reference_set_last_column (ValaSourceReference* self, int value)
{
	g_return_if_fail (VALA_IS_SOURCE_REFERENCE (self));
	self->priv->_last_column = value;
}


char* vala_source_reference_get_comment (ValaSourceReference* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_REFERENCE (self), NULL);
	return self->priv->_comment;
}


void vala_source_reference_set_comment (ValaSourceReference* self, char* value)
{
	g_return_if_fail (VALA_IS_SOURCE_REFERENCE (self));
	char* __temp10 = NULL;
	char* __temp9 = NULL;
	self->priv->_comment = (__temp10 = (__temp9 = value, (__temp9 == NULL ? __temp9 : g_strdup (__temp9))), (self->priv->_comment == NULL ? NULL : (self->priv->_comment = (g_free (self->priv->_comment), NULL))), __temp10);
}


void vala_source_reference_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaSourceReference * self = VALA_SOURCE_REFERENCE (object);
	switch (property_id) {
		case VALA_SOURCE_REFERENCE_FILE:
		g_value_set_object (value, vala_source_reference_get_file (self));
		break;
		case VALA_SOURCE_REFERENCE_FIRST_LINE:
		g_value_set_int (value, vala_source_reference_get_first_line (self));
		break;
		case VALA_SOURCE_REFERENCE_FIRST_COLUMN:
		g_value_set_int (value, vala_source_reference_get_first_column (self));
		break;
		case VALA_SOURCE_REFERENCE_LAST_LINE:
		g_value_set_int (value, vala_source_reference_get_last_line (self));
		break;
		case VALA_SOURCE_REFERENCE_LAST_COLUMN:
		g_value_set_int (value, vala_source_reference_get_last_column (self));
		break;
		case VALA_SOURCE_REFERENCE_COMMENT:
		g_value_set_string (value, vala_source_reference_get_comment (self));
		break;
	}
}


void vala_source_reference_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaSourceReference * self = VALA_SOURCE_REFERENCE (object);
	switch (property_id) {
		case VALA_SOURCE_REFERENCE_FILE:
		vala_source_reference_set_file (self, g_value_get_object (value));
		break;
		case VALA_SOURCE_REFERENCE_FIRST_LINE:
		vala_source_reference_set_first_line (self, g_value_get_int (value));
		break;
		case VALA_SOURCE_REFERENCE_FIRST_COLUMN:
		vala_source_reference_set_first_column (self, g_value_get_int (value));
		break;
		case VALA_SOURCE_REFERENCE_LAST_LINE:
		vala_source_reference_set_last_line (self, g_value_get_int (value));
		break;
		case VALA_SOURCE_REFERENCE_LAST_COLUMN:
		vala_source_reference_set_last_column (self, g_value_get_int (value));
		break;
		case VALA_SOURCE_REFERENCE_COMMENT:
		vala_source_reference_set_comment (self, g_value_get_string (value));
		break;
	}
}


static void vala_source_reference_class_init (ValaSourceReferenceClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaSourceReferencePrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_source_reference_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_source_reference_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_source_reference_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SOURCE_REFERENCE_FILE, g_param_spec_object ("file", "foo", "bar", VALA_TYPE_SOURCE_FILE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SOURCE_REFERENCE_FIRST_LINE, g_param_spec_int ("first-line", "foo", "bar", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SOURCE_REFERENCE_FIRST_COLUMN, g_param_spec_int ("first-column", "foo", "bar", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SOURCE_REFERENCE_LAST_LINE, g_param_spec_int ("last-line", "foo", "bar", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SOURCE_REFERENCE_LAST_COLUMN, g_param_spec_int ("last-column", "foo", "bar", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SOURCE_REFERENCE_COMMENT, g_param_spec_string ("comment", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_source_reference_init (ValaSourceReference * self)
{
	self->priv = VALA_SOURCE_REFERENCE_GET_PRIVATE (self);
}


static void vala_source_reference_dispose (GObject * obj)
{
	ValaSourceReference * self = VALA_SOURCE_REFERENCE (obj);
	(self->priv->_comment == NULL ? NULL : (self->priv->_comment = (g_free (self->priv->_comment), NULL)));
	ValaSourceReferenceClass * klass;
	GObjectClass * parent_class;
	klass = VALA_SOURCE_REFERENCE_CLASS (g_type_class_peek (VALA_TYPE_SOURCE_REFERENCE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_source_reference_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaSourceReferenceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_source_reference_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaSourceReference), 0, (GInstanceInitFunc) vala_source_reference_init };
		g_define_type_id = g_type_register_static (G_TYPE_OBJECT, "ValaSourceReference", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



