/* valastringliteral.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valastringliteral.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaStringLiteralPrivate {
	char* _value;
};
#define VALA_STRING_LITERAL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_STRING_LITERAL, ValaStringLiteralPrivate))
enum  {
	VALA_STRING_LITERAL_DUMMY_PROPERTY,
	VALA_STRING_LITERAL_VALUE,
};
static void vala_string_literal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_string_literal_dispose (GObject * obj);

ValaStringLiteral* vala_string_literal_new (char* s, ValaSourceReference* source)
{
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_STRING_LITERAL, "value", s, "source_reference", source, NULL));
}


char* vala_string_literal_eval (ValaStringLiteral* self)
{
	g_return_val_if_fail (VALA_IS_STRING_LITERAL (self), NULL);
	if (vala_string_literal_get_value (self) == NULL) {
		return NULL;
	}
	/* remove quotes */
	char* noquotes = g_strndup (g_utf8_offset_to_pointer (vala_string_literal_get_value (self), 1), (g_utf8_strlen (vala_string_literal_get_value (self), -1) - 2));
	char* __temp2 = NULL;
	return (__temp2 = g_strcompress (noquotes), (noquotes == NULL ? NULL : (noquotes = (g_free (noquotes), NULL))), __temp2);
	(noquotes == NULL ? NULL : (noquotes = (g_free (noquotes), NULL)));
}


/* unescape string */
static void vala_string_literal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaStringLiteral * self = VALA_STRING_LITERAL (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_string_literal (visitor, self);
}


char* vala_string_literal_get_value (ValaStringLiteral* self)
{
	g_return_val_if_fail (VALA_IS_STRING_LITERAL (self), NULL);
	return self->priv->_value;
}


void vala_string_literal_set_value (ValaStringLiteral* self, char* value)
{
	g_return_if_fail (VALA_IS_STRING_LITERAL (self));
	char* __temp5 = NULL;
	char* __temp4 = NULL;
	self->priv->_value = (__temp5 = (__temp4 = value, (__temp4 == NULL ? __temp4 : g_strdup (__temp4))), (self->priv->_value == NULL ? NULL : (self->priv->_value = (g_free (self->priv->_value), NULL))), __temp5);
}


void vala_string_literal_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaStringLiteral * self = VALA_STRING_LITERAL (object);
	switch (property_id) {
		case VALA_STRING_LITERAL_VALUE:
		g_value_set_string (value, vala_string_literal_get_value (self));
		break;
	}
}


void vala_string_literal_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaStringLiteral * self = VALA_STRING_LITERAL (object);
	switch (property_id) {
		case VALA_STRING_LITERAL_VALUE:
		vala_string_literal_set_value (self, g_value_get_string (value));
		break;
	}
}


static void vala_string_literal_class_init (ValaStringLiteralClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaStringLiteralPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_string_literal_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_string_literal_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_string_literal_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_string_literal_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_STRING_LITERAL_VALUE, g_param_spec_string ("value", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_string_literal_init (ValaStringLiteral * self)
{
	self->priv = VALA_STRING_LITERAL_GET_PRIVATE (self);
}


static void vala_string_literal_dispose (GObject * obj)
{
	ValaStringLiteral * self = VALA_STRING_LITERAL (obj);
	(self->priv->_value == NULL ? NULL : (self->priv->_value = (g_free (self->priv->_value), NULL)));
	ValaStringLiteralClass * klass;
	GObjectClass * parent_class;
	klass = VALA_STRING_LITERAL_CLASS (g_type_class_peek (VALA_TYPE_STRING_LITERAL));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_string_literal_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaStringLiteralClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_string_literal_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaStringLiteral), 0, (GInstanceInitFunc) vala_string_literal_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_LITERAL, "ValaStringLiteral", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



