/* valasymbol.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valasymbol.h>

struct _ValaSymbolPrivate {
	GHashTable* symbol_table;
	ValaCodeNode* _node;
};
#define VALA_SYMBOL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SYMBOL, ValaSymbolPrivate))
enum  {
	VALA_SYMBOL_DUMMY_PROPERTY,
	VALA_SYMBOL_NODE,
};
static void vala_symbol_dispose (GObject * obj);

char* vala_symbol_get_full_name (ValaSymbol* self)
{
	g_return_val_if_fail (VALA_IS_SYMBOL (self), NULL);
	if (self->parent_symbol == NULL) {
		char* __temp0 = NULL;
		return (__temp0 = self->name, (__temp0 == NULL ? __temp0 : g_strdup (__temp0)));
	}
	if (self->name == NULL) {
		return vala_symbol_get_full_name (self->parent_symbol);
	}
	char* __temp3 = NULL;
	gboolean __temp4;
	if ((__temp4 = (__temp3 = vala_symbol_get_full_name (self->parent_symbol)) == NULL, (__temp3 == NULL ? NULL : (__temp3 = (g_free (__temp3), NULL))), __temp4)) {
		char* __temp5 = NULL;
		return (__temp5 = self->name, (__temp5 == NULL ? __temp5 : g_strdup (__temp5)));
	}
	char* __temp7 = NULL;
	char* __temp8 = NULL;
	return (__temp8 = g_strdup_printf ("%s.%s", (__temp7 = vala_symbol_get_full_name (self->parent_symbol)), self->name), (__temp7 == NULL ? NULL : (__temp7 = (g_free (__temp7), NULL))), __temp8);
}


void vala_symbol_add (ValaSymbol* self, char* s, ValaSymbol* sym)
{
	g_return_if_fail (VALA_IS_SYMBOL (self));
	g_return_if_fail (sym == NULL || VALA_IS_SYMBOL (sym));
	ValaSymbol* __temp11 = NULL;
	char* __temp10 = NULL;
	g_hash_table_insert (self->priv->symbol_table, (__temp10 = s, (__temp10 == NULL ? __temp10 : g_strdup (__temp10))), (__temp11 = sym, (__temp11 == NULL ? __temp11 : g_object_ref (__temp11))));
	sym->parent_symbol = self;
	char* __temp13 = NULL;
	char* __temp12 = NULL;
	sym->name = (__temp13 = (__temp12 = s, (__temp12 == NULL ? __temp12 : g_strdup (__temp12))), (sym->name == NULL ? NULL : (sym->name = (g_free (sym->name), NULL))), __temp13);
}


ValaSymbol* vala_symbol_lookup (ValaSymbol* self, char* s)
{
	g_return_val_if_fail (VALA_IS_SYMBOL (self), NULL);
	ValaSymbol* sym = g_hash_table_lookup (self->priv->symbol_table, s);
	if (sym != NULL && !sym->active) {
		sym = NULL;
	}
	return sym;
}


ValaCodeNode* vala_symbol_get_node (ValaSymbol* self)
{
	g_return_val_if_fail (VALA_IS_SYMBOL (self), NULL);
	return self->priv->_node;
}


void vala_symbol_set_node (ValaSymbol* self, ValaCodeNode* value)
{
	g_return_if_fail (VALA_IS_SYMBOL (self));
	self->priv->_node = value;
}


void vala_symbol_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaSymbol * self = VALA_SYMBOL (object);
	switch (property_id) {
		case VALA_SYMBOL_NODE:
		g_value_set_object (value, vala_symbol_get_node (self));
		break;
	}
}


void vala_symbol_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaSymbol * self = VALA_SYMBOL (object);
	switch (property_id) {
		case VALA_SYMBOL_NODE:
		vala_symbol_set_node (self, g_value_get_object (value));
		break;
	}
}


static void vala_symbol_class_init (ValaSymbolClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaSymbolPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_symbol_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_symbol_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_symbol_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SYMBOL_NODE, g_param_spec_object ("node", "foo", "bar", VALA_TYPE_CODE_NODE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_symbol_init (ValaSymbol * self)
{
	self->priv = VALA_SYMBOL_GET_PRIVATE (self);
	self->priv->symbol_table = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
	self->active = TRUE;
}


static void vala_symbol_dispose (GObject * obj)
{
	ValaSymbol * self = VALA_SYMBOL (obj);
	(self->priv->symbol_table == NULL ? NULL : (self->priv->symbol_table = (g_hash_table_unref (self->priv->symbol_table), NULL)));
	(self->name == NULL ? NULL : (self->name = (g_free (self->name), NULL)));
	ValaSymbolClass * klass;
	GObjectClass * parent_class;
	klass = VALA_SYMBOL_CLASS (g_type_class_peek (VALA_TYPE_SYMBOL));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_symbol_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaSymbolClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_symbol_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaSymbol), 0, (GInstanceInitFunc) vala_symbol_init };
		g_define_type_id = g_type_register_static (G_TYPE_OBJECT, "ValaSymbol", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



