/* valatypeparameter.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valatypeparameter.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaTypeParameterPrivate {
	char* _name;
};
#define VALA_TYPE_PARAMETER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_TYPE_PARAMETER, ValaTypeParameterPrivate))
enum  {
	VALA_TYPE_PARAMETER_DUMMY_PROPERTY,
	VALA_TYPE_PARAMETER_NAME,
};
static void vala_type_parameter_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_type_parameter_dispose (GObject * obj);

ValaTypeParameter* vala_type_parameter_new (char* name, ValaSourceReference* source)
{
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_TYPE_PARAMETER, "name", name, "source_reference", source, NULL));
}


static void vala_type_parameter_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaTypeParameter * self = VALA_TYPE_PARAMETER (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_type_parameter (visitor, self);
}


char* vala_type_parameter_get_name (ValaTypeParameter* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_PARAMETER (self), NULL);
	return self->priv->_name;
}


void vala_type_parameter_set_name (ValaTypeParameter* self, char* value)
{
	g_return_if_fail (VALA_IS_TYPE_PARAMETER (self));
	char* __temp3 = NULL;
	char* __temp2 = NULL;
	self->priv->_name = (__temp3 = (__temp2 = value, (__temp2 == NULL ? __temp2 : g_strdup (__temp2))), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp3);
}


void vala_type_parameter_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaTypeParameter * self = VALA_TYPE_PARAMETER (object);
	switch (property_id) {
		case VALA_TYPE_PARAMETER_NAME:
		g_value_set_string (value, vala_type_parameter_get_name (self));
		break;
	}
}


void vala_type_parameter_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaTypeParameter * self = VALA_TYPE_PARAMETER (object);
	switch (property_id) {
		case VALA_TYPE_PARAMETER_NAME:
		vala_type_parameter_set_name (self, g_value_get_string (value));
		break;
	}
}


static void vala_type_parameter_class_init (ValaTypeParameterClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaTypeParameterPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_type_parameter_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_type_parameter_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_type_parameter_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_type_parameter_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_TYPE_PARAMETER_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_type_parameter_init (ValaTypeParameter * self)
{
	self->priv = VALA_TYPE_PARAMETER_GET_PRIVATE (self);
}


static void vala_type_parameter_dispose (GObject * obj)
{
	ValaTypeParameter * self = VALA_TYPE_PARAMETER (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	ValaTypeParameterClass * klass;
	GObjectClass * parent_class;
	klass = VALA_TYPE_PARAMETER_CLASS (g_type_class_peek (VALA_TYPE_TYPE_PARAMETER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_type_parameter_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaTypeParameterClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_type_parameter_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaTypeParameter), 0, (GInstanceInitFunc) vala_type_parameter_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaTypeParameter", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



