/* valatyperegisterfunction.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valatyperegisterfunction.h>
#include <valaccodeblock.h>
#include <valaccodedeclaration.h>
#include <valaccodeconstant.h>
#include <valaccodevariabledeclarator.h>
#include <valaccodedeclarator.h>
#include <valaccodemodifiers.h>
#include <valaccodenode.h>
#include <valaccodeidentifier.h>
#include <valaccodefunctioncall.h>
#include <valaccodebinaryexpression.h>
#include <valaccodeexpression.h>
#include <valaccodeassignment.h>
#include <valaccodeexpressionstatement.h>
#include <valaccodeifstatement.h>
#include <valaccodereturnstatement.h>

struct _ValaTypeRegisterFunctionPrivate {
};
#define VALA_TYPE_REGISTER_FUNCTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_TYPE_REGISTER_FUNCTION, ValaTypeRegisterFunctionPrivate))
enum  {
	VALA_TYPE_REGISTER_FUNCTION_DUMMY_PROPERTY,
};
static char* vala_type_register_function_real_get_type_flags (ValaTypeRegisterFunction* self);
static void vala_type_register_function_dispose (GObject * obj);

/**
 * Constructs the C function from the specified type.
 */
void vala_type_register_function_init_from_type (ValaTypeRegisterFunction* self)
{
	g_return_if_fail (VALA_IS_TYPE_REGISTER_FUNCTION (self));
	char* __temp1 = NULL;
	char* __temp0 = NULL;
	vala_ccode_function_set_name (VALA_CCODE_FUNCTION (self), (__temp1 = g_strdup_printf ("%s_get_type", (__temp0 = vala_data_type_get_lower_case_cname (vala_type_register_function_get_type_declaration (self), NULL)))));
	(__temp1 == NULL ? NULL : (__temp1 = (g_free (__temp1), NULL)));
	(__temp0 == NULL ? NULL : (__temp0 = (g_free (__temp0), NULL)));
	vala_ccode_function_set_return_type (VALA_CCODE_FUNCTION (self), "GType");
	ValaCCodeBlock* type_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeDeclaration* cdecl = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", "GType", NULL);
	ValaCCodeVariableDeclarator* __temp3 = NULL;
	ValaCCodeConstant* __temp2 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp3 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", "g_define_type_id", "initializer", (__temp2 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "0", NULL)), NULL))));
	(__temp3 == NULL ? NULL : (__temp3 = (g_object_unref (__temp3), NULL)));
	(__temp2 == NULL ? NULL : (__temp2 = (g_object_unref (__temp2), NULL)));
	vala_ccode_declaration_set_modifiers (cdecl, VALA_CCODE_MODIFIERS_STATIC);
	vala_ccode_block_add_statement (type_block, VALA_CCODE_NODE (cdecl));
	ValaCCodeIdentifier* __temp4 = NULL;
	ValaCCodeFunctionCall* __temp5 = NULL;
	ValaCCodeFunctionCall* cond = (__temp5 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp4 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "G_UNLIKELY", NULL)), NULL), (__temp4 == NULL ? NULL : (__temp4 = (g_object_unref (__temp4), NULL))), __temp5);
	ValaCCodeBinaryExpression* __temp8 = NULL;
	ValaCCodeConstant* __temp7 = NULL;
	ValaCCodeIdentifier* __temp6 = NULL;
	vala_ccode_function_call_add_argument (cond, VALA_CCODE_EXPRESSION ((__temp8 = g_object_new (VALA_TYPE_CCODE_BINARY_EXPRESSION, "operator", VALA_CCODE_BINARY_OPERATOR_EQUALITY, "left", (__temp6 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_define_type_id", NULL)), "right", (__temp7 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "0", NULL)), NULL))));
	(__temp8 == NULL ? NULL : (__temp8 = (g_object_unref (__temp8), NULL)));
	(__temp7 == NULL ? NULL : (__temp7 = (g_object_unref (__temp7), NULL)));
	(__temp6 == NULL ? NULL : (__temp6 = (g_object_unref (__temp6), NULL)));
	ValaCCodeBlock* type_init = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeDeclaration* ctypedecl = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", "const GTypeInfo", NULL);
	vala_ccode_declaration_set_modifiers (ctypedecl, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeVariableDeclarator* __temp15 = NULL;
	ValaCCodeConstant* __temp14 = NULL;
	char* __temp13 = NULL;
	char* __temp12 = NULL;
	char* __temp11 = NULL;
	char* __temp10 = NULL;
	char* __temp9 = NULL;
	vala_ccode_declaration_add_declarator (ctypedecl, VALA_CCODE_DECLARATOR ((__temp15 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", "g_define_type_info", "initializer", (__temp14 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", (__temp13 = g_strdup_printf ("{ sizeof (%s), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) %s, (GClassFinalizeFunc) NULL, NULL, %s, 0, (GInstanceInitFunc) %s }", (__temp9 = vala_type_register_function_get_type_struct_name (self)), (__temp10 = vala_type_register_function_get_class_init_func_name (self)), (__temp11 = vala_type_register_function_get_instance_struct_size (self)), (__temp12 = vala_type_register_function_get_instance_init_func_name (self)))), NULL)), NULL))));
	(__temp15 == NULL ? NULL : (__temp15 = (g_object_unref (__temp15), NULL)));
	(__temp14 == NULL ? NULL : (__temp14 = (g_object_unref (__temp14), NULL)));
	(__temp13 == NULL ? NULL : (__temp13 = (g_free (__temp13), NULL)));
	(__temp12 == NULL ? NULL : (__temp12 = (g_free (__temp12), NULL)));
	(__temp11 == NULL ? NULL : (__temp11 = (g_free (__temp11), NULL)));
	(__temp10 == NULL ? NULL : (__temp10 = (g_free (__temp10), NULL)));
	(__temp9 == NULL ? NULL : (__temp9 = (g_free (__temp9), NULL)));
	vala_ccode_block_add_statement (type_init, VALA_CCODE_NODE (ctypedecl));
	ValaCCodeIdentifier* __temp16 = NULL;
	ValaCCodeFunctionCall* __temp17 = NULL;
	ValaCCodeFunctionCall* reg_call = (__temp17 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp16 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_type_register_static", NULL)), NULL), (__temp16 == NULL ? NULL : (__temp16 = (g_object_unref (__temp16), NULL))), __temp17);
	ValaCCodeIdentifier* __temp19 = NULL;
	char* __temp18 = NULL;
	vala_ccode_function_call_add_argument (reg_call, VALA_CCODE_EXPRESSION ((__temp19 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp18 = vala_type_register_function_get_parent_type_name (self)), NULL))));
	(__temp19 == NULL ? NULL : (__temp19 = (g_object_unref (__temp19), NULL)));
	(__temp18 == NULL ? NULL : (__temp18 = (g_free (__temp18), NULL)));
	ValaCCodeConstant* __temp21 = NULL;
	char* __temp20 = NULL;
	vala_ccode_function_call_add_argument (reg_call, VALA_CCODE_EXPRESSION ((__temp21 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", (__temp20 = g_strdup_printf ("\"%s\"", vala_data_type_get_cname (vala_type_register_function_get_type_declaration (self)))), NULL))));
	(__temp21 == NULL ? NULL : (__temp21 = (g_object_unref (__temp21), NULL)));
	(__temp20 == NULL ? NULL : (__temp20 = (g_free (__temp20), NULL)));
	ValaCCodeIdentifier* __temp22 = NULL;
	vala_ccode_function_call_add_argument (reg_call, VALA_CCODE_EXPRESSION ((__temp22 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "&g_define_type_info", NULL))));
	(__temp22 == NULL ? NULL : (__temp22 = (g_object_unref (__temp22), NULL)));
	ValaCCodeConstant* __temp23 = NULL;
	vala_ccode_function_call_add_argument (reg_call, VALA_CCODE_EXPRESSION ((__temp23 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", vala_type_register_function_get_type_flags (self), NULL))));
	(__temp23 == NULL ? NULL : (__temp23 = (g_object_unref (__temp23), NULL)));
	ValaCCodeExpressionStatement* __temp26 = NULL;
	ValaCCodeAssignment* __temp25 = NULL;
	ValaCCodeIdentifier* __temp24 = NULL;
	vala_ccode_block_add_statement (type_init, VALA_CCODE_NODE ((__temp26 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp25 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp24 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_define_type_id", NULL)), "right", reg_call, NULL)), NULL))));
	(__temp26 == NULL ? NULL : (__temp26 = (g_object_unref (__temp26), NULL)));
	(__temp25 == NULL ? NULL : (__temp25 = (g_object_unref (__temp25), NULL)));
	(__temp24 == NULL ? NULL : (__temp24 = (g_object_unref (__temp24), NULL)));
	ValaCCodeFragment* __temp27 = NULL;
	vala_ccode_block_add_statement (type_init, VALA_CCODE_NODE ((__temp27 = vala_type_register_function_get_type_interface_init_statements (self))));
	(__temp27 == NULL ? NULL : (__temp27 = (g_object_unref (__temp27), NULL)));
	ValaCCodeIfStatement* cif = g_object_new (VALA_TYPE_CCODE_IF_STATEMENT, "condition", cond, "true_statement", type_init, NULL);
	vala_ccode_block_add_statement (type_block, VALA_CCODE_NODE (cif));
	ValaCCodeReturnStatement* __temp29 = NULL;
	ValaCCodeIdentifier* __temp28 = NULL;
	vala_ccode_block_add_statement (type_block, VALA_CCODE_NODE ((__temp29 = g_object_new (VALA_TYPE_CCODE_RETURN_STATEMENT, "return_expression", (__temp28 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_define_type_id", NULL)), NULL))));
	(__temp29 == NULL ? NULL : (__temp29 = (g_object_unref (__temp29), NULL)));
	(__temp28 == NULL ? NULL : (__temp28 = (g_object_unref (__temp28), NULL)));
	vala_ccode_function_set_block (VALA_CCODE_FUNCTION (self), type_block);
	(type_block == NULL ? NULL : (type_block = (g_object_unref (type_block), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(cond == NULL ? NULL : (cond = (g_object_unref (cond), NULL)));
	(type_init == NULL ? NULL : (type_init = (g_object_unref (type_init), NULL)));
	(ctypedecl == NULL ? NULL : (ctypedecl = (g_object_unref (ctypedecl), NULL)));
	(reg_call == NULL ? NULL : (reg_call = (g_object_unref (reg_call), NULL)));
	(cif == NULL ? NULL : (cif = (g_object_unref (cif), NULL)));
}


ValaDataType* vala_type_register_function_get_type_declaration (ValaTypeRegisterFunction* self)
{
	VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_type_declaration (self);
}


char* vala_type_register_function_get_type_struct_name (ValaTypeRegisterFunction* self)
{
	VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_type_struct_name (self);
}


char* vala_type_register_function_get_class_init_func_name (ValaTypeRegisterFunction* self)
{
	VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_class_init_func_name (self);
}


char* vala_type_register_function_get_instance_struct_size (ValaTypeRegisterFunction* self)
{
	VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_instance_struct_size (self);
}


char* vala_type_register_function_get_instance_init_func_name (ValaTypeRegisterFunction* self)
{
	VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_instance_init_func_name (self);
}


char* vala_type_register_function_get_parent_type_name (ValaTypeRegisterFunction* self)
{
	VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_parent_type_name (self);
}


/**
 * Returns the set of type flags to be applied when registering.
 *
 * @return type flags
 */
static char* vala_type_register_function_real_get_type_flags (ValaTypeRegisterFunction* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REGISTER_FUNCTION (self), NULL);
	return "0";
}


char* vala_type_register_function_get_type_flags (ValaTypeRegisterFunction* self)
{
	VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_type_flags (self);
}


ValaCCodeFragment* vala_type_register_function_get_type_interface_init_statements (ValaTypeRegisterFunction* self)
{
	VALA_TYPE_REGISTER_FUNCTION_GET_CLASS (self)->get_type_interface_init_statements (self);
}


/**
 * Returns the declaration for this type register function in C code.
 *
 * @return C function declaration
 */
ValaCCodeFunction* vala_type_register_function_get_declaration (ValaTypeRegisterFunction* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REGISTER_FUNCTION (self), NULL);
	return g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", vala_ccode_function_get_name (VALA_CCODE_FUNCTION (self)), "return_type", vala_ccode_function_get_return_type (VALA_CCODE_FUNCTION (self)), NULL);
}


void vala_type_register_function_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaTypeRegisterFunction * self = VALA_TYPE_REGISTER_FUNCTION (object);
	switch (property_id) {
	}
}


void vala_type_register_function_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaTypeRegisterFunction * self = VALA_TYPE_REGISTER_FUNCTION (object);
	switch (property_id) {
	}
}


static void vala_type_register_function_class_init (ValaTypeRegisterFunctionClass * klass)
{
	G_OBJECT_CLASS (klass)->get_property = vala_type_register_function_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_type_register_function_set_property;
	VALA_TYPE_REGISTER_FUNCTION_CLASS (klass)->get_type_flags = vala_type_register_function_real_get_type_flags;
}


static void vala_type_register_function_init (ValaTypeRegisterFunction * self)
{
}


static void vala_type_register_function_dispose (GObject * obj)
{
	ValaTypeRegisterFunction * self = VALA_TYPE_REGISTER_FUNCTION (obj);
	ValaTypeRegisterFunctionClass * klass;
	GObjectClass * parent_class;
	klass = VALA_TYPE_REGISTER_FUNCTION_CLASS (g_type_class_peek (VALA_TYPE_TYPE_REGISTER_FUNCTION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_type_register_function_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaTypeRegisterFunctionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_type_register_function_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaTypeRegisterFunction), 0, (GInstanceInitFunc) vala_type_register_function_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CCODE_FUNCTION, "ValaTypeRegisterFunction", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return g_define_type_id;
}



