/* valaunaryexpression.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaunaryexpression.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaUnaryExpressionPrivate {
	ValaUnaryOperator _operator;
	ValaExpression* _inner;
};
#define VALA_UNARY_EXPRESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_UNARY_EXPRESSION, ValaUnaryExpressionPrivate))
enum  {
	VALA_UNARY_EXPRESSION_DUMMY_PROPERTY,
	VALA_UNARY_EXPRESSION_OPERATOR,
	VALA_UNARY_EXPRESSION_INNER,
};
static void vala_unary_expression_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_unary_expression_dispose (GObject * obj);

ValaUnaryExpression* vala_unary_expression_new (ValaUnaryOperator op, ValaExpression* inner, ValaSourceReference* source)
{
	g_return_val_if_fail (inner == NULL || VALA_IS_EXPRESSION (inner), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_UNARY_EXPRESSION, "operator", op, "inner", inner, "source_reference", source, NULL));
}


static void vala_unary_expression_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaUnaryExpression * self = VALA_UNARY_EXPRESSION (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (vala_unary_expression_get_inner (self)), visitor);
	vala_code_visitor_visit_unary_expression (visitor, self);
}


ValaUnaryOperator vala_unary_expression_get_operator (ValaUnaryExpression* self)
{
	g_return_val_if_fail (VALA_IS_UNARY_EXPRESSION (self), 0);
	return self->priv->_operator;
}


void vala_unary_expression_set_operator (ValaUnaryExpression* self, ValaUnaryOperator value)
{
	g_return_if_fail (VALA_IS_UNARY_EXPRESSION (self));
	self->priv->_operator = value;
}


ValaExpression* vala_unary_expression_get_inner (ValaUnaryExpression* self)
{
	g_return_val_if_fail (VALA_IS_UNARY_EXPRESSION (self), NULL);
	return self->priv->_inner;
}


void vala_unary_expression_set_inner (ValaUnaryExpression* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_UNARY_EXPRESSION (self));
	ValaExpression* __temp4 = NULL;
	ValaExpression* __temp3 = NULL;
	self->priv->_inner = (__temp4 = (__temp3 = value, (__temp3 == NULL ? __temp3 : g_object_ref (__temp3))), (self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL))), __temp4);
}


void vala_unary_expression_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaUnaryExpression * self = VALA_UNARY_EXPRESSION (object);
	switch (property_id) {
		case VALA_UNARY_EXPRESSION_OPERATOR:
		g_value_set_int (value, vala_unary_expression_get_operator (self));
		break;
		case VALA_UNARY_EXPRESSION_INNER:
		g_value_set_object (value, vala_unary_expression_get_inner (self));
		break;
	}
}


void vala_unary_expression_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaUnaryExpression * self = VALA_UNARY_EXPRESSION (object);
	switch (property_id) {
		case VALA_UNARY_EXPRESSION_OPERATOR:
		vala_unary_expression_set_operator (self, g_value_get_int (value));
		break;
		case VALA_UNARY_EXPRESSION_INNER:
		vala_unary_expression_set_inner (self, g_value_get_object (value));
		break;
	}
}


static void vala_unary_expression_class_init (ValaUnaryExpressionClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaUnaryExpressionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_unary_expression_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_unary_expression_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_unary_expression_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_unary_expression_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_UNARY_EXPRESSION_OPERATOR, g_param_spec_int ("operator", "foo", "bar", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_UNARY_EXPRESSION_INNER, g_param_spec_object ("inner", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_unary_expression_init (ValaUnaryExpression * self)
{
	self->priv = VALA_UNARY_EXPRESSION_GET_PRIVATE (self);
}


static void vala_unary_expression_dispose (GObject * obj)
{
	ValaUnaryExpression * self = VALA_UNARY_EXPRESSION (obj);
	(self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL)));
	ValaUnaryExpressionClass * klass;
	GObjectClass * parent_class;
	klass = VALA_UNARY_EXPRESSION_CLASS (g_type_class_peek (VALA_TYPE_UNARY_EXPRESSION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_unary_expression_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaUnaryExpressionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_unary_expression_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaUnaryExpression), 0, (GInstanceInitFunc) vala_unary_expression_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_EXPRESSION, "ValaUnaryExpression", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



