/* valavariabledeclarator.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valavariabledeclarator.h>
#include <valaexpression.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>
#include <valatypereference.h>

struct _ValaVariableDeclaratorPrivate {
	char* _name;
	ValaExpression* _initializer;
};
#define VALA_VARIABLE_DECLARATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_VARIABLE_DECLARATOR, ValaVariableDeclaratorPrivate))
enum  {
	VALA_VARIABLE_DECLARATOR_DUMMY_PROPERTY,
	VALA_VARIABLE_DECLARATOR_NAME,
	VALA_VARIABLE_DECLARATOR_INITIALIZER,
};
static void vala_variable_declarator_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_variable_declarator_dispose (GObject * obj);

ValaVariableDeclarator* vala_variable_declarator_new (char* name, ValaExpression* init, ValaSourceReference* source)
{
	g_return_val_if_fail (init == NULL || VALA_IS_EXPRESSION (init), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_VARIABLE_DECLARATOR, "name", name, "initializer", init, "source_reference", source, NULL));
}


static void vala_variable_declarator_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaVariableDeclarator * self = VALA_VARIABLE_DECLARATOR (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	if (vala_variable_declarator_get_initializer (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_variable_declarator_get_initializer (self)), visitor);
		vala_code_visitor_visit_end_full_expression (visitor, vala_variable_declarator_get_initializer (self));
	}
	if (self->type_reference != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (self->type_reference), visitor);
	}
	vala_code_visitor_visit_variable_declarator (visitor, self);
}


char* vala_variable_declarator_get_name (ValaVariableDeclarator* self)
{
	g_return_val_if_fail (VALA_IS_VARIABLE_DECLARATOR (self), NULL);
	return self->priv->_name;
}


void vala_variable_declarator_set_name (ValaVariableDeclarator* self, char* value)
{
	g_return_if_fail (VALA_IS_VARIABLE_DECLARATOR (self));
	char* __temp3 = NULL;
	char* __temp2 = NULL;
	self->priv->_name = (__temp3 = (__temp2 = value, (__temp2 == NULL ? __temp2 : g_strdup (__temp2))), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp3);
}


ValaExpression* vala_variable_declarator_get_initializer (ValaVariableDeclarator* self)
{
	g_return_val_if_fail (VALA_IS_VARIABLE_DECLARATOR (self), NULL);
	return self->priv->_initializer;
}


void vala_variable_declarator_set_initializer (ValaVariableDeclarator* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_VARIABLE_DECLARATOR (self));
	ValaExpression* __temp6 = NULL;
	ValaExpression* __temp5 = NULL;
	self->priv->_initializer = (__temp6 = (__temp5 = value, (__temp5 == NULL ? __temp5 : g_object_ref (__temp5))), (self->priv->_initializer == NULL ? NULL : (self->priv->_initializer = (g_object_unref (self->priv->_initializer), NULL))), __temp6);
}


void vala_variable_declarator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaVariableDeclarator * self = VALA_VARIABLE_DECLARATOR (object);
	switch (property_id) {
		case VALA_VARIABLE_DECLARATOR_NAME:
		g_value_set_string (value, vala_variable_declarator_get_name (self));
		break;
		case VALA_VARIABLE_DECLARATOR_INITIALIZER:
		g_value_set_object (value, vala_variable_declarator_get_initializer (self));
		break;
	}
}


void vala_variable_declarator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaVariableDeclarator * self = VALA_VARIABLE_DECLARATOR (object);
	switch (property_id) {
		case VALA_VARIABLE_DECLARATOR_NAME:
		vala_variable_declarator_set_name (self, g_value_get_string (value));
		break;
		case VALA_VARIABLE_DECLARATOR_INITIALIZER:
		vala_variable_declarator_set_initializer (self, g_value_get_object (value));
		break;
	}
}


static void vala_variable_declarator_class_init (ValaVariableDeclaratorClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaVariableDeclaratorPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_variable_declarator_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_variable_declarator_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_variable_declarator_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_variable_declarator_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_VARIABLE_DECLARATOR_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_VARIABLE_DECLARATOR_INITIALIZER, g_param_spec_object ("initializer", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_variable_declarator_init (ValaVariableDeclarator * self)
{
	self->priv = VALA_VARIABLE_DECLARATOR_GET_PRIVATE (self);
}


static void vala_variable_declarator_dispose (GObject * obj)
{
	ValaVariableDeclarator * self = VALA_VARIABLE_DECLARATOR (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	(self->priv->_initializer == NULL ? NULL : (self->priv->_initializer = (g_object_unref (self->priv->_initializer), NULL)));
	(self->type_reference == NULL ? NULL : (self->type_reference = (g_object_unref (self->type_reference), NULL)));
	ValaVariableDeclaratorClass * klass;
	GObjectClass * parent_class;
	klass = VALA_VARIABLE_DECLARATOR_CLASS (g_type_class_peek (VALA_TYPE_VARIABLE_DECLARATOR));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_variable_declarator_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaVariableDeclaratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_variable_declarator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaVariableDeclarator), 0, (GInstanceInitFunc) vala_variable_declarator_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaVariableDeclarator", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



