/* valawhilestatement.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valawhilestatement.h>
#include <valaexpression.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaWhileStatementPrivate {
	ValaExpression* _condition;
	ValaStatement* _body;
};
#define VALA_WHILE_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_WHILE_STATEMENT, ValaWhileStatementPrivate))
enum  {
	VALA_WHILE_STATEMENT_DUMMY_PROPERTY,
	VALA_WHILE_STATEMENT_CONDITION,
	VALA_WHILE_STATEMENT_BODY,
};
static void vala_while_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_while_statement_dispose (GObject * obj);

ValaWhileStatement* vala_while_statement_new (ValaExpression* cond, ValaStatement* body, ValaSourceReference* source)
{
	g_return_val_if_fail (cond == NULL || VALA_IS_EXPRESSION (cond), NULL);
	g_return_val_if_fail (body == NULL || VALA_IS_STATEMENT (body), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_WHILE_STATEMENT, "condition", cond, "body", body, "source_reference", source, NULL));
}


static void vala_while_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaWhileStatement * self = VALA_WHILE_STATEMENT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (vala_while_statement_get_condition (self)), visitor);
	vala_code_visitor_visit_end_full_expression (visitor, vala_while_statement_get_condition (self));
	vala_code_node_accept (VALA_CODE_NODE (vala_while_statement_get_body (self)), visitor);
	vala_code_visitor_visit_while_statement (visitor, self);
}


ValaExpression* vala_while_statement_get_condition (ValaWhileStatement* self)
{
	g_return_val_if_fail (VALA_IS_WHILE_STATEMENT (self), NULL);
	return self->priv->_condition;
}


void vala_while_statement_set_condition (ValaWhileStatement* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_WHILE_STATEMENT (self));
	ValaExpression* __temp3 = NULL;
	ValaExpression* __temp2 = NULL;
	self->priv->_condition = (__temp3 = (__temp2 = value, (__temp2 == NULL ? __temp2 : g_object_ref (__temp2))), (self->priv->_condition == NULL ? NULL : (self->priv->_condition = (g_object_unref (self->priv->_condition), NULL))), __temp3);
}


ValaStatement* vala_while_statement_get_body (ValaWhileStatement* self)
{
	g_return_val_if_fail (VALA_IS_WHILE_STATEMENT (self), NULL);
	return self->priv->_body;
}


void vala_while_statement_set_body (ValaWhileStatement* self, ValaStatement* value)
{
	g_return_if_fail (VALA_IS_WHILE_STATEMENT (self));
	ValaStatement* __temp6 = NULL;
	ValaStatement* __temp5 = NULL;
	self->priv->_body = (__temp6 = (__temp5 = value, (__temp5 == NULL ? __temp5 : g_object_ref (__temp5))), (self->priv->_body == NULL ? NULL : (self->priv->_body = (g_object_unref (self->priv->_body), NULL))), __temp6);
}


void vala_while_statement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaWhileStatement * self = VALA_WHILE_STATEMENT (object);
	switch (property_id) {
		case VALA_WHILE_STATEMENT_CONDITION:
		g_value_set_object (value, vala_while_statement_get_condition (self));
		break;
		case VALA_WHILE_STATEMENT_BODY:
		g_value_set_object (value, vala_while_statement_get_body (self));
		break;
	}
}


void vala_while_statement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaWhileStatement * self = VALA_WHILE_STATEMENT (object);
	switch (property_id) {
		case VALA_WHILE_STATEMENT_CONDITION:
		vala_while_statement_set_condition (self, g_value_get_object (value));
		break;
		case VALA_WHILE_STATEMENT_BODY:
		vala_while_statement_set_body (self, g_value_get_object (value));
		break;
	}
}


static void vala_while_statement_class_init (ValaWhileStatementClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaWhileStatementPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_while_statement_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_while_statement_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_while_statement_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_while_statement_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_WHILE_STATEMENT_CONDITION, g_param_spec_object ("condition", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_WHILE_STATEMENT_BODY, g_param_spec_object ("body", "foo", "bar", VALA_TYPE_STATEMENT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_while_statement_init (ValaWhileStatement * self)
{
	self->priv = VALA_WHILE_STATEMENT_GET_PRIVATE (self);
}


static void vala_while_statement_dispose (GObject * obj)
{
	ValaWhileStatement * self = VALA_WHILE_STATEMENT (obj);
	(self->priv->_condition == NULL ? NULL : (self->priv->_condition = (g_object_unref (self->priv->_condition), NULL)));
	(self->priv->_body == NULL ? NULL : (self->priv->_body = (g_object_unref (self->priv->_body), NULL)));
	ValaWhileStatementClass * klass;
	GObjectClass * parent_class;
	klass = VALA_WHILE_STATEMENT_CLASS (g_type_class_peek (VALA_TYPE_WHILE_STATEMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_while_statement_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaWhileStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_while_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaWhileStatement), 0, (GInstanceInitFunc) vala_while_statement_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_STATEMENT, "ValaWhileStatement", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



