/* valacodegenerator.vala
 *
 * Copyright (C) 2006  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <rasa@gmx.ch>
 */

#include <valacodegenerator.h>
#include <valaccodestruct.h>
#include <valaccodeenum.h>
#include <valaccodefunction.h>
#include <valaccodeblock.h>
#include <valaccodeincludedirective.h>
#include <valaccodenode.h>
#include <valasourcefilecycle.h>
#include <valanamespace.h>
#include <valaccodevariabledeclarator.h>
#include <valaccodetypedefinition.h>
#include <valaccodecomment.h>
#include <valaccodewriter.h>
#include <valaccodeoncesection.h>
#include <valaccodenewline.h>
#include <valaccodeidentifier.h>
#include <valaccodemacroreplacement.h>
#include <valasourcereference.h>
#include <valaclassregisterfunction.h>
#include <valatyperegisterfunction.h>
#include <valaccodeformalparameter.h>
#include <valaccodemodifiers.h>
#include <valaccodefunctioncall.h>
#include <valaccodeconstant.h>
#include <valaccodeexpressionstatement.h>
#include <valaccodememberaccess.h>
#include <valaccodeassignment.h>
#include <valasignal.h>
#include <valamemberaccessibility.h>
#include <valaccodedeclaration.h>
#include <valaccodedeclarator.h>
#include <valaccodeswitchstatement.h>
#include <valaccodecasestatement.h>
#include <valaccodebreakstatement.h>
#include <valainterfaceregisterfunction.h>
#include <valaccodefunctiondeclarator.h>
#include <valaarray.h>
#include <valaccodebinaryexpression.h>
#include <valaflags.h>
#include <valareport.h>
#include <valaccodereturnstatement.h>
#include <valaccodeemptystatement.h>
#include <valalocalvariabledeclaration.h>
#include <valainstancecast.h>
#include <valaccodeinitializerlist.h>
#include <valaccodeparenthesizedexpression.h>
#include <valaccodeconditionalexpression.h>
#include <valaccodeifstatement.h>
#include <valaccodewhilestatement.h>
#include <valaccodeforstatement.h>
#include <valaccodecontinuestatement.h>
#include <valaccodeelementaccess.h>
#include <valaccodeunaryexpression.h>
#include <valanamedargument.h>
#include <valaccodecastexpression.h>

struct _ValaCodeGeneratorPrivate {
	gboolean _memory_management;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaSymbol* current_type_symbol;
	ValaCCodeFragment* header_begin;
	ValaCCodeFragment* header_type_declaration;
	ValaCCodeFragment* header_type_definition;
	ValaCCodeFragment* header_type_member_declaration;
	ValaCCodeFragment* source_begin;
	ValaCCodeFragment* source_include_directives;
	ValaCCodeFragment* source_type_member_declaration;
	ValaCCodeFragment* source_type_member_definition;
	ValaCCodeStruct* instance_struct;
	ValaCCodeStruct* type_struct;
	ValaCCodeStruct* instance_priv_struct;
	ValaCCodeEnum* prop_enum;
	ValaCCodeEnum* cenum;
	ValaCCodeFunction* function;
	ValaCCodeBlock* block;
	GList* temp_vars;
	GList* temp_ref_vars;
	int next_temp_var_id;
	ValaTypeReference* bool_type;
	ValaTypeReference* string_type;
};
#define VALA_CODE_GENERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CODE_GENERATOR, ValaCodeGeneratorPrivate))
enum  {
	VALA_CODE_GENERATOR_DUMMY_PROPERTY,
	VALA_CODE_GENERATOR_MEMORY_MANAGEMENT,
};
static void vala_code_generator_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file);
static char* vala_code_generator_get_define_for_filename (char* filename);
static void vala_code_generator_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file);
static void vala_code_generator_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_code_generator_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_code_generator_add_class_init_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_add_interface_init_function (ValaCodeGenerator* self, ValaClass* cl, ValaInterface* iface);
static void vala_code_generator_add_instance_init_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_add_dispose_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_add_get_property_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_add_set_property_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_code_generator_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_code_generator_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_code_generator_real_visit_begin_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vala_code_generator_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev);
static void vala_code_generator_real_visit_end_callback (ValaCodeVisitor* base, ValaCallback* cb);
static void vala_code_generator_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c);
static void vala_code_generator_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vala_code_generator_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m);
static ValaCCodeStatement* vala_code_generator_create_method_type_check_statement (ValaCodeGenerator* self, ValaMethod* m, ValaDataType* t, gboolean non_null, char* var_name);
static ValaCCodeStatement* vala_code_generator_create_property_type_check_statement (ValaCodeGenerator* self, ValaProperty* prop, gboolean getter, ValaDataType* t, gboolean non_null, char* var_name);
static ValaCCodeStatement* vala_code_generator_create_type_check_statement (ValaCodeGenerator* self, ValaCodeNode* method_node, ValaDataType* ret_type, ValaDataType* t, gboolean non_null, char* var_name);
static ValaDataType* vala_code_generator_find_parent_type (ValaCodeGenerator* self, ValaCodeNode* node);
static void vala_code_generator_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_code_generator_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p);
static void vala_code_generator_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_code_generator_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_code_generator_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_code_generator_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_code_generator_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_code_generator_real_visit_empty_statement (ValaCodeVisitor* base, ValaEmptyStatement* stmt);
static void vala_code_generator_real_visit_declaration_statement (ValaCodeVisitor* base, ValaDeclarationStatement* stmt);
static void vala_code_generator_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl);
static void vala_code_generator_real_visit_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list);
static ValaVariableDeclarator* vala_code_generator_get_temp_variable_declarator (ValaCodeGenerator* self, ValaTypeReference* type);
static ValaCCodeExpression* vala_code_generator_get_unref_expression (ValaCodeGenerator* self, ValaCCodeExpression* cvar, ValaTypeReference* type);
static void vala_code_generator_real_visit_end_full_expression (ValaCodeVisitor* base, ValaExpression* expr);
static void vala_code_generator_append_temp_decl (ValaCodeGenerator* self, ValaCCodeFragment* cfrag, GList* temp_vars);
static void vala_code_generator_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt);
static void vala_code_generator_create_temp_decl (ValaCodeGenerator* self, ValaStatement* stmt, GList* temp_vars);
static void vala_code_generator_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt);
static void vala_code_generator_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt);
static void vala_code_generator_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt);
static void vala_code_generator_real_visit_end_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt);
static void vala_code_generator_real_visit_break_statement (ValaCodeVisitor* base, ValaBreakStatement* stmt);
static void vala_code_generator_real_visit_continue_statement (ValaCodeVisitor* base, ValaContinueStatement* stmt);
static void vala_code_generator_append_local_free (ValaCodeGenerator* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop);
static void vala_code_generator_create_local_free (ValaCodeGenerator* self, ValaStatement* stmt);
static gboolean vala_code_generator_append_local_free_expr (ValaCodeGenerator* self, ValaSymbol* sym, ValaCCodeCommaExpression* ccomma, gboolean stop_at_loop);
static void vala_code_generator_create_local_free_expr (ValaCodeGenerator* self, ValaExpression* expr);
static void vala_code_generator_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt);
static void vala_code_generator_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr);
static void vala_code_generator_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr);
static void vala_code_generator_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr);
static void vala_code_generator_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr);
static void vala_code_generator_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr);
static void vala_code_generator_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr);
static void vala_code_generator_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr);
static void vala_code_generator_process_cmember (ValaCodeGenerator* self, ValaMemberAccess* expr, ValaCCodeExpression* pub_inst, ValaDataType* base_type);
static void vala_code_generator_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr);
static void vala_code_generator_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr);
static void vala_code_generator_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr);
static void vala_code_generator_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr);
static void vala_code_generator_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr);
static ValaCCodeExpression* vala_code_generator_get_ref_expression (ValaCodeGenerator* self, ValaExpression* expr);
static void vala_code_generator_visit_expression (ValaCodeGenerator* self, ValaExpression* expr);
static void vala_code_generator_real_visit_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr);
static void vala_code_generator_real_visit_typeof_expression (ValaCodeVisitor* base, ValaTypeofExpression* expr);
static void vala_code_generator_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr);
static void vala_code_generator_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr);
static void vala_code_generator_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr);
static void vala_code_generator_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr);
static void vala_code_generator_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr);
static void vala_code_generator_real_visit_end_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l);
static void vala_code_generator_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a);
static void vala_code_generator_dispose (GObject * obj);

/**
 * Generate and emit C code for the specified code context.
 *
 * @param context a code context
 */
void vala_code_generator_emit (ValaCodeGenerator* self, ValaCodeContext* context)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	vala_code_context_find_header_cycles (context);
	ValaSymbol* __temp0 = NULL;
	self->priv->root_symbol = (__temp0 = g_object_ref (vala_code_context_get_root (context)), (self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL))), __temp0);
	ValaTypeReference* __temp1 = NULL;
	self->priv->bool_type = (__temp1 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL), (self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL))), __temp1);
	self->priv->bool_type->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "bool")));
	ValaTypeReference* __temp2 = NULL;
	self->priv->string_type = (__temp2 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL), (self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL))), __temp2);
	self->priv->string_type->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "string")));
	/* we're only interested in non-pkg source files */
	GList* source_files = vala_code_context_get_source_files (context);
	{
		GList * file_it;
		for (file_it = source_files; file_it != NULL; file_it = file_it->next) {
			ValaSourceFile* file = file_it->data;
			{
				if (!vala_source_file_get_pkg (file)) {
					vala_source_file_accept (file, VALA_CODE_VISITOR (self));
				}
			}
		}
	}
	(source_files == NULL ? NULL : (source_files = (g_list_free (source_files), NULL)));
}


static void vala_code_generator_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (source_file));
	ValaCCodeFragment* __temp3 = NULL;
	self->priv->header_begin = (__temp3 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->header_begin == NULL ? NULL : (self->priv->header_begin = (g_object_unref (self->priv->header_begin), NULL))), __temp3);
	ValaCCodeFragment* __temp4 = NULL;
	self->priv->header_type_declaration = (__temp4 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->header_type_declaration == NULL ? NULL : (self->priv->header_type_declaration = (g_object_unref (self->priv->header_type_declaration), NULL))), __temp4);
	ValaCCodeFragment* __temp5 = NULL;
	self->priv->header_type_definition = (__temp5 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->header_type_definition == NULL ? NULL : (self->priv->header_type_definition = (g_object_unref (self->priv->header_type_definition), NULL))), __temp5);
	ValaCCodeFragment* __temp6 = NULL;
	self->priv->header_type_member_declaration = (__temp6 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->header_type_member_declaration == NULL ? NULL : (self->priv->header_type_member_declaration = (g_object_unref (self->priv->header_type_member_declaration), NULL))), __temp6);
	ValaCCodeFragment* __temp7 = NULL;
	self->priv->source_begin = (__temp7 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_begin == NULL ? NULL : (self->priv->source_begin = (g_object_unref (self->priv->source_begin), NULL))), __temp7);
	ValaCCodeFragment* __temp8 = NULL;
	self->priv->source_include_directives = (__temp8 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_include_directives == NULL ? NULL : (self->priv->source_include_directives = (g_object_unref (self->priv->source_include_directives), NULL))), __temp8);
	ValaCCodeFragment* __temp9 = NULL;
	self->priv->source_type_member_declaration = (__temp9 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_type_member_declaration == NULL ? NULL : (self->priv->source_type_member_declaration = (g_object_unref (self->priv->source_type_member_declaration), NULL))), __temp9);
	ValaCCodeFragment* __temp10 = NULL;
	self->priv->source_type_member_definition = (__temp10 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_type_member_definition == NULL ? NULL : (self->priv->source_type_member_definition = (g_object_unref (self->priv->source_type_member_definition), NULL))), __temp10);
	self->priv->next_temp_var_id = 0;
	ValaCCodeIncludeDirective* __temp11 = NULL;
	vala_ccode_fragment_append (self->priv->header_begin, VALA_CCODE_NODE ((__temp11 = g_object_new (VALA_TYPE_CCODE_INCLUDE_DIRECTIVE, "filename", "glib.h", NULL))));
	(__temp11 == NULL ? NULL : (__temp11 = (g_object_unref (__temp11), NULL)));
	ValaCCodeIncludeDirective* __temp12 = NULL;
	vala_ccode_fragment_append (self->priv->source_include_directives, VALA_CCODE_NODE ((__temp12 = g_object_new (VALA_TYPE_CCODE_INCLUDE_DIRECTIVE, "filename", vala_source_file_get_cheader_filename (source_file), NULL))));
	(__temp12 == NULL ? NULL : (__temp12 = (g_object_unref (__temp12), NULL)));
	GList* used_includes = NULL;
	used_includes = g_list_append (used_includes, "glib.h");
	used_includes = g_list_append (used_includes, vala_source_file_get_cheader_filename (source_file));
	{
		GList * filename1_it;
		for (filename1_it = source_file->header_external_includes; filename1_it != NULL; filename1_it = filename1_it->next) {
			char* filename1 = filename1_it->data;
			{
				if (g_list_find_custom (used_includes, filename1, strcmp) == NULL) {
					ValaCCodeIncludeDirective* __temp13 = NULL;
					vala_ccode_fragment_append (self->priv->header_begin, VALA_CCODE_NODE ((__temp13 = g_object_new (VALA_TYPE_CCODE_INCLUDE_DIRECTIVE, "filename", filename1, NULL))));
					(__temp13 == NULL ? NULL : (__temp13 = (g_object_unref (__temp13), NULL)));
					used_includes = g_list_append (used_includes, filename1);
				}
			}
		}
	}
	{
		GList * filename2_it;
		for (filename2_it = source_file->header_internal_includes; filename2_it != NULL; filename2_it = filename2_it->next) {
			char* filename2 = filename2_it->data;
			{
				if (g_list_find_custom (used_includes, filename2, strcmp) == NULL) {
					ValaCCodeIncludeDirective* __temp14 = NULL;
					vala_ccode_fragment_append (self->priv->header_begin, VALA_CCODE_NODE ((__temp14 = g_object_new (VALA_TYPE_CCODE_INCLUDE_DIRECTIVE, "filename", filename2, NULL))));
					(__temp14 == NULL ? NULL : (__temp14 = (g_object_unref (__temp14), NULL)));
					used_includes = g_list_append (used_includes, filename2);
				}
			}
		}
	}
	{
		GList * filename3_it;
		for (filename3_it = source_file->source_includes; filename3_it != NULL; filename3_it = filename3_it->next) {
			char* filename3 = filename3_it->data;
			{
				if (g_list_find_custom (used_includes, filename3, strcmp) == NULL) {
					ValaCCodeIncludeDirective* __temp15 = NULL;
					vala_ccode_fragment_append (self->priv->source_include_directives, VALA_CCODE_NODE ((__temp15 = g_object_new (VALA_TYPE_CCODE_INCLUDE_DIRECTIVE, "filename", filename3, NULL))));
					(__temp15 == NULL ? NULL : (__temp15 = (g_object_unref (__temp15), NULL)));
					used_includes = g_list_append (used_includes, filename3);
				}
			}
		}
	}
	if (source_file->is_cycle_head) {
		{
			GList * cycle_file_it;
			for (cycle_file_it = source_file->cycle->files; cycle_file_it != NULL; cycle_file_it = cycle_file_it->next) {
				ValaSourceFile* cycle_file = cycle_file_it->data;
				{
					GList* namespaces = vala_source_file_get_namespaces (cycle_file);
					{
						GList * ns_it;
						for (ns_it = namespaces; ns_it != NULL; ns_it = ns_it->next) {
							ValaNamespace* ns = ns_it->data;
							{
								GList* structs = vala_namespace_get_structs (ns);
								{
									GList * st_it;
									for (st_it = structs; st_it != NULL; st_it = st_it->next) {
										ValaStruct* st = st_it->data;
										{
											ValaCCodeTypeDefinition* __temp18 = NULL;
											ValaCCodeVariableDeclarator* __temp17 = NULL;
											char* __temp16 = NULL;
											vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp18 = g_object_new (VALA_TYPE_CCODE_TYPE_DEFINITION, "type_name", (__temp16 = g_strdup_printf ("struct _%s", vala_data_type_get_cname (VALA_DATA_TYPE (st)))), "declarator", (__temp17 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", vala_data_type_get_cname (VALA_DATA_TYPE (st)), NULL)), NULL))));
											(__temp18 == NULL ? NULL : (__temp18 = (g_object_unref (__temp18), NULL)));
											(__temp17 == NULL ? NULL : (__temp17 = (g_object_unref (__temp17), NULL)));
											(__temp16 == NULL ? NULL : (__temp16 = (g_free (__temp16), NULL)));
										}
									}
								}
								GList* classes = vala_namespace_get_classes (ns);
								{
									GList * cl_it;
									for (cl_it = classes; cl_it != NULL; cl_it = cl_it->next) {
										ValaClass* cl = cl_it->data;
										{
											ValaCCodeTypeDefinition* __temp21 = NULL;
											ValaCCodeVariableDeclarator* __temp20 = NULL;
											char* __temp19 = NULL;
											vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp21 = g_object_new (VALA_TYPE_CCODE_TYPE_DEFINITION, "type_name", (__temp19 = g_strdup_printf ("struct _%s", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), "declarator", (__temp20 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", vala_data_type_get_cname (VALA_DATA_TYPE (cl)), NULL)), NULL))));
											(__temp21 == NULL ? NULL : (__temp21 = (g_object_unref (__temp21), NULL)));
											(__temp20 == NULL ? NULL : (__temp20 = (g_object_unref (__temp20), NULL)));
											(__temp19 == NULL ? NULL : (__temp19 = (g_free (__temp19), NULL)));
											ValaCCodeTypeDefinition* __temp25 = NULL;
											ValaCCodeVariableDeclarator* __temp24 = NULL;
											char* __temp23 = NULL;
											char* __temp22 = NULL;
											vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp25 = g_object_new (VALA_TYPE_CCODE_TYPE_DEFINITION, "type_name", (__temp22 = g_strdup_printf ("struct _%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), "declarator", (__temp24 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", (__temp23 = g_strdup_printf ("%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL)), NULL))));
											(__temp25 == NULL ? NULL : (__temp25 = (g_object_unref (__temp25), NULL)));
											(__temp24 == NULL ? NULL : (__temp24 = (g_object_unref (__temp24), NULL)));
											(__temp23 == NULL ? NULL : (__temp23 = (g_free (__temp23), NULL)));
											(__temp22 == NULL ? NULL : (__temp22 = (g_free (__temp22), NULL)));
										}
									}
								}
								(structs == NULL ? NULL : (structs = (g_list_free (structs), NULL)));
								(classes == NULL ? NULL : (classes = (g_list_free (classes), NULL)));
							}
						}
					}
					(namespaces == NULL ? NULL : (namespaces = (g_list_free (namespaces), NULL)));
				}
			}
		}
	}
	(used_includes == NULL ? NULL : (used_includes = (g_list_free (used_includes), NULL)));
}


static char* vala_code_generator_get_define_for_filename (char* filename)
{
	g_return_val_if_fail (filename != NULL, NULL);
	GString* define = g_string_new ("__");
	char* i = filename;
	while (g_utf8_strlen (i, -1) > 0) {
		gunichar c = g_utf8_get_char (i);
		/* FIXME: remove explicit cast when implicit cast works */
		if (g_unichar_isalnum (c) && c < 128) {
			g_string_append_unichar (define, g_unichar_toupper (c));
		} else {
			g_string_append_c (define, '_');
		}
		i = g_utf8_next_char (i);
	}
	g_string_append (define, "__");
	char* __temp26 = NULL;
	char* __temp27 = NULL;
	return (__temp27 = (__temp26 = define->str, (__temp26 == NULL ? __temp26 : g_strdup (__temp26))), (define == NULL ? NULL : (define = (g_string_free (define, TRUE), NULL))), __temp27);
	(define == NULL ? NULL : (define = (g_string_free (define, TRUE), NULL)));
}


static void vala_code_generator_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (source_file));
	char* header_define = vala_code_generator_get_define_for_filename (vala_source_file_get_cheader_filename (source_file));
	ValaCCodeComment* comment = NULL;
	if (source_file->comment != NULL) {
		ValaCCodeComment* __temp28 = NULL;
		comment = (__temp28 = g_object_new (VALA_TYPE_CCODE_COMMENT, "text", source_file->comment, NULL), (comment == NULL ? NULL : (comment = (g_object_unref (comment), NULL))), __temp28);
	}
	ValaCCodeWriter* writer = g_object_new (VALA_TYPE_CCODE_WRITER, "filename", vala_source_file_get_cheader_filename (source_file), NULL);
	if (comment != NULL) {
		vala_ccode_node_write (VALA_CCODE_NODE (comment), writer);
	}
	vala_ccode_writer_write_newline (writer);
	ValaCCodeOnceSection* once = g_object_new (VALA_TYPE_CCODE_ONCE_SECTION, "define", header_define, NULL);
	ValaCCodeNewline* __temp29 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp29 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp29 == NULL ? NULL : (__temp29 = (g_object_unref (__temp29), NULL)));
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE (self->priv->header_begin));
	ValaCCodeNewline* __temp30 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp30 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp30 == NULL ? NULL : (__temp30 = (g_object_unref (__temp30), NULL)));
	ValaCCodeIdentifier* __temp31 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp31 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "G_BEGIN_DECLS", NULL))));
	(__temp31 == NULL ? NULL : (__temp31 = (g_object_unref (__temp31), NULL)));
	ValaCCodeNewline* __temp32 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp32 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp32 == NULL ? NULL : (__temp32 = (g_object_unref (__temp32), NULL)));
	ValaCCodeNewline* __temp33 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp33 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp33 == NULL ? NULL : (__temp33 = (g_object_unref (__temp33), NULL)));
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE (self->priv->header_type_declaration));
	ValaCCodeNewline* __temp34 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp34 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp34 == NULL ? NULL : (__temp34 = (g_object_unref (__temp34), NULL)));
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE (self->priv->header_type_definition));
	ValaCCodeNewline* __temp35 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp35 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp35 == NULL ? NULL : (__temp35 = (g_object_unref (__temp35), NULL)));
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE (self->priv->header_type_member_declaration));
	ValaCCodeNewline* __temp36 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp36 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp36 == NULL ? NULL : (__temp36 = (g_object_unref (__temp36), NULL)));
	ValaCCodeIdentifier* __temp37 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp37 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "G_END_DECLS", NULL))));
	(__temp37 == NULL ? NULL : (__temp37 = (g_object_unref (__temp37), NULL)));
	ValaCCodeNewline* __temp38 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp38 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp38 == NULL ? NULL : (__temp38 = (g_object_unref (__temp38), NULL)));
	ValaCCodeNewline* __temp39 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp39 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp39 == NULL ? NULL : (__temp39 = (g_object_unref (__temp39), NULL)));
	vala_ccode_node_write (VALA_CCODE_NODE (once), writer);
	vala_ccode_writer_close (writer);
	ValaCCodeWriter* __temp40 = NULL;
	writer = (__temp40 = g_object_new (VALA_TYPE_CCODE_WRITER, "filename", vala_source_file_get_csource_filename (source_file), NULL), (writer == NULL ? NULL : (writer = (g_object_unref (writer), NULL))), __temp40);
	if (comment != NULL) {
		vala_ccode_node_write (VALA_CCODE_NODE (comment), writer);
	}
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_begin), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_include_directives), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_type_member_declaration), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_type_member_definition), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_writer_close (writer);
	ValaCCodeFragment* __temp41 = NULL;
	self->priv->header_begin = (__temp41 = NULL, (self->priv->header_begin == NULL ? NULL : (self->priv->header_begin = (g_object_unref (self->priv->header_begin), NULL))), __temp41);
	ValaCCodeFragment* __temp42 = NULL;
	self->priv->header_type_declaration = (__temp42 = NULL, (self->priv->header_type_declaration == NULL ? NULL : (self->priv->header_type_declaration = (g_object_unref (self->priv->header_type_declaration), NULL))), __temp42);
	ValaCCodeFragment* __temp43 = NULL;
	self->priv->header_type_definition = (__temp43 = NULL, (self->priv->header_type_definition == NULL ? NULL : (self->priv->header_type_definition = (g_object_unref (self->priv->header_type_definition), NULL))), __temp43);
	ValaCCodeFragment* __temp44 = NULL;
	self->priv->header_type_member_declaration = (__temp44 = NULL, (self->priv->header_type_member_declaration == NULL ? NULL : (self->priv->header_type_member_declaration = (g_object_unref (self->priv->header_type_member_declaration), NULL))), __temp44);
	ValaCCodeFragment* __temp45 = NULL;
	self->priv->source_begin = (__temp45 = NULL, (self->priv->source_begin == NULL ? NULL : (self->priv->source_begin = (g_object_unref (self->priv->source_begin), NULL))), __temp45);
	ValaCCodeFragment* __temp46 = NULL;
	self->priv->source_include_directives = (__temp46 = NULL, (self->priv->source_include_directives == NULL ? NULL : (self->priv->source_include_directives = (g_object_unref (self->priv->source_include_directives), NULL))), __temp46);
	ValaCCodeFragment* __temp47 = NULL;
	self->priv->source_type_member_declaration = (__temp47 = NULL, (self->priv->source_type_member_declaration == NULL ? NULL : (self->priv->source_type_member_declaration = (g_object_unref (self->priv->source_type_member_declaration), NULL))), __temp47);
	ValaCCodeFragment* __temp48 = NULL;
	self->priv->source_type_member_definition = (__temp48 = NULL, (self->priv->source_type_member_definition == NULL ? NULL : (self->priv->source_type_member_definition = (g_object_unref (self->priv->source_type_member_definition), NULL))), __temp48);
	(header_define == NULL ? NULL : (header_define = (g_free (header_define), NULL)));
	(comment == NULL ? NULL : (comment = (g_object_unref (comment), NULL)));
	(writer == NULL ? NULL : (writer = (g_object_unref (writer), NULL)));
	(once == NULL ? NULL : (once = (g_object_unref (once), NULL)));
}


static void vala_code_generator_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* __temp50 = NULL;
	ValaSymbol* __temp49 = NULL;
	self->priv->current_symbol = (__temp50 = (__temp49 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp49 == NULL ? __temp49 : g_object_ref (__temp49))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp50);
	ValaSymbol* __temp52 = NULL;
	ValaSymbol* __temp51 = NULL;
	self->priv->current_type_symbol = (__temp52 = (__temp51 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp51 == NULL ? __temp51 : g_object_ref (__temp51))), (self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL))), __temp52);
	ValaCCodeStruct* __temp54 = NULL;
	char* __temp53 = NULL;
	self->priv->instance_struct = (__temp54 = g_object_new (VALA_TYPE_CCODE_STRUCT, "name", (__temp53 = g_strdup_printf ("_%s", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL), (self->priv->instance_struct == NULL ? NULL : (self->priv->instance_struct = (g_object_unref (self->priv->instance_struct), NULL))), __temp54);
	(__temp53 == NULL ? NULL : (__temp53 = (g_free (__temp53), NULL)));
	ValaCCodeStruct* __temp56 = NULL;
	char* __temp55 = NULL;
	self->priv->type_struct = (__temp56 = g_object_new (VALA_TYPE_CCODE_STRUCT, "name", (__temp55 = g_strdup_printf ("_%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL), (self->priv->type_struct == NULL ? NULL : (self->priv->type_struct = (g_object_unref (self->priv->type_struct), NULL))), __temp56);
	(__temp55 == NULL ? NULL : (__temp55 = (g_free (__temp55), NULL)));
	ValaCCodeStruct* __temp58 = NULL;
	char* __temp57 = NULL;
	self->priv->instance_priv_struct = (__temp58 = g_object_new (VALA_TYPE_CCODE_STRUCT, "name", (__temp57 = g_strdup_printf ("_%sPrivate", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL), (self->priv->instance_priv_struct == NULL ? NULL : (self->priv->instance_priv_struct = (g_object_unref (self->priv->instance_priv_struct), NULL))), __temp58);
	(__temp57 == NULL ? NULL : (__temp57 = (g_free (__temp57), NULL)));
	ValaCCodeEnum* __temp59 = NULL;
	self->priv->prop_enum = (__temp59 = g_object_new (VALA_TYPE_CCODE_ENUM, NULL), (self->priv->prop_enum == NULL ? NULL : (self->priv->prop_enum = (g_object_unref (self->priv->prop_enum), NULL))), __temp59);
	char* __temp61 = NULL;
	char* __temp60 = NULL;
	vala_ccode_enum_add_value (self->priv->prop_enum, (__temp61 = g_strdup_printf ("%s_DUMMY_PROPERTY", (__temp60 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), NULL);
	(__temp61 == NULL ? NULL : (__temp61 = (g_free (__temp61), NULL)));
	(__temp60 == NULL ? NULL : (__temp60 = (g_free (__temp60), NULL)));
	ValaCCodeNewline* __temp62 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp62 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp62 == NULL ? NULL : (__temp62 = (g_object_unref (__temp62), NULL)));
	char* __temp63 = NULL;
	char* __temp64 = NULL;
	char* macro = (__temp64 = g_strdup_printf ("(%s_get_type ())", (__temp63 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), (__temp63 == NULL ? NULL : (__temp63 = (g_free (__temp63), NULL))), __temp64);
	ValaCCodeMacroReplacement* __temp66 = NULL;
	char* __temp65 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp66 = g_object_new (VALA_TYPE_CCODE_MACRO_REPLACEMENT, "name", (__temp65 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), "replacement", macro, NULL))));
	(__temp66 == NULL ? NULL : (__temp66 = (g_object_unref (__temp66), NULL)));
	(__temp65 == NULL ? NULL : (__temp65 = (g_free (__temp65), NULL)));
	char* __temp68 = NULL;
	char* __temp67 = NULL;
	macro = (__temp68 = g_strdup_printf ("(G_TYPE_CHECK_INSTANCE_CAST ((obj), %s, %s))", (__temp67 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (cl))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp68);
	(__temp67 == NULL ? NULL : (__temp67 = (g_free (__temp67), NULL)));
	ValaCCodeMacroReplacement* __temp71 = NULL;
	char* __temp70 = NULL;
	char* __temp69 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp71 = g_object_new (VALA_TYPE_CCODE_MACRO_REPLACEMENT, "name", (__temp70 = g_strdup_printf ("%s(obj)", (__temp69 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), "replacement", macro, NULL))));
	(__temp71 == NULL ? NULL : (__temp71 = (g_object_unref (__temp71), NULL)));
	(__temp70 == NULL ? NULL : (__temp70 = (g_free (__temp70), NULL)));
	(__temp69 == NULL ? NULL : (__temp69 = (g_free (__temp69), NULL)));
	char* __temp73 = NULL;
	char* __temp72 = NULL;
	macro = (__temp73 = g_strdup_printf ("(G_TYPE_CHECK_CLASS_CAST ((klass), %s, %sClass))", (__temp72 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (cl))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp73);
	(__temp72 == NULL ? NULL : (__temp72 = (g_free (__temp72), NULL)));
	ValaCCodeMacroReplacement* __temp76 = NULL;
	char* __temp75 = NULL;
	char* __temp74 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp76 = g_object_new (VALA_TYPE_CCODE_MACRO_REPLACEMENT, "name", (__temp75 = g_strdup_printf ("%s_CLASS(klass)", (__temp74 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), "replacement", macro, NULL))));
	(__temp76 == NULL ? NULL : (__temp76 = (g_object_unref (__temp76), NULL)));
	(__temp75 == NULL ? NULL : (__temp75 = (g_free (__temp75), NULL)));
	(__temp74 == NULL ? NULL : (__temp74 = (g_free (__temp74), NULL)));
	char* __temp78 = NULL;
	char* __temp77 = NULL;
	macro = (__temp78 = g_strdup_printf ("(G_TYPE_CHECK_INSTANCE_TYPE ((obj), %s))", (__temp77 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_"))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp78);
	(__temp77 == NULL ? NULL : (__temp77 = (g_free (__temp77), NULL)));
	ValaCCodeMacroReplacement* __temp81 = NULL;
	char* __temp80 = NULL;
	char* __temp79 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp81 = g_object_new (VALA_TYPE_CCODE_MACRO_REPLACEMENT, "name", (__temp80 = g_strdup_printf ("%s(obj)", (__temp79 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "IS_")))), "replacement", macro, NULL))));
	(__temp81 == NULL ? NULL : (__temp81 = (g_object_unref (__temp81), NULL)));
	(__temp80 == NULL ? NULL : (__temp80 = (g_free (__temp80), NULL)));
	(__temp79 == NULL ? NULL : (__temp79 = (g_free (__temp79), NULL)));
	char* __temp83 = NULL;
	char* __temp82 = NULL;
	macro = (__temp83 = g_strdup_printf ("(G_TYPE_CHECK_CLASS_TYPE ((klass), %s))", (__temp82 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_"))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp83);
	(__temp82 == NULL ? NULL : (__temp82 = (g_free (__temp82), NULL)));
	ValaCCodeMacroReplacement* __temp86 = NULL;
	char* __temp85 = NULL;
	char* __temp84 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp86 = g_object_new (VALA_TYPE_CCODE_MACRO_REPLACEMENT, "name", (__temp85 = g_strdup_printf ("%s_CLASS(klass)", (__temp84 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "IS_")))), "replacement", macro, NULL))));
	(__temp86 == NULL ? NULL : (__temp86 = (g_object_unref (__temp86), NULL)));
	(__temp85 == NULL ? NULL : (__temp85 = (g_free (__temp85), NULL)));
	(__temp84 == NULL ? NULL : (__temp84 = (g_free (__temp84), NULL)));
	char* __temp88 = NULL;
	char* __temp87 = NULL;
	macro = (__temp88 = g_strdup_printf ("(G_TYPE_INSTANCE_GET_CLASS ((obj), %s, %sClass))", (__temp87 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (cl))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp88);
	(__temp87 == NULL ? NULL : (__temp87 = (g_free (__temp87), NULL)));
	ValaCCodeMacroReplacement* __temp91 = NULL;
	char* __temp90 = NULL;
	char* __temp89 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp91 = g_object_new (VALA_TYPE_CCODE_MACRO_REPLACEMENT, "name", (__temp90 = g_strdup_printf ("%s_GET_CLASS(obj)", (__temp89 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), "replacement", macro, NULL))));
	(__temp91 == NULL ? NULL : (__temp91 = (g_object_unref (__temp91), NULL)));
	(__temp90 == NULL ? NULL : (__temp90 = (g_free (__temp90), NULL)));
	(__temp89 == NULL ? NULL : (__temp89 = (g_free (__temp89), NULL)));
	ValaCCodeNewline* __temp92 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp92 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp92 == NULL ? NULL : (__temp92 = (g_object_unref (__temp92), NULL)));
	if (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (cl)))->cycle == NULL) {
		ValaCCodeTypeDefinition* __temp95 = NULL;
		ValaCCodeVariableDeclarator* __temp94 = NULL;
		char* __temp93 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp95 = g_object_new (VALA_TYPE_CCODE_TYPE_DEFINITION, "type_name", (__temp93 = g_strdup_printf ("struct %s", vala_ccode_struct_get_name (self->priv->instance_struct))), "declarator", (__temp94 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", vala_data_type_get_cname (VALA_DATA_TYPE (cl)), NULL)), NULL))));
		(__temp95 == NULL ? NULL : (__temp95 = (g_object_unref (__temp95), NULL)));
		(__temp94 == NULL ? NULL : (__temp94 = (g_object_unref (__temp94), NULL)));
		(__temp93 == NULL ? NULL : (__temp93 = (g_free (__temp93), NULL)));
		ValaCCodeTypeDefinition* __temp99 = NULL;
		ValaCCodeVariableDeclarator* __temp98 = NULL;
		char* __temp97 = NULL;
		char* __temp96 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp99 = g_object_new (VALA_TYPE_CCODE_TYPE_DEFINITION, "type_name", (__temp96 = g_strdup_printf ("struct %s", vala_ccode_struct_get_name (self->priv->type_struct))), "declarator", (__temp98 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", (__temp97 = g_strdup_printf ("%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL)), NULL))));
		(__temp99 == NULL ? NULL : (__temp99 = (g_object_unref (__temp99), NULL)));
		(__temp98 == NULL ? NULL : (__temp98 = (g_object_unref (__temp98), NULL)));
		(__temp97 == NULL ? NULL : (__temp97 = (g_free (__temp97), NULL)));
		(__temp96 == NULL ? NULL : (__temp96 = (g_free (__temp96), NULL)));
	}
	ValaCCodeTypeDefinition* __temp103 = NULL;
	ValaCCodeVariableDeclarator* __temp102 = NULL;
	char* __temp101 = NULL;
	char* __temp100 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp103 = g_object_new (VALA_TYPE_CCODE_TYPE_DEFINITION, "type_name", (__temp100 = g_strdup_printf ("struct %s", vala_ccode_struct_get_name (self->priv->instance_priv_struct))), "declarator", (__temp102 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", (__temp101 = g_strdup_printf ("%sPrivate", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL)), NULL))));
	(__temp103 == NULL ? NULL : (__temp103 = (g_object_unref (__temp103), NULL)));
	(__temp102 == NULL ? NULL : (__temp102 = (g_object_unref (__temp102), NULL)));
	(__temp101 == NULL ? NULL : (__temp101 = (g_free (__temp101), NULL)));
	(__temp100 == NULL ? NULL : (__temp100 = (g_free (__temp100), NULL)));
	vala_ccode_struct_add_field (self->priv->instance_struct, vala_data_type_get_cname (VALA_DATA_TYPE (vala_class_get_base_class (cl))), "parent");
	char* __temp104 = NULL;
	vala_ccode_struct_add_field (self->priv->instance_struct, (__temp104 = g_strdup_printf ("%sPrivate *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), "priv");
	(__temp104 == NULL ? NULL : (__temp104 = (g_free (__temp104), NULL)));
	char* __temp105 = NULL;
	vala_ccode_struct_add_field (self->priv->type_struct, (__temp105 = g_strdup_printf ("%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (vala_class_get_base_class (cl))))), "parent");
	(__temp105 == NULL ? NULL : (__temp105 = (g_free (__temp105), NULL)));
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (cl))) != NULL) {
		ValaCCodeComment* __temp106 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE ((__temp106 = g_object_new (VALA_TYPE_CCODE_COMMENT, "text", vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (cl))), NULL))));
		(__temp106 == NULL ? NULL : (__temp106 = (g_object_unref (__temp106), NULL)));
	}
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->instance_struct));
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->type_struct));
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (self->priv->instance_priv_struct));
	char* __temp108 = NULL;
	char* __temp107 = NULL;
	macro = (__temp108 = g_strdup_printf ("(G_TYPE_INSTANCE_GET_PRIVATE ((o), %s, %sPrivate))", (__temp107 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (cl))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp108);
	(__temp107 == NULL ? NULL : (__temp107 = (g_free (__temp107), NULL)));
	ValaCCodeMacroReplacement* __temp111 = NULL;
	char* __temp110 = NULL;
	char* __temp109 = NULL;
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE ((__temp111 = g_object_new (VALA_TYPE_CCODE_MACRO_REPLACEMENT, "name", (__temp110 = g_strdup_printf ("%s_GET_PRIVATE(o)", (__temp109 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), "replacement", macro, NULL))));
	(__temp111 == NULL ? NULL : (__temp111 = (g_object_unref (__temp111), NULL)));
	(__temp110 == NULL ? NULL : (__temp110 = (g_free (__temp110), NULL)));
	(__temp109 == NULL ? NULL : (__temp109 = (g_free (__temp109), NULL)));
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (self->priv->prop_enum));
	(macro == NULL ? NULL : (macro = (g_free (macro), NULL)));
}


static void vala_code_generator_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	vala_code_generator_add_get_property_function (self, cl);
	vala_code_generator_add_set_property_function (self, cl);
	vala_code_generator_add_class_init_function (self, cl);
	{
		GList * base_type_it;
		for (base_type_it = vala_class_get_base_types (cl); base_type_it != NULL; base_type_it = base_type_it->next) {
			ValaTypeReference* base_type = base_type_it->data;
			{
				if (VALA_IS_INTERFACE (base_type->type)) {
					vala_code_generator_add_interface_init_function (self, cl, VALA_INTERFACE (base_type->type));
				}
			}
		}
	}
	vala_code_generator_add_instance_init_function (self, cl);
	if (vala_code_generator_get_memory_management (self)) {
		vala_code_generator_add_dispose_function (self, cl);
	}
	ValaClassRegisterFunction* type_fun = g_object_new (VALA_TYPE_CLASS_REGISTER_FUNCTION, "class_reference", cl, NULL);
	vala_type_register_function_init_from_type (VALA_TYPE_REGISTER_FUNCTION (type_fun));
	ValaCCodeFunction* __temp112 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp112 = vala_type_register_function_get_declaration (VALA_TYPE_REGISTER_FUNCTION (type_fun)))));
	(__temp112 == NULL ? NULL : (__temp112 = (g_object_unref (__temp112), NULL)));
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (type_fun));
	ValaSymbol* __temp113 = NULL;
	self->priv->current_type_symbol = (__temp113 = NULL, (self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL))), __temp113);
	(type_fun == NULL ? NULL : (type_fun = (g_object_unref (type_fun), NULL)));
}


static void vala_code_generator_add_class_init_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	char* __temp115 = NULL;
	char* __temp114 = NULL;
	ValaCCodeFunction* __temp116 = NULL;
	ValaCCodeFunction* class_init = (__temp116 = g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", (__temp115 = g_strdup_printf ("%s_class_init", (__temp114 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)))), "return_type", "void", NULL), (__temp115 == NULL ? NULL : (__temp115 = (g_free (__temp115), NULL))), (__temp114 == NULL ? NULL : (__temp114 = (g_free (__temp114), NULL))), __temp116);
	ValaCCodeFormalParameter* __temp118 = NULL;
	char* __temp117 = NULL;
	vala_ccode_function_add_parameter (class_init, (__temp118 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", (__temp117 = g_strdup_printf ("%sClass *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), "name", "klass", NULL)));
	(__temp118 == NULL ? NULL : (__temp118 = (g_object_unref (__temp118), NULL)));
	(__temp117 == NULL ? NULL : (__temp117 = (g_free (__temp117), NULL)));
	vala_ccode_function_set_modifiers (class_init, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeBlock* init_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	vala_ccode_function_set_block (class_init, init_block);
	ValaCCodeFunctionCall* ccall = NULL;
	if (vala_class_get_has_private_fields (cl)) {
		ValaCCodeFunctionCall* __temp120 = NULL;
		ValaCCodeIdentifier* __temp119 = NULL;
		ccall = (__temp120 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp119 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_type_class_add_private", NULL)), NULL), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp120);
		(__temp119 == NULL ? NULL : (__temp119 = (g_object_unref (__temp119), NULL)));
		ValaCCodeIdentifier* __temp121 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp121 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "klass", NULL))));
		(__temp121 == NULL ? NULL : (__temp121 = (g_object_unref (__temp121), NULL)));
		ValaCCodeConstant* __temp123 = NULL;
		char* __temp122 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp123 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", (__temp122 = g_strdup_printf ("sizeof (%sPrivate)", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL))));
		(__temp123 == NULL ? NULL : (__temp123 = (g_object_unref (__temp123), NULL)));
		(__temp122 == NULL ? NULL : (__temp122 = (g_free (__temp122), NULL)));
		ValaCCodeExpressionStatement* __temp124 = NULL;
		vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp124 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", ccall, NULL))));
		(__temp124 == NULL ? NULL : (__temp124 = (g_object_unref (__temp124), NULL)));
	}
	ValaCCodeFunctionCall* __temp126 = NULL;
	ValaCCodeIdentifier* __temp125 = NULL;
	ccall = (__temp126 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp125 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "G_OBJECT_CLASS", NULL)), NULL), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp126);
	(__temp125 == NULL ? NULL : (__temp125 = (g_object_unref (__temp125), NULL)));
	ValaCCodeIdentifier* __temp127 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp127 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "klass", NULL))));
	(__temp127 == NULL ? NULL : (__temp127 = (g_object_unref (__temp127), NULL)));
	ValaCCodeExpressionStatement* __temp133 = NULL;
	ValaCCodeAssignment* __temp132 = NULL;
	ValaCCodeIdentifier* __temp131 = NULL;
	char* __temp130 = NULL;
	char* __temp129 = NULL;
	ValaCCodeMemberAccess* __temp128 = NULL;
	vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp133 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp132 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp128 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", ccall, "member_name", "get_property", "is_pointer", TRUE, NULL)), "right", (__temp131 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp130 = g_strdup_printf ("%s_get_property", (__temp129 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)))), NULL)), NULL)), NULL))));
	(__temp133 == NULL ? NULL : (__temp133 = (g_object_unref (__temp133), NULL)));
	(__temp132 == NULL ? NULL : (__temp132 = (g_object_unref (__temp132), NULL)));
	(__temp131 == NULL ? NULL : (__temp131 = (g_object_unref (__temp131), NULL)));
	(__temp130 == NULL ? NULL : (__temp130 = (g_free (__temp130), NULL)));
	(__temp129 == NULL ? NULL : (__temp129 = (g_free (__temp129), NULL)));
	(__temp128 == NULL ? NULL : (__temp128 = (g_object_unref (__temp128), NULL)));
	ValaCCodeExpressionStatement* __temp139 = NULL;
	ValaCCodeAssignment* __temp138 = NULL;
	ValaCCodeIdentifier* __temp137 = NULL;
	char* __temp136 = NULL;
	char* __temp135 = NULL;
	ValaCCodeMemberAccess* __temp134 = NULL;
	vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp139 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp138 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp134 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", ccall, "member_name", "set_property", "is_pointer", TRUE, NULL)), "right", (__temp137 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp136 = g_strdup_printf ("%s_set_property", (__temp135 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)))), NULL)), NULL)), NULL))));
	(__temp139 == NULL ? NULL : (__temp139 = (g_object_unref (__temp139), NULL)));
	(__temp138 == NULL ? NULL : (__temp138 = (g_object_unref (__temp138), NULL)));
	(__temp137 == NULL ? NULL : (__temp137 = (g_object_unref (__temp137), NULL)));
	(__temp136 == NULL ? NULL : (__temp136 = (g_free (__temp136), NULL)));
	(__temp135 == NULL ? NULL : (__temp135 = (g_free (__temp135), NULL)));
	(__temp134 == NULL ? NULL : (__temp134 = (g_object_unref (__temp134), NULL)));
	if (vala_class_get_constructor (cl) != NULL) {
		ValaCCodeIdentifier* __temp140 = NULL;
		ValaCCodeFunctionCall* __temp141 = NULL;
		ValaCCodeFunctionCall* ccast = (__temp141 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp140 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "G_OBJECT_CLASS", NULL)), NULL), (__temp140 == NULL ? NULL : (__temp140 = (g_object_unref (__temp140), NULL))), __temp141);
		ValaCCodeIdentifier* __temp142 = NULL;
		vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION ((__temp142 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "klass", NULL))));
		(__temp142 == NULL ? NULL : (__temp142 = (g_object_unref (__temp142), NULL)));
		ValaCCodeExpressionStatement* __temp148 = NULL;
		ValaCCodeAssignment* __temp147 = NULL;
		ValaCCodeIdentifier* __temp146 = NULL;
		char* __temp145 = NULL;
		char* __temp144 = NULL;
		ValaCCodeMemberAccess* __temp143 = NULL;
		vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp148 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp147 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp143 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", ccast, "member_name", "constructor", "is_pointer", TRUE, NULL)), "right", (__temp146 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp145 = g_strdup_printf ("%s_constructor", (__temp144 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)))), NULL)), NULL)), NULL))));
		(__temp148 == NULL ? NULL : (__temp148 = (g_object_unref (__temp148), NULL)));
		(__temp147 == NULL ? NULL : (__temp147 = (g_object_unref (__temp147), NULL)));
		(__temp146 == NULL ? NULL : (__temp146 = (g_object_unref (__temp146), NULL)));
		(__temp145 == NULL ? NULL : (__temp145 = (g_free (__temp145), NULL)));
		(__temp144 == NULL ? NULL : (__temp144 = (g_free (__temp144), NULL)));
		(__temp143 == NULL ? NULL : (__temp143 = (g_object_unref (__temp143), NULL)));
		(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
	}
	GList* __temp149 = NULL;
	gboolean __temp150;
	if ((__temp150 = vala_code_generator_get_memory_management (self) && (__temp149 = vala_class_get_fields (cl)) != NULL, (__temp149 == NULL ? NULL : (__temp149 = (g_list_free (__temp149), NULL))), __temp150)) {
		ValaCCodeIdentifier* __temp151 = NULL;
		ValaCCodeFunctionCall* __temp152 = NULL;
		ValaCCodeFunctionCall* ccast = (__temp152 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp151 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "G_OBJECT_CLASS", NULL)), NULL), (__temp151 == NULL ? NULL : (__temp151 = (g_object_unref (__temp151), NULL))), __temp152);
		ValaCCodeIdentifier* __temp153 = NULL;
		vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION ((__temp153 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "klass", NULL))));
		(__temp153 == NULL ? NULL : (__temp153 = (g_object_unref (__temp153), NULL)));
		ValaCCodeExpressionStatement* __temp159 = NULL;
		ValaCCodeAssignment* __temp158 = NULL;
		ValaCCodeIdentifier* __temp157 = NULL;
		char* __temp156 = NULL;
		char* __temp155 = NULL;
		ValaCCodeMemberAccess* __temp154 = NULL;
		vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp159 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp158 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp154 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", ccast, "member_name", "dispose", "is_pointer", TRUE, NULL)), "right", (__temp157 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp156 = g_strdup_printf ("%s_dispose", (__temp155 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)))), NULL)), NULL)), NULL))));
		(__temp159 == NULL ? NULL : (__temp159 = (g_object_unref (__temp159), NULL)));
		(__temp158 == NULL ? NULL : (__temp158 = (g_object_unref (__temp158), NULL)));
		(__temp157 == NULL ? NULL : (__temp157 = (g_object_unref (__temp157), NULL)));
		(__temp156 == NULL ? NULL : (__temp156 = (g_free (__temp156), NULL)));
		(__temp155 == NULL ? NULL : (__temp155 = (g_free (__temp155), NULL)));
		(__temp154 == NULL ? NULL : (__temp154 = (g_object_unref (__temp154), NULL)));
		(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
	}
	GList* methods = vala_class_get_methods (cl);
	{
		GList * m_it;
		for (m_it = methods; m_it != NULL; m_it = m_it->next) {
			ValaMethod* m = m_it->data;
			{
				if (!vala_method_get_is_virtual (m) && !vala_method_get_overrides (m)) {
					continue;
				}
				ValaCodeNode* base_type = vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_base_method (m)))->parent_symbol);
				if (VALA_IS_INTERFACE (base_type)) {
					continue;
				}
				ValaCCodeIdentifier* __temp162 = NULL;
				char* __temp161 = NULL;
				char* __temp160 = NULL;
				ValaCCodeFunctionCall* __temp163 = NULL;
				ValaCCodeFunctionCall* ccast = (__temp163 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp162 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp161 = g_strdup_printf ("%s_CLASS", (__temp160 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE ((VALA_CLASS (base_type))), NULL)))), NULL)), NULL), (__temp162 == NULL ? NULL : (__temp162 = (g_object_unref (__temp162), NULL))), (__temp161 == NULL ? NULL : (__temp161 = (g_free (__temp161), NULL))), (__temp160 == NULL ? NULL : (__temp160 = (g_free (__temp160), NULL))), __temp163);
				ValaCCodeIdentifier* __temp164 = NULL;
				vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION ((__temp164 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "klass", NULL))));
				(__temp164 == NULL ? NULL : (__temp164 = (g_object_unref (__temp164), NULL)));
				ValaCCodeExpressionStatement* __temp169 = NULL;
				ValaCCodeAssignment* __temp168 = NULL;
				ValaCCodeIdentifier* __temp167 = NULL;
				char* __temp166 = NULL;
				ValaCCodeMemberAccess* __temp165 = NULL;
				vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp169 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp168 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp165 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", ccast, "member_name", vala_method_get_name (m), "is_pointer", TRUE, NULL)), "right", (__temp167 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp166 = vala_method_get_real_cname (m)), NULL)), NULL)), NULL))));
				(__temp169 == NULL ? NULL : (__temp169 = (g_object_unref (__temp169), NULL)));
				(__temp168 == NULL ? NULL : (__temp168 = (g_object_unref (__temp168), NULL)));
				(__temp167 == NULL ? NULL : (__temp167 = (g_object_unref (__temp167), NULL)));
				(__temp166 == NULL ? NULL : (__temp166 = (g_free (__temp166), NULL)));
				(__temp165 == NULL ? NULL : (__temp165 = (g_object_unref (__temp165), NULL)));
				(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
			}
		}
	}
	GList* props = vala_class_get_properties (cl);
	{
		GList * prop_it;
		for (prop_it = props; prop_it != NULL; prop_it = prop_it->next) {
			ValaProperty* prop = prop_it->data;
			{
				ValaCCodeIdentifier* __temp170 = NULL;
				ValaCCodeFunctionCall* __temp171 = NULL;
				ValaCCodeFunctionCall* cinst = (__temp171 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp170 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_object_class_install_property", NULL)), NULL), (__temp170 == NULL ? NULL : (__temp170 = (g_object_unref (__temp170), NULL))), __temp171);
				vala_ccode_function_call_add_argument (cinst, VALA_CCODE_EXPRESSION (ccall));
				ValaCCodeConstant* __temp173 = NULL;
				char* __temp172 = NULL;
				vala_ccode_function_call_add_argument (cinst, VALA_CCODE_EXPRESSION ((__temp173 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", (__temp172 = vala_property_get_upper_case_cname (prop)), NULL))));
				(__temp173 == NULL ? NULL : (__temp173 = (g_object_unref (__temp173), NULL)));
				(__temp172 == NULL ? NULL : (__temp172 = (g_free (__temp172), NULL)));
				ValaCCodeFunctionCall* cspec = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, NULL);
				ValaCCodeConstant* __temp174 = NULL;
				vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp174 = vala_property_get_canonical_cconstant (prop))));
				(__temp174 == NULL ? NULL : (__temp174 = (g_object_unref (__temp174), NULL)));
				ValaCCodeConstant* __temp175 = NULL;
				vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp175 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "\"foo\"", NULL))));
				(__temp175 == NULL ? NULL : (__temp175 = (g_object_unref (__temp175), NULL)));
				ValaCCodeConstant* __temp176 = NULL;
				vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp176 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "\"bar\"", NULL))));
				(__temp176 == NULL ? NULL : (__temp176 = (g_object_unref (__temp176), NULL)));
				if (VALA_IS_CLASS (vala_property_get_type_reference (prop)->type)) {
					ValaCCodeIdentifier* __temp177 = NULL;
					vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp177 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_param_spec_object", NULL))));
					(__temp177 == NULL ? NULL : (__temp177 = (g_object_unref (__temp177), NULL)));
					ValaCCodeIdentifier* __temp179 = NULL;
					char* __temp178 = NULL;
					vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp179 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp178 = vala_data_type_get_upper_case_cname (vala_property_get_type_reference (prop)->type, "TYPE_")), NULL))));
					(__temp179 == NULL ? NULL : (__temp179 = (g_object_unref (__temp179), NULL)));
					(__temp178 == NULL ? NULL : (__temp178 = (g_free (__temp178), NULL)));
				} else if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "string") == 0) {
					ValaCCodeIdentifier* __temp180 = NULL;
					vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp180 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_param_spec_string", NULL))));
					(__temp180 == NULL ? NULL : (__temp180 = (g_object_unref (__temp180), NULL)));
					ValaCCodeConstant* __temp181 = NULL;
					vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp181 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL))));
					(__temp181 == NULL ? NULL : (__temp181 = (g_object_unref (__temp181), NULL)));
				} else if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "int") == 0 || VALA_IS_ENUM (vala_property_get_type_reference (prop)->type)) {
					ValaCCodeIdentifier* __temp182 = NULL;
					vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp182 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_param_spec_int", NULL))));
					(__temp182 == NULL ? NULL : (__temp182 = (g_object_unref (__temp182), NULL)));
					ValaCCodeConstant* __temp183 = NULL;
					vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp183 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "G_MININT", NULL))));
					(__temp183 == NULL ? NULL : (__temp183 = (g_object_unref (__temp183), NULL)));
					ValaCCodeConstant* __temp184 = NULL;
					vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp184 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "G_MAXINT", NULL))));
					(__temp184 == NULL ? NULL : (__temp184 = (g_object_unref (__temp184), NULL)));
					ValaCCodeConstant* __temp185 = NULL;
					vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp185 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "0", NULL))));
					(__temp185 == NULL ? NULL : (__temp185 = (g_object_unref (__temp185), NULL)));
				} else if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "bool") == 0) {
					ValaCCodeIdentifier* __temp186 = NULL;
					vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp186 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_param_spec_boolean", NULL))));
					(__temp186 == NULL ? NULL : (__temp186 = (g_object_unref (__temp186), NULL)));
					ValaCCodeConstant* __temp187 = NULL;
					vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp187 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "FALSE", NULL))));
					(__temp187 == NULL ? NULL : (__temp187 = (g_object_unref (__temp187), NULL)));
				} else {
					ValaCCodeIdentifier* __temp188 = NULL;
					vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp188 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_param_spec_pointer", NULL))));
					(__temp188 == NULL ? NULL : (__temp188 = (g_object_unref (__temp188), NULL)));
				}
				char* pflags = g_strdup ("G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB");
				if (vala_property_get_get_accessor (prop) != NULL) {
					char* __temp189 = NULL;
					pflags = (__temp189 = g_strdup_printf ("%s%s", pflags, " | G_PARAM_READABLE"), (pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL))), __temp189);
				}
				if (vala_property_get_set_accessor (prop) != NULL) {
					char* __temp190 = NULL;
					pflags = (__temp190 = g_strdup_printf ("%s%s", pflags, " | G_PARAM_WRITABLE"), (pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL))), __temp190);
					if (vala_property_accessor_get_construct_ (vala_property_get_set_accessor (prop))) {
						if (vala_property_accessor_get_writable (vala_property_get_set_accessor (prop))) {
							char* __temp191 = NULL;
							pflags = (__temp191 = g_strdup_printf ("%s%s", pflags, " | G_PARAM_CONSTRUCT"), (pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL))), __temp191);
						} else {
							char* __temp192 = NULL;
							pflags = (__temp192 = g_strdup_printf ("%s%s", pflags, " | G_PARAM_CONSTRUCT_ONLY"), (pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL))), __temp192);
						}
					}
				}
				ValaCCodeConstant* __temp193 = NULL;
				vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp193 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", pflags, NULL))));
				(__temp193 == NULL ? NULL : (__temp193 = (g_object_unref (__temp193), NULL)));
				vala_ccode_function_call_add_argument (cinst, VALA_CCODE_EXPRESSION (cspec));
				ValaCCodeExpressionStatement* __temp194 = NULL;
				vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp194 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", cinst, NULL))));
				(__temp194 == NULL ? NULL : (__temp194 = (g_object_unref (__temp194), NULL)));
				(cinst == NULL ? NULL : (cinst = (g_object_unref (cinst), NULL)));
				(cspec == NULL ? NULL : (cspec = (g_object_unref (cspec), NULL)));
				(pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL)));
			}
		}
	}
	{
		GList * sig_it;
		for (sig_it = vala_class_get_signals (cl); sig_it != NULL; sig_it = sig_it->next) {
			ValaSignal* sig = sig_it->data;
			{
				ValaCCodeIdentifier* __temp195 = NULL;
				ValaCCodeFunctionCall* __temp196 = NULL;
				ValaCCodeFunctionCall* csignew = (__temp196 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp195 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_signal_new", NULL)), NULL), (__temp195 == NULL ? NULL : (__temp195 = (g_object_unref (__temp195), NULL))), __temp196);
				ValaCCodeConstant* __temp198 = NULL;
				char* __temp197 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp198 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", (__temp197 = g_strdup_printf ("\"%s\"", vala_signal_get_name (sig))), NULL))));
				(__temp198 == NULL ? NULL : (__temp198 = (g_object_unref (__temp198), NULL)));
				(__temp197 == NULL ? NULL : (__temp197 = (g_free (__temp197), NULL)));
				ValaCCodeIdentifier* __temp200 = NULL;
				char* __temp199 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp200 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp199 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), NULL))));
				(__temp200 == NULL ? NULL : (__temp200 = (g_object_unref (__temp200), NULL)));
				(__temp199 == NULL ? NULL : (__temp199 = (g_free (__temp199), NULL)));
				ValaCCodeConstant* __temp201 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp201 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "G_SIGNAL_RUN_LAST", NULL))));
				(__temp201 == NULL ? NULL : (__temp201 = (g_object_unref (__temp201), NULL)));
				ValaCCodeConstant* __temp202 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp202 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "0", NULL))));
				(__temp202 == NULL ? NULL : (__temp202 = (g_object_unref (__temp202), NULL)));
				ValaCCodeConstant* __temp203 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp203 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL))));
				(__temp203 == NULL ? NULL : (__temp203 = (g_object_unref (__temp203), NULL)));
				ValaCCodeConstant* __temp204 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp204 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL))));
				(__temp204 == NULL ? NULL : (__temp204 = (g_object_unref (__temp204), NULL)));
				/* TODO: generate marshallers */
				char* marshaller = g_strdup ("g_cclosure_marshal");
				ValaCCodeIdentifier* marshal_arg = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", marshaller, NULL);
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION (marshal_arg));
				GList* params = vala_signal_get_parameters (sig);
				unsigned int params_len = g_list_length (params);
				if (vala_signal_get_return_type (sig)->type == NULL) {
					char* __temp205 = NULL;
					marshaller = (__temp205 = g_strdup_printf ("%s_VOID_", marshaller), (marshaller == NULL ? NULL : (marshaller = (g_free (marshaller), NULL))), __temp205);
					ValaCCodeConstant* __temp206 = NULL;
					vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp206 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "G_TYPE_NONE", NULL))));
					(__temp206 == NULL ? NULL : (__temp206 = (g_object_unref (__temp206), NULL)));
				} else {
					char* __temp207 = NULL;
					marshaller = (__temp207 = g_strdup_printf ("%s_%s_", marshaller, vala_data_type_get_marshaller_type_name (vala_signal_get_return_type (sig)->type)), (marshaller == NULL ? NULL : (marshaller = (g_free (marshaller), NULL))), __temp207);
					ValaCCodeConstant* __temp208 = NULL;
					vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp208 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", vala_data_type_get_type_id (vala_signal_get_return_type (sig)->type), NULL))));
					(__temp208 == NULL ? NULL : (__temp208 = (g_object_unref (__temp208), NULL)));
				}
				ValaCCodeConstant* __temp210 = NULL;
				char* __temp209 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp210 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", (__temp209 = g_strdup_printf ("%d", params_len)), NULL))));
				(__temp210 == NULL ? NULL : (__temp210 = (g_object_unref (__temp210), NULL)));
				(__temp209 == NULL ? NULL : (__temp209 = (g_free (__temp209), NULL)));
				{
					GList * param_it;
					for (param_it = params; param_it != NULL; param_it = param_it->next) {
						ValaFormalParameter* param = param_it->data;
						{
							char* __temp211 = NULL;
							marshaller = (__temp211 = g_strdup_printf ("%s_%s", marshaller, vala_data_type_get_marshaller_type_name (vala_formal_parameter_get_type_reference (param)->type)), (marshaller == NULL ? NULL : (marshaller = (g_free (marshaller), NULL))), __temp211);
							ValaCCodeConstant* __temp212 = NULL;
							vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp212 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", vala_data_type_get_type_id (vala_formal_parameter_get_type_reference (param)->type), NULL))));
							(__temp212 == NULL ? NULL : (__temp212 = (g_object_unref (__temp212), NULL)));
						}
					}
				}
				if (params_len == 0) {
					char* __temp213 = NULL;
					marshaller = (__temp213 = g_strdup_printf ("%s_VOID", marshaller), (marshaller == NULL ? NULL : (marshaller = (g_free (marshaller), NULL))), __temp213);
				}
				vala_ccode_identifier_set_name (marshal_arg, marshaller);
				ValaCCodeExpressionStatement* __temp214 = NULL;
				vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp214 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", csignew, NULL))));
				(__temp214 == NULL ? NULL : (__temp214 = (g_object_unref (__temp214), NULL)));
				(csignew == NULL ? NULL : (csignew = (g_object_unref (csignew), NULL)));
				(marshaller == NULL ? NULL : (marshaller = (g_free (marshaller), NULL)));
				(marshal_arg == NULL ? NULL : (marshal_arg = (g_object_unref (marshal_arg), NULL)));
				(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
			}
		}
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (class_init));
	(class_init == NULL ? NULL : (class_init = (g_object_unref (class_init), NULL)));
	(init_block == NULL ? NULL : (init_block = (g_object_unref (init_block), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(methods == NULL ? NULL : (methods = (g_list_free (methods), NULL)));
	(props == NULL ? NULL : (props = (g_list_free (props), NULL)));
}


static void vala_code_generator_add_interface_init_function (ValaCodeGenerator* self, ValaClass* cl, ValaInterface* iface)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	char* __temp217 = NULL;
	char* __temp216 = NULL;
	char* __temp215 = NULL;
	ValaCCodeFunction* __temp218 = NULL;
	ValaCCodeFunction* iface_init = (__temp218 = g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", (__temp217 = g_strdup_printf ("%s_%s_interface_init", (__temp215 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), (__temp216 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (iface), NULL)))), "return_type", "void", NULL), (__temp217 == NULL ? NULL : (__temp217 = (g_free (__temp217), NULL))), (__temp216 == NULL ? NULL : (__temp216 = (g_free (__temp216), NULL))), (__temp215 == NULL ? NULL : (__temp215 = (g_free (__temp215), NULL))), __temp218);
	ValaCCodeFormalParameter* __temp220 = NULL;
	char* __temp219 = NULL;
	vala_ccode_function_add_parameter (iface_init, (__temp220 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", (__temp219 = g_strdup_printf ("%sInterface *", vala_data_type_get_cname (VALA_DATA_TYPE (iface)))), "name", "iface", NULL)));
	(__temp220 == NULL ? NULL : (__temp220 = (g_object_unref (__temp220), NULL)));
	(__temp219 == NULL ? NULL : (__temp219 = (g_free (__temp219), NULL)));
	vala_ccode_function_set_modifiers (iface_init, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeBlock* init_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	vala_ccode_function_set_block (iface_init, init_block);
	ValaCCodeFunctionCall* ccall = NULL;
	GList* methods = vala_class_get_methods (cl);
	{
		GList * m_it;
		for (m_it = methods; m_it != NULL; m_it = m_it->next) {
			ValaMethod* m = m_it->data;
			{
				if (!vala_method_get_overrides (m)) {
					continue;
				}
				ValaCodeNode* base_type = vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_base_method (m)))->parent_symbol);
				if (base_type != iface) {
					continue;
				}
				ValaCCodeIdentifier* ciface = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "iface", NULL);
				ValaCCodeExpressionStatement* __temp225 = NULL;
				ValaCCodeAssignment* __temp224 = NULL;
				ValaCCodeIdentifier* __temp223 = NULL;
				char* __temp222 = NULL;
				ValaCCodeMemberAccess* __temp221 = NULL;
				vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp225 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp224 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp221 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", ciface, "member_name", vala_method_get_name (m), "is_pointer", TRUE, NULL)), "right", (__temp223 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp222 = vala_method_get_real_cname (m)), NULL)), NULL)), NULL))));
				(__temp225 == NULL ? NULL : (__temp225 = (g_object_unref (__temp225), NULL)));
				(__temp224 == NULL ? NULL : (__temp224 = (g_object_unref (__temp224), NULL)));
				(__temp223 == NULL ? NULL : (__temp223 = (g_object_unref (__temp223), NULL)));
				(__temp222 == NULL ? NULL : (__temp222 = (g_free (__temp222), NULL)));
				(__temp221 == NULL ? NULL : (__temp221 = (g_object_unref (__temp221), NULL)));
				(ciface == NULL ? NULL : (ciface = (g_object_unref (ciface), NULL)));
			}
		}
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (iface_init));
	(iface_init == NULL ? NULL : (iface_init = (g_object_unref (iface_init), NULL)));
	(init_block == NULL ? NULL : (init_block = (g_object_unref (init_block), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(methods == NULL ? NULL : (methods = (g_list_free (methods), NULL)));
}


static void vala_code_generator_add_instance_init_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	char* __temp227 = NULL;
	char* __temp226 = NULL;
	ValaCCodeFunction* __temp228 = NULL;
	ValaCCodeFunction* instance_init = (__temp228 = g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", (__temp227 = g_strdup_printf ("%s_init", (__temp226 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)))), "return_type", "void", NULL), (__temp227 == NULL ? NULL : (__temp227 = (g_free (__temp227), NULL))), (__temp226 == NULL ? NULL : (__temp226 = (g_free (__temp226), NULL))), __temp228);
	ValaCCodeFormalParameter* __temp230 = NULL;
	char* __temp229 = NULL;
	vala_ccode_function_add_parameter (instance_init, (__temp230 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", (__temp229 = g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), "name", "self", NULL)));
	(__temp230 == NULL ? NULL : (__temp230 = (g_object_unref (__temp230), NULL)));
	(__temp229 == NULL ? NULL : (__temp229 = (g_free (__temp229), NULL)));
	vala_ccode_function_set_modifiers (instance_init, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeBlock* init_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	vala_ccode_function_set_block (instance_init, init_block);
	if (vala_class_get_has_private_fields (cl)) {
		ValaCCodeIdentifier* __temp233 = NULL;
		char* __temp232 = NULL;
		char* __temp231 = NULL;
		ValaCCodeFunctionCall* __temp234 = NULL;
		ValaCCodeFunctionCall* ccall = (__temp234 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp233 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp232 = g_strdup_printf ("%s_GET_PRIVATE", (__temp231 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), NULL)), NULL), (__temp233 == NULL ? NULL : (__temp233 = (g_object_unref (__temp233), NULL))), (__temp232 == NULL ? NULL : (__temp232 = (g_free (__temp232), NULL))), (__temp231 == NULL ? NULL : (__temp231 = (g_free (__temp231), NULL))), __temp234);
		ValaCCodeIdentifier* __temp235 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp235 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL))));
		(__temp235 == NULL ? NULL : (__temp235 = (g_object_unref (__temp235), NULL)));
		ValaCCodeExpressionStatement* __temp239 = NULL;
		ValaCCodeAssignment* __temp238 = NULL;
		ValaCCodeMemberAccess* __temp237 = NULL;
		ValaCCodeIdentifier* __temp236 = NULL;
		vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp239 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp238 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp237 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", (__temp236 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL)), "member_name", "priv", "is_pointer", TRUE, NULL)), "right", ccall, NULL)), NULL))));
		(__temp239 == NULL ? NULL : (__temp239 = (g_object_unref (__temp239), NULL)));
		(__temp238 == NULL ? NULL : (__temp238 = (g_object_unref (__temp238), NULL)));
		(__temp237 == NULL ? NULL : (__temp237 = (g_object_unref (__temp237), NULL)));
		(__temp236 == NULL ? NULL : (__temp236 = (g_object_unref (__temp236), NULL)));
		(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	}
	GList* fields = vala_class_get_fields (cl);
	{
		GList * f_it;
		for (f_it = fields; f_it != NULL; f_it = f_it->next) {
			ValaField* f = f_it->data;
			{
				if (vala_field_get_initializer (f) != NULL) {
					ValaCCodeExpression* lhs = NULL;
					if (vala_field_get_instance (f)) {
						if (f->access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
							ValaCCodeExpression* __temp242 = NULL;
							ValaCCodeMemberAccess* __temp241 = NULL;
							ValaCCodeIdentifier* __temp240 = NULL;
							lhs = (__temp242 = VALA_CCODE_EXPRESSION (g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", (__temp241 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", (__temp240 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL)), "member_name", "priv", "is_pointer", TRUE, NULL)), "member_name", vala_field_get_cname (f), "is_pointer", TRUE, NULL)), (lhs == NULL ? NULL : (lhs = (g_object_unref (lhs), NULL))), __temp242);
							(__temp241 == NULL ? NULL : (__temp241 = (g_object_unref (__temp241), NULL)));
							(__temp240 == NULL ? NULL : (__temp240 = (g_object_unref (__temp240), NULL)));
						} else {
							ValaCCodeExpression* __temp244 = NULL;
							ValaCCodeIdentifier* __temp243 = NULL;
							lhs = (__temp244 = VALA_CCODE_EXPRESSION (g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", (__temp243 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL)), "member_name", vala_field_get_cname (f), "is_pointer", TRUE, NULL)), (lhs == NULL ? NULL : (lhs = (g_object_unref (lhs), NULL))), __temp244);
							(__temp243 == NULL ? NULL : (__temp243 = (g_object_unref (__temp243), NULL)));
						}
					}
					/* else {
					lhs = new CCodeIdentifier (name = "%s_%s".printf (cl.get_lower_case_cname (null), f.get_cname ()));
					} */
					if (lhs != NULL) {
						ValaCCodeExpressionStatement* __temp246 = NULL;
						ValaCCodeAssignment* __temp245 = NULL;
						vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp246 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp245 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", lhs, "right", vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_field_get_initializer (f))), NULL)), NULL))));
						(__temp246 == NULL ? NULL : (__temp246 = (g_object_unref (__temp246), NULL)));
						(__temp245 == NULL ? NULL : (__temp245 = (g_object_unref (__temp245), NULL)));
					}
					(lhs == NULL ? NULL : (lhs = (g_object_unref (lhs), NULL)));
				}
			}
		}
	}
	ValaSymbol* init_sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (cl)), "init");
	if (init_sym != NULL) {
		ValaMethod* init_fun = VALA_METHOD (vala_symbol_get_node (init_sym));
		vala_ccode_block_add_statement (init_block, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_method_get_body (init_fun))));
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (instance_init));
	(instance_init == NULL ? NULL : (instance_init = (g_object_unref (instance_init), NULL)));
	(init_block == NULL ? NULL : (init_block = (g_object_unref (init_block), NULL)));
	(fields == NULL ? NULL : (fields = (g_list_free (fields), NULL)));
}


static void vala_code_generator_add_dispose_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaCCodeFunction* __temp249 = NULL;
	char* __temp248 = NULL;
	char* __temp247 = NULL;
	self->priv->function = (__temp249 = g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", (__temp248 = g_strdup_printf ("%s_dispose", (__temp247 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)))), "return_type", "void", NULL), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp249);
	(__temp248 == NULL ? NULL : (__temp248 = (g_free (__temp248), NULL)));
	(__temp247 == NULL ? NULL : (__temp247 = (g_free (__temp247), NULL)));
	vala_ccode_function_set_modifiers (self->priv->function, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeFormalParameter* __temp250 = NULL;
	vala_ccode_function_add_parameter (self->priv->function, (__temp250 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "name", "obj", "type_name", "GObject *", NULL)));
	(__temp250 == NULL ? NULL : (__temp250 = (g_object_unref (__temp250), NULL)));
	ValaCCodeFunction* __temp251 = NULL;
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE ((__temp251 = vala_ccode_function_copy (self->priv->function))));
	(__temp251 == NULL ? NULL : (__temp251 = (g_object_unref (__temp251), NULL)));
	ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeIdentifier* __temp253 = NULL;
	char* __temp252 = NULL;
	ValaCCodeFunctionCall* __temp254 = NULL;
	ValaCCodeFunctionCall* ccall = (__temp254 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp253 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp252 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)), NULL)), NULL), (__temp253 == NULL ? NULL : (__temp253 = (g_object_unref (__temp253), NULL))), (__temp252 == NULL ? NULL : (__temp252 = (g_free (__temp252), NULL))), __temp254);
	ValaCCodeIdentifier* __temp255 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp255 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "obj", NULL))));
	(__temp255 == NULL ? NULL : (__temp255 = (g_object_unref (__temp255), NULL)));
	char* __temp256 = NULL;
	ValaCCodeDeclaration* __temp257 = NULL;
	ValaCCodeDeclaration* cdecl = (__temp257 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp256 = g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL), (__temp256 == NULL ? NULL : (__temp256 = (g_free (__temp256), NULL))), __temp257);
	ValaCCodeVariableDeclarator* __temp258 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp258 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", "self", "initializer", ccall, NULL))));
	(__temp258 == NULL ? NULL : (__temp258 = (g_object_unref (__temp258), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	GList* fields = vala_class_get_fields (cl);
	{
		GList * f_it;
		for (f_it = fields; f_it != NULL; f_it = f_it->next) {
			ValaField* f = f_it->data;
			{
				if (vala_field_get_instance (f) && vala_type_reference_get_is_lvalue_ref (vala_field_get_type_reference (f))) {
					ValaCCodeIdentifier* cself = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL);
					ValaCCodeIdentifier* __temp259 = NULL;
					ValaCCodeExpression* cstruct = VALA_CCODE_EXPRESSION ((__temp259 = cself, (__temp259 == NULL ? __temp259 : g_object_ref (__temp259))));
					if (f->access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
						ValaCCodeExpression* __temp260 = NULL;
						cstruct = (__temp260 = VALA_CCODE_EXPRESSION (g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", cself, "member_name", "priv", "is_pointer", TRUE, NULL)), (cstruct == NULL ? NULL : (cstruct = (g_object_unref (cstruct), NULL))), __temp260);
					}
					ValaCCodeMemberAccess* cfield = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", cstruct, "member_name", vala_field_get_cname (f), "is_pointer", TRUE, NULL);
					ValaCCodeExpressionStatement* __temp262 = NULL;
					ValaCCodeExpression* __temp261 = NULL;
					vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp262 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp261 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION (cfield), vala_field_get_type_reference (f))), NULL))));
					(__temp262 == NULL ? NULL : (__temp262 = (g_object_unref (__temp262), NULL)));
					(__temp261 == NULL ? NULL : (__temp261 = (g_object_unref (__temp261), NULL)));
					(cself == NULL ? NULL : (cself = (g_object_unref (cself), NULL)));
					(cstruct == NULL ? NULL : (cstruct = (g_object_unref (cstruct), NULL)));
					(cfield == NULL ? NULL : (cfield = (g_object_unref (cfield), NULL)));
				}
			}
		}
	}
	ValaCCodeDeclaration* __temp264 = NULL;
	char* __temp263 = NULL;
	cdecl = (__temp264 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp263 = g_strdup_printf ("%sClass *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp264);
	(__temp263 == NULL ? NULL : (__temp263 = (g_free (__temp263), NULL)));
	ValaCCodeVariableDeclarator* __temp265 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp265 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", "klass", NULL))));
	(__temp265 == NULL ? NULL : (__temp265 = (g_object_unref (__temp265), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeDeclaration* __temp266 = NULL;
	cdecl = (__temp266 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", "GObjectClass *", NULL), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp266);
	ValaCCodeVariableDeclarator* __temp267 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp267 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", "parent_class", NULL))));
	(__temp267 == NULL ? NULL : (__temp267 = (g_object_unref (__temp267), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeFunctionCall* __temp269 = NULL;
	ValaCCodeIdentifier* __temp268 = NULL;
	ccall = (__temp269 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp268 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_type_class_peek", NULL)), NULL), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp269);
	(__temp268 == NULL ? NULL : (__temp268 = (g_object_unref (__temp268), NULL)));
	ValaCCodeIdentifier* __temp271 = NULL;
	char* __temp270 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp271 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp270 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), NULL))));
	(__temp271 == NULL ? NULL : (__temp271 = (g_object_unref (__temp271), NULL)));
	(__temp270 == NULL ? NULL : (__temp270 = (g_free (__temp270), NULL)));
	ValaCCodeIdentifier* __temp274 = NULL;
	char* __temp273 = NULL;
	char* __temp272 = NULL;
	ValaCCodeFunctionCall* __temp275 = NULL;
	ValaCCodeFunctionCall* ccast = (__temp275 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp274 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp273 = g_strdup_printf ("%s_CLASS", (__temp272 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), NULL)), NULL), (__temp274 == NULL ? NULL : (__temp274 = (g_object_unref (__temp274), NULL))), (__temp273 == NULL ? NULL : (__temp273 = (g_free (__temp273), NULL))), (__temp272 == NULL ? NULL : (__temp272 = (g_free (__temp272), NULL))), __temp275);
	vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeExpressionStatement* __temp278 = NULL;
	ValaCCodeAssignment* __temp277 = NULL;
	ValaCCodeIdentifier* __temp276 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp278 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp277 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp276 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "klass", NULL)), "right", ccast, NULL)), NULL))));
	(__temp278 == NULL ? NULL : (__temp278 = (g_object_unref (__temp278), NULL)));
	(__temp277 == NULL ? NULL : (__temp277 = (g_object_unref (__temp277), NULL)));
	(__temp276 == NULL ? NULL : (__temp276 = (g_object_unref (__temp276), NULL)));
	ValaCCodeFunctionCall* __temp280 = NULL;
	ValaCCodeIdentifier* __temp279 = NULL;
	ccall = (__temp280 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp279 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_type_class_peek_parent", NULL)), NULL), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp280);
	(__temp279 == NULL ? NULL : (__temp279 = (g_object_unref (__temp279), NULL)));
	ValaCCodeIdentifier* __temp281 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp281 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "klass", NULL))));
	(__temp281 == NULL ? NULL : (__temp281 = (g_object_unref (__temp281), NULL)));
	ValaCCodeFunctionCall* __temp283 = NULL;
	ValaCCodeIdentifier* __temp282 = NULL;
	ccast = (__temp283 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp282 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "G_OBJECT_CLASS", NULL)), NULL), (ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL))), __temp283);
	(__temp282 == NULL ? NULL : (__temp282 = (g_object_unref (__temp282), NULL)));
	vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeExpressionStatement* __temp286 = NULL;
	ValaCCodeAssignment* __temp285 = NULL;
	ValaCCodeIdentifier* __temp284 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp286 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp285 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp284 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "parent_class", NULL)), "right", ccast, NULL)), NULL))));
	(__temp286 == NULL ? NULL : (__temp286 = (g_object_unref (__temp286), NULL)));
	(__temp285 == NULL ? NULL : (__temp285 = (g_object_unref (__temp285), NULL)));
	(__temp284 == NULL ? NULL : (__temp284 = (g_object_unref (__temp284), NULL)));
	ValaCCodeFunctionCall* __temp289 = NULL;
	ValaCCodeMemberAccess* __temp288 = NULL;
	ValaCCodeIdentifier* __temp287 = NULL;
	ccall = (__temp289 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp288 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", (__temp287 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "parent_class", NULL)), "member_name", "dispose", "is_pointer", TRUE, NULL)), NULL), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp289);
	(__temp288 == NULL ? NULL : (__temp288 = (g_object_unref (__temp288), NULL)));
	(__temp287 == NULL ? NULL : (__temp287 = (g_object_unref (__temp287), NULL)));
	ValaCCodeIdentifier* __temp290 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp290 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "obj", NULL))));
	(__temp290 == NULL ? NULL : (__temp290 = (g_object_unref (__temp290), NULL)));
	ValaCCodeExpressionStatement* __temp291 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp291 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", ccall, NULL))));
	(__temp291 == NULL ? NULL : (__temp291 = (g_object_unref (__temp291), NULL)));
	vala_ccode_function_set_block (self->priv->function, cblock);
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (self->priv->function));
	(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(fields == NULL ? NULL : (fields = (g_list_free (fields), NULL)));
	(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
}


static void vala_code_generator_add_get_property_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	char* __temp293 = NULL;
	char* __temp292 = NULL;
	ValaCCodeFunction* __temp294 = NULL;
	ValaCCodeFunction* get_prop = (__temp294 = g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", (__temp293 = g_strdup_printf ("%s_get_property", (__temp292 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)))), "return_type", "void", NULL), (__temp293 == NULL ? NULL : (__temp293 = (g_free (__temp293), NULL))), (__temp292 == NULL ? NULL : (__temp292 = (g_free (__temp292), NULL))), __temp294);
	ValaCCodeFormalParameter* __temp295 = NULL;
	vala_ccode_function_add_parameter (get_prop, (__temp295 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", "GObject *", "name", "object", NULL)));
	(__temp295 == NULL ? NULL : (__temp295 = (g_object_unref (__temp295), NULL)));
	ValaCCodeFormalParameter* __temp296 = NULL;
	vala_ccode_function_add_parameter (get_prop, (__temp296 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", "guint", "name", "property_id", NULL)));
	(__temp296 == NULL ? NULL : (__temp296 = (g_object_unref (__temp296), NULL)));
	ValaCCodeFormalParameter* __temp297 = NULL;
	vala_ccode_function_add_parameter (get_prop, (__temp297 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", "GValue *", "name", "value", NULL)));
	(__temp297 == NULL ? NULL : (__temp297 = (g_object_unref (__temp297), NULL)));
	ValaCCodeFormalParameter* __temp298 = NULL;
	vala_ccode_function_add_parameter (get_prop, (__temp298 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", "GParamSpec *", "name", "pspec", NULL)));
	(__temp298 == NULL ? NULL : (__temp298 = (g_object_unref (__temp298), NULL)));
	ValaCCodeBlock* block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeIdentifier* __temp300 = NULL;
	char* __temp299 = NULL;
	ValaCCodeFunctionCall* __temp301 = NULL;
	ValaCCodeFunctionCall* ccall = (__temp301 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp300 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp299 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)), NULL)), NULL), (__temp300 == NULL ? NULL : (__temp300 = (g_object_unref (__temp300), NULL))), (__temp299 == NULL ? NULL : (__temp299 = (g_free (__temp299), NULL))), __temp301);
	ValaCCodeIdentifier* __temp302 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp302 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "object", NULL))));
	(__temp302 == NULL ? NULL : (__temp302 = (g_object_unref (__temp302), NULL)));
	char* __temp303 = NULL;
	ValaCCodeDeclaration* __temp304 = NULL;
	ValaCCodeDeclaration* cdecl = (__temp304 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp303 = g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL), (__temp303 == NULL ? NULL : (__temp303 = (g_free (__temp303), NULL))), __temp304);
	ValaCCodeVariableDeclarator* __temp305 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp305 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", "self", "initializer", ccall, NULL))));
	(__temp305 == NULL ? NULL : (__temp305 = (g_object_unref (__temp305), NULL)));
	vala_ccode_block_add_statement (block, VALA_CCODE_NODE (cdecl));
	ValaCCodeIdentifier* __temp306 = NULL;
	ValaCCodeSwitchStatement* __temp307 = NULL;
	ValaCCodeSwitchStatement* cswitch = (__temp307 = g_object_new (VALA_TYPE_CCODE_SWITCH_STATEMENT, "expression", (__temp306 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "property_id", NULL)), NULL), (__temp306 == NULL ? NULL : (__temp306 = (g_object_unref (__temp306), NULL))), __temp307);
	GList* props = vala_class_get_properties (cl);
	{
		GList * prop_it;
		for (prop_it = props; prop_it != NULL; prop_it = prop_it->next) {
			ValaProperty* prop = prop_it->data;
			{
				if (vala_property_get_get_accessor (prop) == NULL) {
					continue;
				}
				ValaCCodeIdentifier* __temp309 = NULL;
				char* __temp308 = NULL;
				ValaCCodeCaseStatement* __temp310 = NULL;
				ValaCCodeCaseStatement* ccase = (__temp310 = g_object_new (VALA_TYPE_CCODE_CASE_STATEMENT, "expression", (__temp309 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp308 = vala_property_get_upper_case_cname (prop)), NULL)), NULL), (__temp309 == NULL ? NULL : (__temp309 = (g_object_unref (__temp309), NULL))), (__temp308 == NULL ? NULL : (__temp308 = (g_free (__temp308), NULL))), __temp310);
				ValaCCodeIdentifier* __temp313 = NULL;
				char* __temp312 = NULL;
				char* __temp311 = NULL;
				ValaCCodeFunctionCall* __temp314 = NULL;
				ValaCCodeFunctionCall* ccall = (__temp314 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp313 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp312 = g_strdup_printf ("%s_get_%s", (__temp311 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop))), NULL)), NULL), (__temp313 == NULL ? NULL : (__temp313 = (g_object_unref (__temp313), NULL))), (__temp312 == NULL ? NULL : (__temp312 = (g_free (__temp312), NULL))), (__temp311 == NULL ? NULL : (__temp311 = (g_free (__temp311), NULL))), __temp314);
				ValaCCodeIdentifier* __temp315 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp315 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL))));
				(__temp315 == NULL ? NULL : (__temp315 = (g_object_unref (__temp315), NULL)));
				ValaCCodeFunctionCall* csetcall = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, NULL);
				if (VALA_IS_CLASS (vala_property_get_type_reference (prop)->type)) {
					ValaCCodeIdentifier* __temp316 = NULL;
					vala_ccode_function_call_set_call (csetcall, VALA_CCODE_EXPRESSION ((__temp316 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_value_set_object", NULL))));
					(__temp316 == NULL ? NULL : (__temp316 = (g_object_unref (__temp316), NULL)));
				} else if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "string") == 0) {
					ValaCCodeIdentifier* __temp317 = NULL;
					vala_ccode_function_call_set_call (csetcall, VALA_CCODE_EXPRESSION ((__temp317 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_value_set_string", NULL))));
					(__temp317 == NULL ? NULL : (__temp317 = (g_object_unref (__temp317), NULL)));
				} else if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "int") == 0 || VALA_IS_ENUM (vala_property_get_type_reference (prop)->type)) {
					ValaCCodeIdentifier* __temp318 = NULL;
					vala_ccode_function_call_set_call (csetcall, VALA_CCODE_EXPRESSION ((__temp318 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_value_set_int", NULL))));
					(__temp318 == NULL ? NULL : (__temp318 = (g_object_unref (__temp318), NULL)));
				} else if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "bool") == 0) {
					ValaCCodeIdentifier* __temp319 = NULL;
					vala_ccode_function_call_set_call (csetcall, VALA_CCODE_EXPRESSION ((__temp319 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_value_set_boolean", NULL))));
					(__temp319 == NULL ? NULL : (__temp319 = (g_object_unref (__temp319), NULL)));
				} else {
					ValaCCodeIdentifier* __temp320 = NULL;
					vala_ccode_function_call_set_call (csetcall, VALA_CCODE_EXPRESSION ((__temp320 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_value_set_pointer", NULL))));
					(__temp320 == NULL ? NULL : (__temp320 = (g_object_unref (__temp320), NULL)));
				}
				ValaCCodeIdentifier* __temp321 = NULL;
				vala_ccode_function_call_add_argument (csetcall, VALA_CCODE_EXPRESSION ((__temp321 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "value", NULL))));
				(__temp321 == NULL ? NULL : (__temp321 = (g_object_unref (__temp321), NULL)));
				vala_ccode_function_call_add_argument (csetcall, VALA_CCODE_EXPRESSION (ccall));
				ValaCCodeExpressionStatement* __temp322 = NULL;
				vala_ccode_case_statement_add_statement (ccase, VALA_CCODE_STATEMENT ((__temp322 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", csetcall, NULL))));
				(__temp322 == NULL ? NULL : (__temp322 = (g_object_unref (__temp322), NULL)));
				ValaCCodeBreakStatement* __temp323 = NULL;
				vala_ccode_case_statement_add_statement (ccase, VALA_CCODE_STATEMENT ((__temp323 = g_object_new (VALA_TYPE_CCODE_BREAK_STATEMENT, NULL))));
				(__temp323 == NULL ? NULL : (__temp323 = (g_object_unref (__temp323), NULL)));
				vala_ccode_switch_statement_add_case (cswitch, ccase);
				(ccase == NULL ? NULL : (ccase = (g_object_unref (ccase), NULL)));
				(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
				(csetcall == NULL ? NULL : (csetcall = (g_object_unref (csetcall), NULL)));
			}
		}
	}
	vala_ccode_block_add_statement (block, VALA_CCODE_NODE (cswitch));
	vala_ccode_function_set_block (get_prop, block);
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (get_prop));
	(get_prop == NULL ? NULL : (get_prop = (g_object_unref (get_prop), NULL)));
	(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(cswitch == NULL ? NULL : (cswitch = (g_object_unref (cswitch), NULL)));
	(props == NULL ? NULL : (props = (g_list_free (props), NULL)));
}


static void vala_code_generator_add_set_property_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	char* __temp325 = NULL;
	char* __temp324 = NULL;
	ValaCCodeFunction* __temp326 = NULL;
	ValaCCodeFunction* set_prop = (__temp326 = g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", (__temp325 = g_strdup_printf ("%s_set_property", (__temp324 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)))), "return_type", "void", NULL), (__temp325 == NULL ? NULL : (__temp325 = (g_free (__temp325), NULL))), (__temp324 == NULL ? NULL : (__temp324 = (g_free (__temp324), NULL))), __temp326);
	ValaCCodeFormalParameter* __temp327 = NULL;
	vala_ccode_function_add_parameter (set_prop, (__temp327 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", "GObject *", "name", "object", NULL)));
	(__temp327 == NULL ? NULL : (__temp327 = (g_object_unref (__temp327), NULL)));
	ValaCCodeFormalParameter* __temp328 = NULL;
	vala_ccode_function_add_parameter (set_prop, (__temp328 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", "guint", "name", "property_id", NULL)));
	(__temp328 == NULL ? NULL : (__temp328 = (g_object_unref (__temp328), NULL)));
	ValaCCodeFormalParameter* __temp329 = NULL;
	vala_ccode_function_add_parameter (set_prop, (__temp329 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", "const GValue *", "name", "value", NULL)));
	(__temp329 == NULL ? NULL : (__temp329 = (g_object_unref (__temp329), NULL)));
	ValaCCodeFormalParameter* __temp330 = NULL;
	vala_ccode_function_add_parameter (set_prop, (__temp330 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", "GParamSpec *", "name", "pspec", NULL)));
	(__temp330 == NULL ? NULL : (__temp330 = (g_object_unref (__temp330), NULL)));
	ValaCCodeBlock* block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeIdentifier* __temp332 = NULL;
	char* __temp331 = NULL;
	ValaCCodeFunctionCall* __temp333 = NULL;
	ValaCCodeFunctionCall* ccall = (__temp333 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp332 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp331 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)), NULL)), NULL), (__temp332 == NULL ? NULL : (__temp332 = (g_object_unref (__temp332), NULL))), (__temp331 == NULL ? NULL : (__temp331 = (g_free (__temp331), NULL))), __temp333);
	ValaCCodeIdentifier* __temp334 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp334 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "object", NULL))));
	(__temp334 == NULL ? NULL : (__temp334 = (g_object_unref (__temp334), NULL)));
	char* __temp335 = NULL;
	ValaCCodeDeclaration* __temp336 = NULL;
	ValaCCodeDeclaration* cdecl = (__temp336 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp335 = g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL), (__temp335 == NULL ? NULL : (__temp335 = (g_free (__temp335), NULL))), __temp336);
	ValaCCodeVariableDeclarator* __temp337 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp337 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", "self", "initializer", ccall, NULL))));
	(__temp337 == NULL ? NULL : (__temp337 = (g_object_unref (__temp337), NULL)));
	vala_ccode_block_add_statement (block, VALA_CCODE_NODE (cdecl));
	ValaCCodeIdentifier* __temp338 = NULL;
	ValaCCodeSwitchStatement* __temp339 = NULL;
	ValaCCodeSwitchStatement* cswitch = (__temp339 = g_object_new (VALA_TYPE_CCODE_SWITCH_STATEMENT, "expression", (__temp338 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "property_id", NULL)), NULL), (__temp338 == NULL ? NULL : (__temp338 = (g_object_unref (__temp338), NULL))), __temp339);
	GList* props = vala_class_get_properties (cl);
	{
		GList * prop_it;
		for (prop_it = props; prop_it != NULL; prop_it = prop_it->next) {
			ValaProperty* prop = prop_it->data;
			{
				if (vala_property_get_set_accessor (prop) == NULL) {
					continue;
				}
				ValaCCodeIdentifier* __temp341 = NULL;
				char* __temp340 = NULL;
				ValaCCodeCaseStatement* __temp342 = NULL;
				ValaCCodeCaseStatement* ccase = (__temp342 = g_object_new (VALA_TYPE_CCODE_CASE_STATEMENT, "expression", (__temp341 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp340 = vala_property_get_upper_case_cname (prop)), NULL)), NULL), (__temp341 == NULL ? NULL : (__temp341 = (g_object_unref (__temp341), NULL))), (__temp340 == NULL ? NULL : (__temp340 = (g_free (__temp340), NULL))), __temp342);
				ValaCCodeIdentifier* __temp345 = NULL;
				char* __temp344 = NULL;
				char* __temp343 = NULL;
				ValaCCodeFunctionCall* __temp346 = NULL;
				ValaCCodeFunctionCall* ccall = (__temp346 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp345 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp344 = g_strdup_printf ("%s_set_%s", (__temp343 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop))), NULL)), NULL), (__temp345 == NULL ? NULL : (__temp345 = (g_object_unref (__temp345), NULL))), (__temp344 == NULL ? NULL : (__temp344 = (g_free (__temp344), NULL))), (__temp343 == NULL ? NULL : (__temp343 = (g_free (__temp343), NULL))), __temp346);
				ValaCCodeIdentifier* __temp347 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp347 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL))));
				(__temp347 == NULL ? NULL : (__temp347 = (g_object_unref (__temp347), NULL)));
				ValaCCodeFunctionCall* cgetcall = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, NULL);
				if (VALA_IS_CLASS (vala_property_get_type_reference (prop)->type)) {
					ValaCCodeIdentifier* __temp348 = NULL;
					vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp348 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_value_get_object", NULL))));
					(__temp348 == NULL ? NULL : (__temp348 = (g_object_unref (__temp348), NULL)));
				} else if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "string") == 0) {
					ValaCCodeIdentifier* __temp349 = NULL;
					vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp349 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_value_get_string", NULL))));
					(__temp349 == NULL ? NULL : (__temp349 = (g_object_unref (__temp349), NULL)));
				} else if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "int") == 0 || VALA_IS_ENUM (vala_property_get_type_reference (prop)->type)) {
					ValaCCodeIdentifier* __temp350 = NULL;
					vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp350 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_value_get_int", NULL))));
					(__temp350 == NULL ? NULL : (__temp350 = (g_object_unref (__temp350), NULL)));
				} else if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "bool") == 0) {
					ValaCCodeIdentifier* __temp351 = NULL;
					vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp351 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_value_get_boolean", NULL))));
					(__temp351 == NULL ? NULL : (__temp351 = (g_object_unref (__temp351), NULL)));
				} else {
					ValaCCodeIdentifier* __temp352 = NULL;
					vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp352 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_value_get_pointer", NULL))));
					(__temp352 == NULL ? NULL : (__temp352 = (g_object_unref (__temp352), NULL)));
				}
				ValaCCodeIdentifier* __temp353 = NULL;
				vala_ccode_function_call_add_argument (cgetcall, VALA_CCODE_EXPRESSION ((__temp353 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "value", NULL))));
				(__temp353 == NULL ? NULL : (__temp353 = (g_object_unref (__temp353), NULL)));
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (cgetcall));
				ValaCCodeExpressionStatement* __temp354 = NULL;
				vala_ccode_case_statement_add_statement (ccase, VALA_CCODE_STATEMENT ((__temp354 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", ccall, NULL))));
				(__temp354 == NULL ? NULL : (__temp354 = (g_object_unref (__temp354), NULL)));
				ValaCCodeBreakStatement* __temp355 = NULL;
				vala_ccode_case_statement_add_statement (ccase, VALA_CCODE_STATEMENT ((__temp355 = g_object_new (VALA_TYPE_CCODE_BREAK_STATEMENT, NULL))));
				(__temp355 == NULL ? NULL : (__temp355 = (g_object_unref (__temp355), NULL)));
				vala_ccode_switch_statement_add_case (cswitch, ccase);
				(ccase == NULL ? NULL : (ccase = (g_object_unref (ccase), NULL)));
				(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
				(cgetcall == NULL ? NULL : (cgetcall = (g_object_unref (cgetcall), NULL)));
			}
		}
	}
	vala_ccode_block_add_statement (block, VALA_CCODE_NODE (cswitch));
	vala_ccode_function_set_block (set_prop, block);
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (set_prop));
	(set_prop == NULL ? NULL : (set_prop = (g_object_unref (set_prop), NULL)));
	(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(cswitch == NULL ? NULL : (cswitch = (g_object_unref (cswitch), NULL)));
	(props == NULL ? NULL : (props = (g_list_free (props), NULL)));
}


static void vala_code_generator_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaCCodeStruct* __temp357 = NULL;
	char* __temp356 = NULL;
	self->priv->instance_struct = (__temp357 = g_object_new (VALA_TYPE_CCODE_STRUCT, "name", (__temp356 = g_strdup_printf ("_%s", vala_data_type_get_name (VALA_DATA_TYPE (st)))), NULL), (self->priv->instance_struct == NULL ? NULL : (self->priv->instance_struct = (g_object_unref (self->priv->instance_struct), NULL))), __temp357);
	(__temp356 == NULL ? NULL : (__temp356 = (g_free (__temp356), NULL)));
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (st))) != NULL) {
		ValaCCodeComment* __temp358 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE ((__temp358 = g_object_new (VALA_TYPE_CCODE_COMMENT, "text", vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (st))), NULL))));
		(__temp358 == NULL ? NULL : (__temp358 = (g_object_unref (__temp358), NULL)));
	}
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->instance_struct));
}


static void vala_code_generator_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	ValaSymbol* __temp360 = NULL;
	ValaSymbol* __temp359 = NULL;
	self->priv->current_symbol = (__temp360 = (__temp359 = vala_code_node_get_symbol (VALA_CODE_NODE (iface)), (__temp359 == NULL ? __temp359 : g_object_ref (__temp359))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp360);
	ValaSymbol* __temp362 = NULL;
	ValaSymbol* __temp361 = NULL;
	self->priv->current_type_symbol = (__temp362 = (__temp361 = vala_code_node_get_symbol (VALA_CODE_NODE (iface)), (__temp361 == NULL ? __temp361 : g_object_ref (__temp361))), (self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL))), __temp362);
	ValaCCodeStruct* __temp364 = NULL;
	char* __temp363 = NULL;
	self->priv->type_struct = (__temp364 = g_object_new (VALA_TYPE_CCODE_STRUCT, "name", (__temp363 = g_strdup_printf ("_%sInterface", vala_data_type_get_cname (VALA_DATA_TYPE (iface)))), NULL), (self->priv->type_struct == NULL ? NULL : (self->priv->type_struct = (g_object_unref (self->priv->type_struct), NULL))), __temp364);
	(__temp363 == NULL ? NULL : (__temp363 = (g_free (__temp363), NULL)));
	ValaCCodeNewline* __temp365 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp365 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp365 == NULL ? NULL : (__temp365 = (g_object_unref (__temp365), NULL)));
	char* __temp366 = NULL;
	char* __temp367 = NULL;
	char* macro = (__temp367 = g_strdup_printf ("(%s_get_type ())", (__temp366 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (iface), NULL))), (__temp366 == NULL ? NULL : (__temp366 = (g_free (__temp366), NULL))), __temp367);
	ValaCCodeMacroReplacement* __temp369 = NULL;
	char* __temp368 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp369 = g_object_new (VALA_TYPE_CCODE_MACRO_REPLACEMENT, "name", (__temp368 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_")), "replacement", macro, NULL))));
	(__temp369 == NULL ? NULL : (__temp369 = (g_object_unref (__temp369), NULL)));
	(__temp368 == NULL ? NULL : (__temp368 = (g_free (__temp368), NULL)));
	char* __temp371 = NULL;
	char* __temp370 = NULL;
	macro = (__temp371 = g_strdup_printf ("(G_TYPE_CHECK_INSTANCE_CAST ((obj), %s, %s))", (__temp370 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (iface))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp371);
	(__temp370 == NULL ? NULL : (__temp370 = (g_free (__temp370), NULL)));
	ValaCCodeMacroReplacement* __temp374 = NULL;
	char* __temp373 = NULL;
	char* __temp372 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp374 = g_object_new (VALA_TYPE_CCODE_MACRO_REPLACEMENT, "name", (__temp373 = g_strdup_printf ("%s(obj)", (__temp372 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), NULL)))), "replacement", macro, NULL))));
	(__temp374 == NULL ? NULL : (__temp374 = (g_object_unref (__temp374), NULL)));
	(__temp373 == NULL ? NULL : (__temp373 = (g_free (__temp373), NULL)));
	(__temp372 == NULL ? NULL : (__temp372 = (g_free (__temp372), NULL)));
	char* __temp376 = NULL;
	char* __temp375 = NULL;
	macro = (__temp376 = g_strdup_printf ("(G_TYPE_CHECK_INSTANCE_TYPE ((obj), %s))", (__temp375 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_"))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp376);
	(__temp375 == NULL ? NULL : (__temp375 = (g_free (__temp375), NULL)));
	ValaCCodeMacroReplacement* __temp379 = NULL;
	char* __temp378 = NULL;
	char* __temp377 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp379 = g_object_new (VALA_TYPE_CCODE_MACRO_REPLACEMENT, "name", (__temp378 = g_strdup_printf ("%s(obj)", (__temp377 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "IS_")))), "replacement", macro, NULL))));
	(__temp379 == NULL ? NULL : (__temp379 = (g_object_unref (__temp379), NULL)));
	(__temp378 == NULL ? NULL : (__temp378 = (g_free (__temp378), NULL)));
	(__temp377 == NULL ? NULL : (__temp377 = (g_free (__temp377), NULL)));
	char* __temp381 = NULL;
	char* __temp380 = NULL;
	macro = (__temp381 = g_strdup_printf ("(G_TYPE_INSTANCE_GET_INTERFACE ((obj), %s, %sInterface))", (__temp380 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (iface))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp381);
	(__temp380 == NULL ? NULL : (__temp380 = (g_free (__temp380), NULL)));
	ValaCCodeMacroReplacement* __temp384 = NULL;
	char* __temp383 = NULL;
	char* __temp382 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp384 = g_object_new (VALA_TYPE_CCODE_MACRO_REPLACEMENT, "name", (__temp383 = g_strdup_printf ("%s_GET_INTERFACE(obj)", (__temp382 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), NULL)))), "replacement", macro, NULL))));
	(__temp384 == NULL ? NULL : (__temp384 = (g_object_unref (__temp384), NULL)));
	(__temp383 == NULL ? NULL : (__temp383 = (g_free (__temp383), NULL)));
	(__temp382 == NULL ? NULL : (__temp382 = (g_free (__temp382), NULL)));
	ValaCCodeNewline* __temp385 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp385 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp385 == NULL ? NULL : (__temp385 = (g_object_unref (__temp385), NULL)));
	if (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (iface)))->cycle == NULL) {
		ValaCCodeTypeDefinition* __temp388 = NULL;
		ValaCCodeVariableDeclarator* __temp387 = NULL;
		char* __temp386 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp388 = g_object_new (VALA_TYPE_CCODE_TYPE_DEFINITION, "type_name", (__temp386 = g_strdup_printf ("struct _%s", vala_data_type_get_cname (VALA_DATA_TYPE (iface)))), "declarator", (__temp387 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", vala_data_type_get_cname (VALA_DATA_TYPE (iface)), NULL)), NULL))));
		(__temp388 == NULL ? NULL : (__temp388 = (g_object_unref (__temp388), NULL)));
		(__temp387 == NULL ? NULL : (__temp387 = (g_object_unref (__temp387), NULL)));
		(__temp386 == NULL ? NULL : (__temp386 = (g_free (__temp386), NULL)));
		ValaCCodeTypeDefinition* __temp392 = NULL;
		ValaCCodeVariableDeclarator* __temp391 = NULL;
		char* __temp390 = NULL;
		char* __temp389 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp392 = g_object_new (VALA_TYPE_CCODE_TYPE_DEFINITION, "type_name", (__temp389 = g_strdup_printf ("struct %s", vala_ccode_struct_get_name (self->priv->type_struct))), "declarator", (__temp391 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", (__temp390 = g_strdup_printf ("%sInterface", vala_data_type_get_cname (VALA_DATA_TYPE (iface)))), NULL)), NULL))));
		(__temp392 == NULL ? NULL : (__temp392 = (g_object_unref (__temp392), NULL)));
		(__temp391 == NULL ? NULL : (__temp391 = (g_object_unref (__temp391), NULL)));
		(__temp390 == NULL ? NULL : (__temp390 = (g_free (__temp390), NULL)));
		(__temp389 == NULL ? NULL : (__temp389 = (g_free (__temp389), NULL)));
	}
	vala_ccode_struct_add_field (self->priv->type_struct, "GTypeInterface", "parent");
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (iface))) != NULL) {
		ValaCCodeComment* __temp393 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE ((__temp393 = g_object_new (VALA_TYPE_CCODE_COMMENT, "text", vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (iface))), NULL))));
		(__temp393 == NULL ? NULL : (__temp393 = (g_object_unref (__temp393), NULL)));
	}
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->type_struct));
	(macro == NULL ? NULL : (macro = (g_free (macro), NULL)));
}


static void vala_code_generator_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	ValaInterfaceRegisterFunction* type_fun = g_object_new (VALA_TYPE_INTERFACE_REGISTER_FUNCTION, "interface_reference", iface, NULL);
	vala_type_register_function_init_from_type (VALA_TYPE_REGISTER_FUNCTION (type_fun));
	ValaCCodeFunction* __temp394 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp394 = vala_type_register_function_get_declaration (VALA_TYPE_REGISTER_FUNCTION (type_fun)))));
	(__temp394 == NULL ? NULL : (__temp394 = (g_object_unref (__temp394), NULL)));
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (type_fun));
	ValaSymbol* __temp395 = NULL;
	self->priv->current_type_symbol = (__temp395 = NULL, (self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL))), __temp395);
	(type_fun == NULL ? NULL : (type_fun = (g_object_unref (type_fun), NULL)));
}


static void vala_code_generator_real_visit_begin_enum (ValaCodeVisitor* base, ValaEnum* en)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ENUM (en));
	ValaCCodeEnum* __temp396 = NULL;
	self->priv->cenum = (__temp396 = g_object_new (VALA_TYPE_CCODE_ENUM, "name", vala_data_type_get_cname (VALA_DATA_TYPE (en)), NULL), (self->priv->cenum == NULL ? NULL : (self->priv->cenum = (g_object_unref (self->priv->cenum), NULL))), __temp396);
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (en))) != NULL) {
		ValaCCodeComment* __temp397 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE ((__temp397 = g_object_new (VALA_TYPE_CCODE_COMMENT, "text", vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (en))), NULL))));
		(__temp397 == NULL ? NULL : (__temp397 = (g_object_unref (__temp397), NULL)));
	}
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->cenum));
}


static void vala_code_generator_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ENUM_VALUE (ev));
	vala_ccode_enum_add_value (self->priv->cenum, vala_enum_value_get_cname (ev), NULL);
}


static void vala_code_generator_real_visit_end_callback (ValaCodeVisitor* base, ValaCallback* cb)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CALLBACK (cb));
	ValaCCodeTypeDefinition* ctypedef = g_object_new (VALA_TYPE_CCODE_TYPE_DEFINITION, NULL);
	char* __temp398 = NULL;
	vala_ccode_type_definition_set_type_name (ctypedef, (__temp398 = vala_type_reference_get_cname (vala_callback_get_return_type (cb), FALSE)));
	(__temp398 == NULL ? NULL : (__temp398 = (g_free (__temp398), NULL)));
	ValaCCodeFunctionDeclarator* cfundecl = g_object_new (VALA_TYPE_CCODE_FUNCTION_DECLARATOR, "name", vala_data_type_get_cname (VALA_DATA_TYPE (cb)), NULL);
	{
		GList * param_it;
		for (param_it = vala_callback_get_parameters (cb); param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				vala_ccode_function_declarator_add_parameter (cfundecl, VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode (VALA_CODE_NODE (param))));
			}
		}
	}
	vala_ccode_type_definition_set_declarator (ctypedef, VALA_CCODE_DECLARATOR (cfundecl));
	if (VALA_DATA_TYPE (cb)->access == VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE (ctypedef));
	} else {
		vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (ctypedef));
	}
	(ctypedef == NULL ? NULL : (ctypedef = (g_object_unref (ctypedef), NULL)));
	(cfundecl == NULL ? NULL : (cfundecl = (g_object_unref (cfundecl), NULL)));
}


static void vala_code_generator_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CONSTANT (c));
	if (VALA_IS_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (c))->parent_symbol))) {
		ValaDataType* t = VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (c))->parent_symbol));
		char* __temp399 = NULL;
		ValaCCodeDeclaration* __temp400 = NULL;
		ValaCCodeDeclaration* cdecl = (__temp400 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp399 = vala_type_reference_get_const_cname (vala_constant_get_type_reference (c))), NULL), (__temp399 == NULL ? NULL : (__temp399 = (g_free (__temp399), NULL))), __temp400);
		char* arr = "";
		if (VALA_IS_ARRAY (vala_constant_get_type_reference (c)->type)) {
			arr = "[]";
		}
		ValaCCodeVariableDeclarator* __temp402 = NULL;
		char* __temp401 = NULL;
		vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp402 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", (__temp401 = g_strdup_printf ("%s%s", vala_constant_get_cname (c), arr)), "initializer", vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_constant_get_initializer (c))), NULL))));
		(__temp402 == NULL ? NULL : (__temp402 = (g_object_unref (__temp402), NULL)));
		(__temp401 == NULL ? NULL : (__temp401 = (g_free (__temp401), NULL)));
		vala_ccode_declaration_set_modifiers (cdecl, VALA_CCODE_MODIFIERS_STATIC);
		vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (cdecl));
		(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	}
}


static void vala_code_generator_real_visit_field (ValaCodeVisitor* base, ValaField* f)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_FIELD (f));
	if (f->access == VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		char* __temp403 = NULL;
		vala_ccode_struct_add_field (self->priv->instance_struct, (__temp403 = vala_type_reference_get_cname (vala_field_get_type_reference (f), FALSE)), vala_field_get_cname (f));
		(__temp403 == NULL ? NULL : (__temp403 = (g_free (__temp403), NULL)));
	} else if (f->access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
		if (vala_field_get_instance (f)) {
			char* __temp404 = NULL;
			vala_ccode_struct_add_field (self->priv->instance_priv_struct, (__temp404 = vala_type_reference_get_cname (vala_field_get_type_reference (f), FALSE)), vala_field_get_cname (f));
			(__temp404 == NULL ? NULL : (__temp404 = (g_free (__temp404), NULL)));
		} else {
			if (VALA_IS_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (f))->parent_symbol))) {
				ValaDataType* t = VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (f))->parent_symbol));
				char* __temp405 = NULL;
				ValaCCodeDeclaration* __temp406 = NULL;
				ValaCCodeDeclaration* cdecl = (__temp406 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp405 = vala_type_reference_get_cname (vala_field_get_type_reference (f), FALSE)), NULL), (__temp405 == NULL ? NULL : (__temp405 = (g_free (__temp405), NULL))), __temp406);
				char* __temp408 = NULL;
				char* __temp407 = NULL;
				ValaCCodeVariableDeclarator* __temp409 = NULL;
				ValaCCodeVariableDeclarator* var_decl = (__temp409 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", (__temp408 = g_strdup_printf ("%s_%s", (__temp407 = vala_data_type_get_lower_case_cname (t, NULL)), vala_field_get_cname (f))), NULL), (__temp408 == NULL ? NULL : (__temp408 = (g_free (__temp408), NULL))), (__temp407 == NULL ? NULL : (__temp407 = (g_free (__temp407), NULL))), __temp409);
				if (vala_field_get_initializer (f) != NULL) {
					vala_ccode_variable_declarator_set_initializer (var_decl, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_field_get_initializer (f)))));
				}
				vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR (var_decl));
				vala_ccode_declaration_set_modifiers (cdecl, VALA_CCODE_MODIFIERS_STATIC);
				vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (cdecl));
				(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
				(var_decl == NULL ? NULL : (var_decl = (g_object_unref (var_decl), NULL)));
			}
		}
	}
}


static void vala_code_generator_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp411 = NULL;
	ValaSymbol* __temp410 = NULL;
	self->priv->current_symbol = (__temp411 = (__temp410 = vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp410 == NULL ? __temp410 : g_object_ref (__temp410))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp411);
}


static ValaCCodeStatement* vala_code_generator_create_method_type_check_statement (ValaCodeGenerator* self, ValaMethod* m, ValaDataType* t, gboolean non_null, char* var_name)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_METHOD (m), NULL);
	g_return_val_if_fail (VALA_IS_DATA_TYPE (t), NULL);
	g_return_val_if_fail (var_name != NULL, NULL);
	return vala_code_generator_create_type_check_statement (self, VALA_CODE_NODE (m), vala_method_get_return_type (m)->type, t, non_null, var_name);
}


static ValaCCodeStatement* vala_code_generator_create_property_type_check_statement (ValaCodeGenerator* self, ValaProperty* prop, gboolean getter, ValaDataType* t, gboolean non_null, char* var_name)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_PROPERTY (prop), NULL);
	g_return_val_if_fail (VALA_IS_DATA_TYPE (t), NULL);
	g_return_val_if_fail (var_name != NULL, NULL);
	if (getter) {
		return vala_code_generator_create_type_check_statement (self, VALA_CODE_NODE (prop), vala_property_get_type_reference (prop)->type, t, non_null, var_name);
	} else {
		return vala_code_generator_create_type_check_statement (self, VALA_CODE_NODE (prop), NULL, t, non_null, var_name);
	}
}


static ValaCCodeStatement* vala_code_generator_create_type_check_statement (ValaCodeGenerator* self, ValaCodeNode* method_node, ValaDataType* ret_type, ValaDataType* t, gboolean non_null, char* var_name)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_CODE_NODE (method_node), NULL);
	g_return_val_if_fail (ret_type == NULL || VALA_IS_DATA_TYPE (ret_type), NULL);
	g_return_val_if_fail (VALA_IS_DATA_TYPE (t), NULL);
	g_return_val_if_fail (var_name != NULL, NULL);
	ValaCCodeFunctionCall* ccheck = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, NULL);
	if (VALA_IS_CLASS (t) || VALA_IS_INTERFACE (t)) {
		ValaCCodeIdentifier* __temp416 = NULL;
		char* __temp415 = NULL;
		ValaCCodeFunctionCall* __temp417 = NULL;
		ValaCCodeFunctionCall* ctype_check = (__temp417 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp416 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp415 = vala_data_type_get_upper_case_cname (t, "IS_")), NULL)), NULL), (__temp416 == NULL ? NULL : (__temp416 = (g_object_unref (__temp416), NULL))), (__temp415 == NULL ? NULL : (__temp415 = (g_free (__temp415), NULL))), __temp417);
		ValaCCodeIdentifier* __temp418 = NULL;
		vala_ccode_function_call_add_argument (ctype_check, VALA_CCODE_EXPRESSION ((__temp418 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", var_name, NULL))));
		(__temp418 == NULL ? NULL : (__temp418 = (g_object_unref (__temp418), NULL)));
		ValaCCodeFunctionCall* __temp419 = NULL;
		ValaCCodeExpression* cexpr = VALA_CCODE_EXPRESSION ((__temp419 = ctype_check, (__temp419 == NULL ? __temp419 : g_object_ref (__temp419))));
		if (!non_null) {
			ValaCCodeConstant* __temp421 = NULL;
			ValaCCodeIdentifier* __temp420 = NULL;
			ValaCCodeBinaryExpression* __temp422 = NULL;
			ValaCCodeBinaryExpression* cnull = (__temp422 = g_object_new (VALA_TYPE_CCODE_BINARY_EXPRESSION, "operator", VALA_CCODE_BINARY_OPERATOR_EQUALITY, "left", (__temp420 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", var_name, NULL)), "right", (__temp421 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL)), NULL), (__temp421 == NULL ? NULL : (__temp421 = (g_object_unref (__temp421), NULL))), (__temp420 == NULL ? NULL : (__temp420 = (g_object_unref (__temp420), NULL))), __temp422);
			ValaCCodeExpression* __temp423 = NULL;
			cexpr = (__temp423 = VALA_CCODE_EXPRESSION (g_object_new (VALA_TYPE_CCODE_BINARY_EXPRESSION, "operator", VALA_CCODE_BINARY_OPERATOR_OR, "left", cnull, "right", ctype_check, NULL)), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp423);
			(cnull == NULL ? NULL : (cnull = (g_object_unref (cnull), NULL)));
		}
		vala_ccode_function_call_add_argument (ccheck, cexpr);
		(ctype_check == NULL ? NULL : (ctype_check = (g_object_unref (ctype_check), NULL)));
		(cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL)));
	} else if (!non_null) {
		gpointer __temp424;
		return (__temp424 = NULL, (ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL))), __temp424);
	} else {
		ValaCCodeConstant* __temp426 = NULL;
		ValaCCodeIdentifier* __temp425 = NULL;
		ValaCCodeBinaryExpression* __temp427 = NULL;
		ValaCCodeBinaryExpression* cnonnull = (__temp427 = g_object_new (VALA_TYPE_CCODE_BINARY_EXPRESSION, "operator", VALA_CCODE_BINARY_OPERATOR_INEQUALITY, "left", (__temp425 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", var_name, NULL)), "right", (__temp426 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL)), NULL), (__temp426 == NULL ? NULL : (__temp426 = (g_object_unref (__temp426), NULL))), (__temp425 == NULL ? NULL : (__temp425 = (g_object_unref (__temp425), NULL))), __temp427);
		vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION (cnonnull));
		(cnonnull == NULL ? NULL : (cnonnull = (g_object_unref (cnonnull), NULL)));
	}
	if (ret_type == NULL) {
		/* void function */
		ValaCCodeIdentifier* __temp428 = NULL;
		vala_ccode_function_call_set_call (ccheck, VALA_CCODE_EXPRESSION ((__temp428 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_return_if_fail", NULL))));
		(__temp428 == NULL ? NULL : (__temp428 = (g_object_unref (__temp428), NULL)));
	} else {
		ValaCCodeIdentifier* __temp429 = NULL;
		vala_ccode_function_call_set_call (ccheck, VALA_CCODE_EXPRESSION ((__temp429 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_return_val_if_fail", NULL))));
		(__temp429 == NULL ? NULL : (__temp429 = (g_object_unref (__temp429), NULL)));
		if (vala_data_type_is_reference_type (ret_type)) {
			ValaCCodeConstant* __temp430 = NULL;
			vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION ((__temp430 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL))));
			(__temp430 == NULL ? NULL : (__temp430 = (g_object_unref (__temp430), NULL)));
		} else if (g_utf8_collate (vala_data_type_get_name (ret_type), "bool") == 0) {
			ValaCCodeConstant* __temp431 = NULL;
			vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION ((__temp431 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "FALSE", NULL))));
			(__temp431 == NULL ? NULL : (__temp431 = (g_object_unref (__temp431), NULL)));
		} else if (g_utf8_collate (vala_data_type_get_name (ret_type), "int") == 0 || VALA_IS_ENUM (ret_type) || VALA_IS_FLAGS (ret_type)) {
			ValaCCodeConstant* __temp432 = NULL;
			vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION ((__temp432 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "0", NULL))));
			(__temp432 == NULL ? NULL : (__temp432 = (g_object_unref (__temp432), NULL)));
		} else {
			vala_report_error (vala_code_node_get_source_reference (method_node), "not supported return type for runtime type checks");
			gpointer __temp433;
			return (__temp433 = NULL, (ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL))), __temp433);
		}
	}
	ValaCCodeExpressionStatement* __temp434 = NULL;
	return (__temp434 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", ccheck, NULL), (ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL))), __temp434);
	(ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL)));
}


static ValaDataType* vala_code_generator_find_parent_type (ValaCodeGenerator* self, ValaCodeNode* node)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (node == NULL || VALA_IS_CODE_NODE (node), NULL);
	ValaSymbol* sym = vala_code_node_get_symbol (node);
	while (sym != NULL) {
		if (VALA_IS_DATA_TYPE (vala_symbol_get_node (sym))) {
			return VALA_DATA_TYPE (vala_symbol_get_node (sym));
		}
		sym = sym->parent_symbol;
	}
	return NULL;
}


static void vala_code_generator_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp438 = NULL;
	ValaSymbol* __temp437 = NULL;
	self->priv->current_symbol = (__temp438 = (__temp437 = self->priv->current_symbol->parent_symbol, (__temp437 == NULL ? __temp437 : g_object_ref (__temp437))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp438);
	if (g_utf8_collate (vala_method_get_name (m), "init") == 0) {
		return;
	}
	ValaCCodeFunction* __temp441 = NULL;
	char* __temp440 = NULL;
	char* __temp439 = NULL;
	self->priv->function = (__temp441 = g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", (__temp439 = vala_method_get_real_cname (m)), "return_type", (__temp440 = vala_type_reference_get_cname (vala_method_get_return_type (m), FALSE)), NULL), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp441);
	(__temp440 == NULL ? NULL : (__temp440 = (g_free (__temp440), NULL)));
	(__temp439 == NULL ? NULL : (__temp439 = (g_free (__temp439), NULL)));
	ValaCCodeFunctionDeclarator* vdeclarator = NULL;
	ValaCCodeFormalParameter* instance_param = NULL;
	if (vala_method_get_instance (m)) {
		ValaTypeReference* this_type = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL);
		this_type->type = vala_code_generator_find_parent_type (self, VALA_CODE_NODE (m));
		if (!vala_method_get_overrides (m)) {
			ValaCCodeFormalParameter* __temp443 = NULL;
			char* __temp442 = NULL;
			instance_param = (__temp443 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", (__temp442 = vala_type_reference_get_cname (this_type, FALSE)), "name", "self", NULL), (instance_param == NULL ? NULL : (instance_param = (g_object_unref (instance_param), NULL))), __temp443);
			(__temp442 == NULL ? NULL : (__temp442 = (g_free (__temp442), NULL)));
		} else {
			ValaTypeReference* base_type = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL);
			base_type->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_base_method (m)))->parent_symbol));
			ValaCCodeFormalParameter* __temp445 = NULL;
			char* __temp444 = NULL;
			instance_param = (__temp445 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", (__temp444 = vala_type_reference_get_cname (base_type, FALSE)), "name", "base", NULL), (instance_param == NULL ? NULL : (instance_param = (g_object_unref (instance_param), NULL))), __temp445);
			(__temp444 == NULL ? NULL : (__temp444 = (g_free (__temp444), NULL)));
			(base_type == NULL ? NULL : (base_type = (g_object_unref (base_type), NULL)));
		}
		if (!vala_method_get_instance_last (m)) {
			vala_ccode_function_add_parameter (self->priv->function, instance_param);
		}
		if (vala_method_get_is_abstract (m) || vala_method_get_is_virtual (m)) {
			char* __temp446 = NULL;
			ValaCCodeDeclaration* __temp447 = NULL;
			ValaCCodeDeclaration* vdecl = (__temp447 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp446 = vala_type_reference_get_cname (vala_method_get_return_type (m), FALSE)), NULL), (__temp446 == NULL ? NULL : (__temp446 = (g_free (__temp446), NULL))), __temp447);
			ValaCCodeFunctionDeclarator* __temp448 = NULL;
			vdeclarator = (__temp448 = g_object_new (VALA_TYPE_CCODE_FUNCTION_DECLARATOR, "name", vala_method_get_name (m), NULL), (vdeclarator == NULL ? NULL : (vdeclarator = (g_object_unref (vdeclarator), NULL))), __temp448);
			vala_ccode_declaration_add_declarator (vdecl, VALA_CCODE_DECLARATOR (vdeclarator));
			vala_ccode_struct_add_declaration (self->priv->type_struct, vdecl);
			vala_ccode_function_declarator_add_parameter (vdeclarator, instance_param);
			(vdecl == NULL ? NULL : (vdecl = (g_object_unref (vdecl), NULL)));
		}
		(this_type == NULL ? NULL : (this_type = (g_object_unref (this_type), NULL)));
	}
	GList* params = vala_method_get_parameters (m);
	{
		GList * param_it;
		for (param_it = params; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				vala_ccode_function_add_parameter (self->priv->function, VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode (VALA_CODE_NODE (param))));
				if (vdeclarator != NULL) {
					vala_ccode_function_declarator_add_parameter (vdeclarator, VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode (VALA_CODE_NODE (param))));
				}
			}
		}
	}
	if (vala_method_get_instance (m) && vala_method_get_instance_last (m)) {
		vala_ccode_function_add_parameter (self->priv->function, instance_param);
	}
	/* real function declaration and definition not needed
	 * for abstract methods */
	if (!vala_method_get_is_abstract (m)) {
		if (m->access == VALA_MEMBER_ACCESSIBILITY_PUBLIC && !(vala_method_get_is_virtual (m) || vala_method_get_overrides (m))) {
			/* public methods need function declaration in
			 * header file except virtual/overridden methods */
			ValaCCodeFunction* __temp449 = NULL;
			vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp449 = vala_ccode_function_copy (self->priv->function))));
			(__temp449 == NULL ? NULL : (__temp449 = (g_object_unref (__temp449), NULL)));
		} else {
			/* declare all other functions in source file to
			 * avoid dependency on order within source file */
			vala_ccode_function_set_modifiers (self->priv->function, VALA_CCODE_MODIFIERS_STATIC);
			ValaCCodeFunction* __temp450 = NULL;
			vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE ((__temp450 = vala_ccode_function_copy (self->priv->function))));
			(__temp450 == NULL ? NULL : (__temp450 = (g_object_unref (__temp450), NULL)));
		}
		/* Methods imported from a plain C file don't
		 * have a body, e.g. Vala.Parser.parse_file () */
		if (vala_method_get_body (m) != NULL) {
			vala_ccode_function_set_block (self->priv->function, VALA_CCODE_BLOCK (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_method_get_body (m)))));
			ValaCCodeFragment* cinit = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
			vala_ccode_block_prepend_statement (vala_ccode_function_get_block (self->priv->function), VALA_CCODE_NODE (cinit));
			if (VALA_IS_CLASS (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (m))->parent_symbol))) {
				ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (m))->parent_symbol));
				if (vala_method_get_overrides (m)) {
					ValaCCodeIdentifier* __temp452 = NULL;
					char* __temp451 = NULL;
					ValaCCodeFunctionCall* __temp453 = NULL;
					ValaCCodeFunctionCall* ccall = (__temp453 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp452 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp451 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)), NULL)), NULL), (__temp452 == NULL ? NULL : (__temp452 = (g_object_unref (__temp452), NULL))), (__temp451 == NULL ? NULL : (__temp451 = (g_free (__temp451), NULL))), __temp453);
					ValaCCodeIdentifier* __temp454 = NULL;
					vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp454 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "base", NULL))));
					(__temp454 == NULL ? NULL : (__temp454 = (g_object_unref (__temp454), NULL)));
					char* __temp455 = NULL;
					ValaCCodeDeclaration* __temp456 = NULL;
					ValaCCodeDeclaration* cdecl = (__temp456 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp455 = g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL), (__temp455 == NULL ? NULL : (__temp455 = (g_free (__temp455), NULL))), __temp456);
					ValaCCodeVariableDeclarator* __temp457 = NULL;
					vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp457 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", "self", "initializer", ccall, NULL))));
					(__temp457 == NULL ? NULL : (__temp457 = (g_object_unref (__temp457), NULL)));
					vala_ccode_fragment_append (cinit, VALA_CCODE_NODE (cdecl));
					(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
					(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
				} else if (vala_method_get_instance (m)) {
					ValaCCodeStatement* __temp458 = NULL;
					vala_ccode_fragment_append (cinit, VALA_CCODE_NODE ((__temp458 = vala_code_generator_create_method_type_check_statement (self, m, VALA_DATA_TYPE (cl), TRUE, "self"))));
					(__temp458 == NULL ? NULL : (__temp458 = (g_object_unref (__temp458), NULL)));
				}
			}
			{
				GList * param_it;
				for (param_it = vala_method_get_parameters (m); param_it != NULL; param_it = param_it->next) {
					ValaFormalParameter* param = param_it->data;
					{
						ValaDataType* t = vala_formal_parameter_get_type_reference (param)->type;
						if (t != NULL && vala_data_type_is_reference_type (t) && !vala_type_reference_get_is_out (vala_formal_parameter_get_type_reference (param))) {
							ValaCCodeStatement* type_check = vala_code_generator_create_method_type_check_statement (self, m, t, vala_type_reference_get_non_null (vala_formal_parameter_get_type_reference (param)), vala_formal_parameter_get_name (param));
							if (type_check != NULL) {
								vala_ccode_fragment_append (cinit, VALA_CCODE_NODE (type_check));
							}
							(type_check == NULL ? NULL : (type_check = (g_object_unref (type_check), NULL)));
						}
					}
				}
			}
			if (vala_code_node_get_source_reference (VALA_CODE_NODE (m)) != NULL && vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (m))) != NULL) {
				ValaCCodeComment* __temp459 = NULL;
				vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE ((__temp459 = g_object_new (VALA_TYPE_CCODE_COMMENT, "text", vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (m))), NULL))));
				(__temp459 == NULL ? NULL : (__temp459 = (g_object_unref (__temp459), NULL)));
			}
			vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (self->priv->function));
			(cinit == NULL ? NULL : (cinit = (g_object_unref (cinit), NULL)));
		}
	}
	if (vala_method_get_is_abstract (m) || vala_method_get_is_virtual (m)) {
		char* __temp460 = NULL;
		ValaCCodeFunction* __temp461 = NULL;
		ValaCCodeFunction* vfunc = (__temp461 = g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", vala_method_get_cname (m), "return_type", (__temp460 = vala_type_reference_get_cname (vala_method_get_return_type (m), FALSE)), NULL), (__temp460 == NULL ? NULL : (__temp460 = (g_free (__temp460), NULL))), __temp461);
		ValaTypeReference* this_type = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL);
		this_type->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (m))->parent_symbol));
		char* __temp462 = NULL;
		ValaCCodeFormalParameter* __temp463 = NULL;
		ValaCCodeFormalParameter* cparam = (__temp463 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", (__temp462 = vala_type_reference_get_cname (this_type, FALSE)), "name", "self", NULL), (__temp462 == NULL ? NULL : (__temp462 = (g_free (__temp462), NULL))), __temp463);
		vala_ccode_function_add_parameter (vfunc, cparam);
		ValaCCodeBlock* vblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
		ValaCCodeFunctionCall* vcast = NULL;
		if (VALA_IS_INTERFACE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (m))->parent_symbol))) {
			ValaInterface* iface = VALA_INTERFACE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (m))->parent_symbol));
			ValaCCodeFunctionCall* __temp467 = NULL;
			ValaCCodeIdentifier* __temp466 = NULL;
			char* __temp465 = NULL;
			char* __temp464 = NULL;
			vcast = (__temp467 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp466 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp465 = g_strdup_printf ("%s_GET_INTERFACE", (__temp464 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), NULL)))), NULL)), NULL), (vcast == NULL ? NULL : (vcast = (g_object_unref (vcast), NULL))), __temp467);
			(__temp466 == NULL ? NULL : (__temp466 = (g_object_unref (__temp466), NULL)));
			(__temp465 == NULL ? NULL : (__temp465 = (g_free (__temp465), NULL)));
			(__temp464 == NULL ? NULL : (__temp464 = (g_free (__temp464), NULL)));
		} else {
			ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (m))->parent_symbol));
			ValaCCodeFunctionCall* __temp471 = NULL;
			ValaCCodeIdentifier* __temp470 = NULL;
			char* __temp469 = NULL;
			char* __temp468 = NULL;
			vcast = (__temp471 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp470 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp469 = g_strdup_printf ("%s_GET_CLASS", (__temp468 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), NULL)), NULL), (vcast == NULL ? NULL : (vcast = (g_object_unref (vcast), NULL))), __temp471);
			(__temp470 == NULL ? NULL : (__temp470 = (g_object_unref (__temp470), NULL)));
			(__temp469 == NULL ? NULL : (__temp469 = (g_free (__temp469), NULL)));
			(__temp468 == NULL ? NULL : (__temp468 = (g_free (__temp468), NULL)));
		}
		ValaCCodeIdentifier* __temp472 = NULL;
		vala_ccode_function_call_add_argument (vcast, VALA_CCODE_EXPRESSION ((__temp472 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL))));
		(__temp472 == NULL ? NULL : (__temp472 = (g_object_unref (__temp472), NULL)));
		ValaCCodeMemberAccess* __temp473 = NULL;
		ValaCCodeFunctionCall* __temp474 = NULL;
		ValaCCodeFunctionCall* vcall = (__temp474 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp473 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", vcast, "member_name", vala_method_get_name (m), "is_pointer", TRUE, NULL)), NULL), (__temp473 == NULL ? NULL : (__temp473 = (g_object_unref (__temp473), NULL))), __temp474);
		ValaCCodeIdentifier* __temp475 = NULL;
		vala_ccode_function_call_add_argument (vcall, VALA_CCODE_EXPRESSION ((__temp475 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL))));
		(__temp475 == NULL ? NULL : (__temp475 = (g_object_unref (__temp475), NULL)));
		GList* params = vala_method_get_parameters (m);
		{
			GList * param_it;
			for (param_it = params; param_it != NULL; param_it = param_it->next) {
				ValaFormalParameter* param = param_it->data;
				{
					vala_ccode_function_add_parameter (vfunc, VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode (VALA_CODE_NODE (param))));
					ValaCCodeIdentifier* __temp476 = NULL;
					vala_ccode_function_call_add_argument (vcall, VALA_CCODE_EXPRESSION ((__temp476 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_formal_parameter_get_name (param), NULL))));
					(__temp476 == NULL ? NULL : (__temp476 = (g_object_unref (__temp476), NULL)));
				}
			}
		}
		ValaCCodeExpressionStatement* __temp477 = NULL;
		vala_ccode_block_add_statement (vblock, VALA_CCODE_NODE ((__temp477 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", vcall, NULL))));
		(__temp477 == NULL ? NULL : (__temp477 = (g_object_unref (__temp477), NULL)));
		ValaCCodeFunction* __temp478 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp478 = vala_ccode_function_copy (vfunc))));
		(__temp478 == NULL ? NULL : (__temp478 = (g_object_unref (__temp478), NULL)));
		vala_ccode_function_set_block (vfunc, vblock);
		vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (vfunc));
		(vfunc == NULL ? NULL : (vfunc = (g_object_unref (vfunc), NULL)));
		(this_type == NULL ? NULL : (this_type = (g_object_unref (this_type), NULL)));
		(cparam == NULL ? NULL : (cparam = (g_object_unref (cparam), NULL)));
		(vblock == NULL ? NULL : (vblock = (g_object_unref (vblock), NULL)));
		(vcast == NULL ? NULL : (vcast = (g_object_unref (vcast), NULL)));
		(vcall == NULL ? NULL : (vcall = (g_object_unref (vcall), NULL)));
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
	}
	if (g_utf8_collate (vala_method_get_name (m), "main") == 0) {
		ValaCCodeFunction* cmain = g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", "main", "return_type", "int", NULL);
		ValaCCodeFormalParameter* __temp479 = NULL;
		vala_ccode_function_add_parameter (cmain, (__temp479 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", "int", "name", "argc", NULL)));
		(__temp479 == NULL ? NULL : (__temp479 = (g_object_unref (__temp479), NULL)));
		ValaCCodeFormalParameter* __temp480 = NULL;
		vala_ccode_function_add_parameter (cmain, (__temp480 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", "char **", "name", "argv", NULL)));
		(__temp480 == NULL ? NULL : (__temp480 = (g_object_unref (__temp480), NULL)));
		ValaCCodeBlock* main_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
		ValaCCodeExpressionStatement* __temp483 = NULL;
		ValaCCodeFunctionCall* __temp482 = NULL;
		ValaCCodeIdentifier* __temp481 = NULL;
		vala_ccode_block_add_statement (main_block, VALA_CCODE_NODE ((__temp483 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp482 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp481 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_type_init", NULL)), NULL)), NULL))));
		(__temp483 == NULL ? NULL : (__temp483 = (g_object_unref (__temp483), NULL)));
		(__temp482 == NULL ? NULL : (__temp482 = (g_object_unref (__temp482), NULL)));
		(__temp481 == NULL ? NULL : (__temp481 = (g_object_unref (__temp481), NULL)));
		ValaCCodeIdentifier* __temp484 = NULL;
		ValaCCodeFunctionCall* __temp485 = NULL;
		ValaCCodeFunctionCall* main_call = (__temp485 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp484 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_ccode_function_get_name (self->priv->function), NULL)), NULL), (__temp484 == NULL ? NULL : (__temp484 = (g_object_unref (__temp484), NULL))), __temp485);
		ValaCCodeIdentifier* __temp486 = NULL;
		vala_ccode_function_call_add_argument (main_call, VALA_CCODE_EXPRESSION ((__temp486 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "argc", NULL))));
		(__temp486 == NULL ? NULL : (__temp486 = (g_object_unref (__temp486), NULL)));
		ValaCCodeIdentifier* __temp487 = NULL;
		vala_ccode_function_call_add_argument (main_call, VALA_CCODE_EXPRESSION ((__temp487 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "argv", NULL))));
		(__temp487 == NULL ? NULL : (__temp487 = (g_object_unref (__temp487), NULL)));
		ValaCCodeReturnStatement* __temp488 = NULL;
		vala_ccode_block_add_statement (main_block, VALA_CCODE_NODE ((__temp488 = g_object_new (VALA_TYPE_CCODE_RETURN_STATEMENT, "return_expression", main_call, NULL))));
		(__temp488 == NULL ? NULL : (__temp488 = (g_object_unref (__temp488), NULL)));
		vala_ccode_function_set_block (cmain, main_block);
		vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (cmain));
		(cmain == NULL ? NULL : (cmain = (g_object_unref (cmain), NULL)));
		(main_block == NULL ? NULL : (main_block = (g_object_unref (main_block), NULL)));
		(main_call == NULL ? NULL : (main_call = (g_object_unref (main_call), NULL)));
	}
	(vdeclarator == NULL ? NULL : (vdeclarator = (g_object_unref (vdeclarator), NULL)));
	(instance_param == NULL ? NULL : (instance_param = (g_object_unref (instance_param), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
}


static void vala_code_generator_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_FORMAL_PARAMETER (p));
	if (!vala_formal_parameter_get_ellipsis (p)) {
		ValaCCodeFormalParameter* __temp490 = NULL;
		char* __temp489 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (p), VALA_CCODE_NODE ((__temp490 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", (__temp489 = vala_type_reference_get_cname (vala_formal_parameter_get_type_reference (p), FALSE)), "name", vala_formal_parameter_get_name (p), NULL))));
		(__temp490 == NULL ? NULL : (__temp490 = (g_object_unref (__temp490), NULL)));
		(__temp489 == NULL ? NULL : (__temp489 = (g_free (__temp489), NULL)));
	}
}


static void vala_code_generator_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	char* __temp491 = NULL;
	vala_ccode_enum_add_value (self->priv->prop_enum, (__temp491 = vala_property_get_upper_case_cname (prop)), NULL);
	(__temp491 == NULL ? NULL : (__temp491 = (g_free (__temp491), NULL)));
}


static void vala_code_generator_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (acc))->parent_symbol));
	ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (prop))->parent_symbol));
	if (vala_property_accessor_get_readable (acc)) {
		ValaCCodeFunction* __temp495 = NULL;
		char* __temp494 = NULL;
		char* __temp493 = NULL;
		char* __temp492 = NULL;
		self->priv->function = (__temp495 = g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", (__temp493 = g_strdup_printf ("%s_get_%s", (__temp492 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop))), "return_type", (__temp494 = vala_type_reference_get_cname (vala_property_get_type_reference (prop), FALSE)), NULL), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp495);
		(__temp494 == NULL ? NULL : (__temp494 = (g_free (__temp494), NULL)));
		(__temp493 == NULL ? NULL : (__temp493 = (g_free (__temp493), NULL)));
		(__temp492 == NULL ? NULL : (__temp492 = (g_free (__temp492), NULL)));
	} else {
		ValaCCodeFunction* __temp498 = NULL;
		char* __temp497 = NULL;
		char* __temp496 = NULL;
		self->priv->function = (__temp498 = g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", (__temp497 = g_strdup_printf ("%s_set_%s", (__temp496 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop))), "return_type", "void", NULL), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp498);
		(__temp497 == NULL ? NULL : (__temp497 = (g_free (__temp497), NULL)));
		(__temp496 == NULL ? NULL : (__temp496 = (g_free (__temp496), NULL)));
	}
	ValaTypeReference* this_type = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL);
	this_type->type = VALA_DATA_TYPE (cl);
	char* __temp499 = NULL;
	ValaCCodeFormalParameter* __temp500 = NULL;
	ValaCCodeFormalParameter* cparam = (__temp500 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", (__temp499 = vala_type_reference_get_cname (this_type, FALSE)), "name", "self", NULL), (__temp499 == NULL ? NULL : (__temp499 = (g_free (__temp499), NULL))), __temp500);
	vala_ccode_function_add_parameter (self->priv->function, cparam);
	if (vala_property_accessor_get_writable (acc) || vala_property_accessor_get_construct_ (acc)) {
		ValaCCodeFormalParameter* __temp502 = NULL;
		char* __temp501 = NULL;
		vala_ccode_function_add_parameter (self->priv->function, (__temp502 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "type_name", (__temp501 = vala_type_reference_get_cname (vala_property_get_type_reference (prop), FALSE)), "name", "value", NULL)));
		(__temp502 == NULL ? NULL : (__temp502 = (g_object_unref (__temp502), NULL)));
		(__temp501 == NULL ? NULL : (__temp501 = (g_free (__temp501), NULL)));
	}
	ValaCCodeFunction* __temp503 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp503 = vala_ccode_function_copy (self->priv->function))));
	(__temp503 == NULL ? NULL : (__temp503 = (g_object_unref (__temp503), NULL)));
	if (vala_property_accessor_get_body (acc) != NULL) {
		vala_ccode_function_set_block (self->priv->function, VALA_CCODE_BLOCK (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_property_accessor_get_body (acc)))));
		ValaCCodeStatement* __temp504 = NULL;
		vala_ccode_block_prepend_statement (vala_ccode_function_get_block (self->priv->function), VALA_CCODE_NODE ((__temp504 = vala_code_generator_create_property_type_check_statement (self, prop, vala_property_accessor_get_readable (acc), VALA_DATA_TYPE (cl), TRUE, "self"))));
		(__temp504 == NULL ? NULL : (__temp504 = (g_object_unref (__temp504), NULL)));
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (self->priv->function));
	(this_type == NULL ? NULL : (this_type = (g_object_unref (this_type), NULL)));
	(cparam == NULL ? NULL : (cparam = (g_object_unref (cparam), NULL)));
}


static void vala_code_generator_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (c))->parent_symbol));
	ValaCCodeFunction* __temp507 = NULL;
	char* __temp506 = NULL;
	char* __temp505 = NULL;
	self->priv->function = (__temp507 = g_object_new (VALA_TYPE_CCODE_FUNCTION, "name", (__temp506 = g_strdup_printf ("%s_constructor", (__temp505 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)))), "return_type", "GObject *", NULL), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp507);
	(__temp506 == NULL ? NULL : (__temp506 = (g_free (__temp506), NULL)));
	(__temp505 == NULL ? NULL : (__temp505 = (g_free (__temp505), NULL)));
	vala_ccode_function_set_modifiers (self->priv->function, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeFormalParameter* __temp508 = NULL;
	vala_ccode_function_add_parameter (self->priv->function, (__temp508 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "name", "type", "type_name", "GType", NULL)));
	(__temp508 == NULL ? NULL : (__temp508 = (g_object_unref (__temp508), NULL)));
	ValaCCodeFormalParameter* __temp509 = NULL;
	vala_ccode_function_add_parameter (self->priv->function, (__temp509 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "name", "n_construct_properties", "type_name", "guint", NULL)));
	(__temp509 == NULL ? NULL : (__temp509 = (g_object_unref (__temp509), NULL)));
	ValaCCodeFormalParameter* __temp510 = NULL;
	vala_ccode_function_add_parameter (self->priv->function, (__temp510 = g_object_new (VALA_TYPE_CCODE_FORMAL_PARAMETER, "name", "construct_properties", "type_name", "GObjectConstructParam *", NULL)));
	(__temp510 == NULL ? NULL : (__temp510 = (g_object_unref (__temp510), NULL)));
	ValaCCodeFunction* __temp511 = NULL;
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE ((__temp511 = vala_ccode_function_copy (self->priv->function))));
	(__temp511 == NULL ? NULL : (__temp511 = (g_object_unref (__temp511), NULL)));
	ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeDeclaration* cdecl = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", "GObject *", NULL);
	ValaCCodeVariableDeclarator* __temp512 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp512 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", "obj", NULL))));
	(__temp512 == NULL ? NULL : (__temp512 = (g_object_unref (__temp512), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeDeclaration* __temp514 = NULL;
	char* __temp513 = NULL;
	cdecl = (__temp514 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp513 = g_strdup_printf ("%sClass *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp514);
	(__temp513 == NULL ? NULL : (__temp513 = (g_free (__temp513), NULL)));
	ValaCCodeVariableDeclarator* __temp515 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp515 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", "klass", NULL))));
	(__temp515 == NULL ? NULL : (__temp515 = (g_object_unref (__temp515), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeDeclaration* __temp516 = NULL;
	cdecl = (__temp516 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", "GObjectClass *", NULL), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp516);
	ValaCCodeVariableDeclarator* __temp517 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp517 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", "parent_class", NULL))));
	(__temp517 == NULL ? NULL : (__temp517 = (g_object_unref (__temp517), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeIdentifier* __temp518 = NULL;
	ValaCCodeFunctionCall* __temp519 = NULL;
	ValaCCodeFunctionCall* ccall = (__temp519 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp518 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_type_class_peek", NULL)), NULL), (__temp518 == NULL ? NULL : (__temp518 = (g_object_unref (__temp518), NULL))), __temp519);
	ValaCCodeIdentifier* __temp521 = NULL;
	char* __temp520 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp521 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp520 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), NULL))));
	(__temp521 == NULL ? NULL : (__temp521 = (g_object_unref (__temp521), NULL)));
	(__temp520 == NULL ? NULL : (__temp520 = (g_free (__temp520), NULL)));
	ValaCCodeIdentifier* __temp524 = NULL;
	char* __temp523 = NULL;
	char* __temp522 = NULL;
	ValaCCodeFunctionCall* __temp525 = NULL;
	ValaCCodeFunctionCall* ccast = (__temp525 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp524 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp523 = g_strdup_printf ("%s_CLASS", (__temp522 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), NULL)), NULL), (__temp524 == NULL ? NULL : (__temp524 = (g_object_unref (__temp524), NULL))), (__temp523 == NULL ? NULL : (__temp523 = (g_free (__temp523), NULL))), (__temp522 == NULL ? NULL : (__temp522 = (g_free (__temp522), NULL))), __temp525);
	vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeExpressionStatement* __temp528 = NULL;
	ValaCCodeAssignment* __temp527 = NULL;
	ValaCCodeIdentifier* __temp526 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp528 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp527 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp526 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "klass", NULL)), "right", ccast, NULL)), NULL))));
	(__temp528 == NULL ? NULL : (__temp528 = (g_object_unref (__temp528), NULL)));
	(__temp527 == NULL ? NULL : (__temp527 = (g_object_unref (__temp527), NULL)));
	(__temp526 == NULL ? NULL : (__temp526 = (g_object_unref (__temp526), NULL)));
	ValaCCodeFunctionCall* __temp530 = NULL;
	ValaCCodeIdentifier* __temp529 = NULL;
	ccall = (__temp530 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp529 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_type_class_peek_parent", NULL)), NULL), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp530);
	(__temp529 == NULL ? NULL : (__temp529 = (g_object_unref (__temp529), NULL)));
	ValaCCodeIdentifier* __temp531 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp531 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "klass", NULL))));
	(__temp531 == NULL ? NULL : (__temp531 = (g_object_unref (__temp531), NULL)));
	ValaCCodeFunctionCall* __temp533 = NULL;
	ValaCCodeIdentifier* __temp532 = NULL;
	ccast = (__temp533 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp532 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "G_OBJECT_CLASS", NULL)), NULL), (ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL))), __temp533);
	(__temp532 == NULL ? NULL : (__temp532 = (g_object_unref (__temp532), NULL)));
	vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeExpressionStatement* __temp536 = NULL;
	ValaCCodeAssignment* __temp535 = NULL;
	ValaCCodeIdentifier* __temp534 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp536 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp535 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp534 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "parent_class", NULL)), "right", ccast, NULL)), NULL))));
	(__temp536 == NULL ? NULL : (__temp536 = (g_object_unref (__temp536), NULL)));
	(__temp535 == NULL ? NULL : (__temp535 = (g_object_unref (__temp535), NULL)));
	(__temp534 == NULL ? NULL : (__temp534 = (g_object_unref (__temp534), NULL)));
	ValaCCodeFunctionCall* __temp539 = NULL;
	ValaCCodeMemberAccess* __temp538 = NULL;
	ValaCCodeIdentifier* __temp537 = NULL;
	ccall = (__temp539 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp538 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", (__temp537 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "parent_class", NULL)), "member_name", "constructor", "is_pointer", TRUE, NULL)), NULL), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp539);
	(__temp538 == NULL ? NULL : (__temp538 = (g_object_unref (__temp538), NULL)));
	(__temp537 == NULL ? NULL : (__temp537 = (g_object_unref (__temp537), NULL)));
	ValaCCodeIdentifier* __temp540 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp540 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "type", NULL))));
	(__temp540 == NULL ? NULL : (__temp540 = (g_object_unref (__temp540), NULL)));
	ValaCCodeIdentifier* __temp541 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp541 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "n_construct_properties", NULL))));
	(__temp541 == NULL ? NULL : (__temp541 = (g_object_unref (__temp541), NULL)));
	ValaCCodeIdentifier* __temp542 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp542 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "construct_properties", NULL))));
	(__temp542 == NULL ? NULL : (__temp542 = (g_object_unref (__temp542), NULL)));
	ValaCCodeExpressionStatement* __temp545 = NULL;
	ValaCCodeAssignment* __temp544 = NULL;
	ValaCCodeIdentifier* __temp543 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp545 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp544 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp543 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "obj", NULL)), "right", ccall, NULL)), NULL))));
	(__temp545 == NULL ? NULL : (__temp545 = (g_object_unref (__temp545), NULL)));
	(__temp544 == NULL ? NULL : (__temp544 = (g_object_unref (__temp544), NULL)));
	(__temp543 == NULL ? NULL : (__temp543 = (g_object_unref (__temp543), NULL)));
	ValaCCodeFunctionCall* __temp548 = NULL;
	ValaCCodeIdentifier* __temp547 = NULL;
	char* __temp546 = NULL;
	ccall = (__temp548 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp547 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp546 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)), NULL)), NULL), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp548);
	(__temp547 == NULL ? NULL : (__temp547 = (g_object_unref (__temp547), NULL)));
	(__temp546 == NULL ? NULL : (__temp546 = (g_free (__temp546), NULL)));
	ValaCCodeIdentifier* __temp549 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp549 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "obj", NULL))));
	(__temp549 == NULL ? NULL : (__temp549 = (g_object_unref (__temp549), NULL)));
	ValaCCodeDeclaration* __temp551 = NULL;
	char* __temp550 = NULL;
	cdecl = (__temp551 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp550 = g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), NULL), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp551);
	(__temp550 == NULL ? NULL : (__temp550 = (g_free (__temp550), NULL)));
	ValaCCodeVariableDeclarator* __temp552 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp552 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", "self", "initializer", ccall, NULL))));
	(__temp552 == NULL ? NULL : (__temp552 = (g_object_unref (__temp552), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	vala_ccode_block_add_statement (cblock, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_constructor_get_body (c))));
	ValaCCodeReturnStatement* __temp554 = NULL;
	ValaCCodeIdentifier* __temp553 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp554 = g_object_new (VALA_TYPE_CCODE_RETURN_STATEMENT, "return_expression", (__temp553 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "obj", NULL)), NULL))));
	(__temp554 == NULL ? NULL : (__temp554 = (g_object_unref (__temp554), NULL)));
	(__temp553 == NULL ? NULL : (__temp553 = (g_object_unref (__temp553), NULL)));
	vala_ccode_function_set_block (self->priv->function, cblock);
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (c))) != NULL) {
		ValaCCodeComment* __temp555 = NULL;
		vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE ((__temp555 = g_object_new (VALA_TYPE_CCODE_COMMENT, "text", vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (c))), NULL))));
		(__temp555 == NULL ? NULL : (__temp555 = (g_object_unref (__temp555), NULL)));
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (self->priv->function));
	(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
}


static void vala_code_generator_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	ValaSymbol* __temp557 = NULL;
	ValaSymbol* __temp556 = NULL;
	self->priv->current_symbol = (__temp557 = (__temp556 = vala_code_node_get_symbol (VALA_CODE_NODE (b)), (__temp556 == NULL ? __temp556 : g_object_ref (__temp556))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp557);
}


static void vala_code_generator_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	GList* local_vars = vala_block_get_local_variables (b);
	{
		GList * decl_it;
		for (decl_it = local_vars; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				vala_code_node_get_symbol (VALA_CODE_NODE (decl))->active = FALSE;
			}
		}
	}
	ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	{
		GList * stmt_it;
		for (stmt_it = vala_block_get_statements (b); stmt_it != NULL; stmt_it = stmt_it->next) {
			ValaStatement* stmt = stmt_it->data;
			{
				ValaSourceReference* src = vala_code_node_get_source_reference (VALA_CODE_NODE (stmt));
				if (src != NULL && vala_source_reference_get_comment (src) != NULL) {
					ValaCCodeComment* __temp558 = NULL;
					vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp558 = g_object_new (VALA_TYPE_CCODE_COMMENT, "text", vala_source_reference_get_comment (src), NULL))));
					(__temp558 == NULL ? NULL : (__temp558 = (g_object_unref (__temp558), NULL)));
				}
				if (VALA_IS_CCODE_FRAGMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)))) {
					{
						GList * cstmt_it;
						for (cstmt_it = vala_ccode_fragment_get_children ((VALA_CCODE_FRAGMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt))))); cstmt_it != NULL; cstmt_it = cstmt_it->next) {
							ValaCCodeStatement* cstmt = cstmt_it->data;
							{
								vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cstmt));
							}
						}
					}
				} else {
					vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)))));
				}
			}
		}
	}
	if (vala_code_generator_get_memory_management (self)) {
		{
			GList * decl_it;
			for (decl_it = local_vars; decl_it != NULL; decl_it = decl_it->next) {
				ValaVariableDeclarator* decl = decl_it->data;
				{
					if (vala_data_type_is_reference_type (decl->type_reference->type) && vala_type_reference_get_is_lvalue_ref (decl->type_reference)) {
						ValaCCodeExpressionStatement* __temp561 = NULL;
						ValaCCodeExpression* __temp560 = NULL;
						ValaCCodeIdentifier* __temp559 = NULL;
						vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp561 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp560 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp559 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (decl), NULL))), decl->type_reference)), NULL))));
						(__temp561 == NULL ? NULL : (__temp561 = (g_object_unref (__temp561), NULL)));
						(__temp560 == NULL ? NULL : (__temp560 = (g_object_unref (__temp560), NULL)));
						(__temp559 == NULL ? NULL : (__temp559 = (g_object_unref (__temp559), NULL)));
					}
				}
			}
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (b), VALA_CCODE_NODE (cblock));
	ValaSymbol* __temp563 = NULL;
	ValaSymbol* __temp562 = NULL;
	self->priv->current_symbol = (__temp563 = (__temp562 = self->priv->current_symbol->parent_symbol, (__temp562 == NULL ? __temp562 : g_object_ref (__temp562))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp563);
	(local_vars == NULL ? NULL : (local_vars = (g_list_free (local_vars), NULL)));
	(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
}


static void vala_code_generator_real_visit_empty_statement (ValaCodeVisitor* base, ValaEmptyStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_EMPTY_STATEMENT (stmt));
	ValaCCodeEmptyStatement* __temp564 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp564 = g_object_new (VALA_TYPE_CCODE_EMPTY_STATEMENT, NULL))));
	(__temp564 == NULL ? NULL : (__temp564 = (g_object_unref (__temp564), NULL)));
}


static void vala_code_generator_real_visit_declaration_statement (ValaCodeVisitor* base, ValaDeclarationStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_DECLARATION_STATEMENT (stmt));
	/* split declaration statement as var declarators
	 * might have different types */
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	{
		GList * decl_it;
		for (decl_it = vala_local_variable_declaration_get_variable_declarators (vala_declaration_statement_get_declaration (stmt)); decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				char* __temp565 = NULL;
				ValaCCodeDeclaration* __temp566 = NULL;
				ValaCCodeDeclaration* cdecl = (__temp566 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp565 = vala_type_reference_get_cname (decl->type_reference, FALSE)), NULL), (__temp565 == NULL ? NULL : (__temp565 = (g_free (__temp565), NULL))), __temp566);
				vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR (VALA_CCODE_VARIABLE_DECLARATOR (vala_code_node_get_ccodenode (VALA_CODE_NODE (decl)))));
				vala_ccode_fragment_append (cfrag, VALA_CCODE_NODE (cdecl));
				(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
			}
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfrag));
	{
		GList * decl_it;
		for (decl_it = vala_local_variable_declaration_get_variable_declarators (vala_declaration_statement_get_declaration (stmt)); decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				if (vala_variable_declarator_get_initializer (decl) != NULL) {
					vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_variable_declarator_get_initializer (decl)->temp_vars);
				}
			}
		}
	}
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static void vala_code_generator_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_VARIABLE_DECLARATOR (decl));
	ValaCCodeExpression* rhs = NULL;
	if (vala_variable_declarator_get_initializer (decl) != NULL) {
		ValaCCodeExpression* __temp568 = NULL;
		ValaCCodeExpression* __temp567 = NULL;
		rhs = (__temp568 = (__temp567 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_variable_declarator_get_initializer (decl)))), (__temp567 == NULL ? __temp567 : g_object_ref (__temp567))), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp568);
		if (decl->type_reference->type != NULL && vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl))->type != NULL && vala_data_type_is_reference_type (decl->type_reference->type) && vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl))->type != decl->type_reference->type) {
			ValaCCodeExpression* __temp569 = NULL;
			rhs = (__temp569 = VALA_CCODE_EXPRESSION (g_object_new (VALA_TYPE_INSTANCE_CAST, "type_reference", decl->type_reference->type, "inner", rhs, NULL)), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp569);
		}
	} else if (decl->type_reference->type != NULL && vala_data_type_is_reference_type (decl->type_reference->type)) {
		ValaCCodeExpression* __temp570 = NULL;
		rhs = (__temp570 = VALA_CCODE_EXPRESSION (g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL)), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp570);
	}
	ValaCCodeVariableDeclarator* __temp571 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (decl), VALA_CCODE_NODE ((__temp571 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", vala_variable_declarator_get_name (decl), "initializer", rhs, NULL))));
	(__temp571 == NULL ? NULL : (__temp571 = (g_object_unref (__temp571), NULL)));
	vala_code_node_get_symbol (VALA_CODE_NODE (decl))->active = TRUE;
	(rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL)));
}


static void vala_code_generator_real_visit_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INITIALIZER_LIST (list));
	ValaCCodeInitializerList* clist = g_object_new (VALA_TYPE_CCODE_INITIALIZER_LIST, NULL);
	{
		GList * expr_it;
		for (expr_it = vala_initializer_list_get_initializers (list); expr_it != NULL; expr_it = expr_it->next) {
			ValaExpression* expr = expr_it->data;
			{
				vala_ccode_initializer_list_append (clist, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))));
			}
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (list), VALA_CCODE_NODE (clist));
	(clist == NULL ? NULL : (clist = (g_object_unref (clist), NULL)));
}


static ValaVariableDeclarator* vala_code_generator_get_temp_variable_declarator (ValaCodeGenerator* self, ValaTypeReference* type)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), NULL);
	char* __temp572 = NULL;
	ValaVariableDeclarator* __temp573 = NULL;
	ValaVariableDeclarator* decl = (__temp573 = g_object_new (VALA_TYPE_VARIABLE_DECLARATOR, "name", (__temp572 = g_strdup_printf ("__temp%d", self->priv->next_temp_var_id)), NULL), (__temp572 == NULL ? NULL : (__temp572 = (g_free (__temp572), NULL))), __temp573);
	ValaTypeReference* __temp574 = NULL;
	decl->type_reference = (__temp574 = vala_type_reference_copy (type), (decl->type_reference == NULL ? NULL : (decl->type_reference = (g_object_unref (decl->type_reference), NULL))), __temp574);
	vala_type_reference_set_is_ref (decl->type_reference, FALSE);
	vala_type_reference_set_is_out (decl->type_reference, FALSE);
	self->priv->next_temp_var_id++;
	ValaVariableDeclarator* __temp575 = NULL;
	ValaVariableDeclarator* __temp576 = NULL;
	return (__temp576 = (__temp575 = decl, (__temp575 == NULL ? __temp575 : g_object_ref (__temp575))), (decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL))), __temp576);
	(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
}


static ValaCCodeExpression* vala_code_generator_get_unref_expression (ValaCodeGenerator* self, ValaCCodeExpression* cvar, ValaTypeReference* type)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_CCODE_EXPRESSION (cvar), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), NULL);
	/* (foo == NULL ? NULL : foo = (unref (foo), NULL)) 
	 can be simplified to
	 * foo = (unref (foo), NULL)
	 * if foo is of static type non-null
	 */
	ValaCCodeConstant* __temp577 = NULL;
	ValaCCodeBinaryExpression* __temp578 = NULL;
	ValaCCodeBinaryExpression* cisnull = (__temp578 = g_object_new (VALA_TYPE_CCODE_BINARY_EXPRESSION, "operator", VALA_CCODE_BINARY_OPERATOR_EQUALITY, "left", cvar, "right", (__temp577 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL)), NULL), (__temp577 == NULL ? NULL : (__temp577 = (g_object_unref (__temp577), NULL))), __temp578);
	char* unref_function = NULL;
	if (vala_data_type_is_reference_counting (type->type)) {
		unref_function = vala_data_type_get_unref_function (type->type);
	} else {
		unref_function = vala_data_type_get_free_function (type->type);
	}
	if (VALA_IS_ARRAY (type->type) && g_utf8_collate (vala_data_type_get_name (vala_array_get_element_type ((VALA_ARRAY (type->type)))), "string") == 0) {
		unref_function = "g_strfreev";
	}
	ValaCCodeIdentifier* __temp579 = NULL;
	ValaCCodeFunctionCall* __temp580 = NULL;
	ValaCCodeFunctionCall* ccall = (__temp580 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp579 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", unref_function, NULL)), NULL), (__temp579 == NULL ? NULL : (__temp579 = (g_object_unref (__temp579), NULL))), __temp580);
	vala_ccode_function_call_add_argument (ccall, cvar);
	/* set freed references to NULL to prevent further use */
	ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
	if (g_utf8_collate (unref_function, "g_list_free") == 0) {
		gboolean is_ref = FALSE;
		gboolean is_class = FALSE;
		GList* type_args = vala_type_reference_get_type_arguments (type);
		{
			GList * type_arg_it;
			for (type_arg_it = type_args; type_arg_it != NULL; type_arg_it = type_arg_it->next) {
				ValaTypeReference* type_arg = type_arg_it->data;
				{
					is_ref = vala_type_reference_get_is_ref (type_arg);
					is_class = VALA_IS_CLASS (type_arg->type);
				}
			}
		}
		if (is_ref) {
			ValaCCodeIdentifier* __temp581 = NULL;
			ValaCCodeFunctionCall* __temp582 = NULL;
			ValaCCodeFunctionCall* cunrefcall = (__temp582 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp581 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_list_foreach", NULL)), NULL), (__temp581 == NULL ? NULL : (__temp581 = (g_object_unref (__temp581), NULL))), __temp582);
			vala_ccode_function_call_add_argument (cunrefcall, cvar);
			if (is_class) {
				ValaCCodeIdentifier* __temp583 = NULL;
				vala_ccode_function_call_add_argument (cunrefcall, VALA_CCODE_EXPRESSION ((__temp583 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "(GFunc) g_object_unref", NULL))));
				(__temp583 == NULL ? NULL : (__temp583 = (g_object_unref (__temp583), NULL)));
			} else {
				ValaCCodeIdentifier* __temp584 = NULL;
				vala_ccode_function_call_add_argument (cunrefcall, VALA_CCODE_EXPRESSION ((__temp584 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "(GFunc) g_free", NULL))));
				(__temp584 == NULL ? NULL : (__temp584 = (g_object_unref (__temp584), NULL)));
			}
			ValaCCodeConstant* __temp585 = NULL;
			vala_ccode_function_call_add_argument (cunrefcall, VALA_CCODE_EXPRESSION ((__temp585 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL))));
			(__temp585 == NULL ? NULL : (__temp585 = (g_object_unref (__temp585), NULL)));
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (cunrefcall));
			(cunrefcall == NULL ? NULL : (cunrefcall = (g_object_unref (cunrefcall), NULL)));
		}
		(type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL)));
	} else if (g_utf8_collate (unref_function, "g_string_free") == 0) {
		ValaCCodeConstant* __temp586 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp586 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "TRUE", NULL))));
		(__temp586 == NULL ? NULL : (__temp586 = (g_object_unref (__temp586), NULL)));
	}
	vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeConstant* __temp587 = NULL;
	vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp587 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL))));
	(__temp587 == NULL ? NULL : (__temp587 = (g_object_unref (__temp587), NULL)));
	ValaCCodeAssignment* cassign = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", cvar, "right", ccomma, NULL);
	ValaCCodeParenthesizedExpression* __temp589 = NULL;
	ValaCCodeConstant* __temp588 = NULL;
	ValaCCodeConditionalExpression* __temp590 = NULL;
	ValaCCodeConditionalExpression* __temp591 = NULL;
	return (__temp591 = (__temp590 = g_object_new (VALA_TYPE_CCODE_CONDITIONAL_EXPRESSION, "condition", cisnull, "true_expression", (__temp588 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL)), "false_expression", (__temp589 = g_object_new (VALA_TYPE_CCODE_PARENTHESIZED_EXPRESSION, "inner", cassign, NULL)), NULL), (__temp589 == NULL ? NULL : (__temp589 = (g_object_unref (__temp589), NULL))), (__temp588 == NULL ? NULL : (__temp588 = (g_object_unref (__temp588), NULL))), __temp590), (cisnull == NULL ? NULL : (cisnull = (g_object_unref (cisnull), NULL))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), (ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL))), (cassign == NULL ? NULL : (cassign = (g_object_unref (cassign), NULL))), __temp591);
	(cisnull == NULL ? NULL : (cisnull = (g_object_unref (cisnull), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
	(cassign == NULL ? NULL : (cassign = (g_object_unref (cassign), NULL)));
}


static void vala_code_generator_real_visit_end_full_expression (ValaCodeVisitor* base, ValaExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_EXPRESSION (expr));
	if (!vala_code_generator_get_memory_management (self)) {
		GList* __temp592 = NULL;
		self->priv->temp_vars = (__temp592 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp592);
		GList* __temp593 = NULL;
		self->priv->temp_ref_vars = (__temp593 = NULL, (self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL))), __temp593);
		return;
	}
	/* expr is a full expression, i.e. an initializer, the
	 * expression in an expression statement, the controlling
	 * expression in if, while, for, or foreach statements
	 *
	 * we unref temporary variables at the end of a full
	 * expression
	 
	 can't automatically deep copy lists yet, so do it
	 * manually for now
	 * replace with
	 * expr.temp_vars = temp_vars;
	 * when deep list copying works
	 */
	GList* __temp594 = NULL;
	expr->temp_vars = (__temp594 = NULL, (expr->temp_vars == NULL ? NULL : (expr->temp_vars = (g_list_foreach (expr->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (expr->temp_vars), NULL))), __temp594);
	{
		GList * decl1_it;
		for (decl1_it = self->priv->temp_vars; decl1_it != NULL; decl1_it = decl1_it->next) {
			ValaVariableDeclarator* decl1 = decl1_it->data;
			{
				ValaVariableDeclarator* __temp595 = NULL;
				expr->temp_vars = g_list_append (expr->temp_vars, (__temp595 = decl1, (__temp595 == NULL ? __temp595 : g_object_ref (__temp595))));
			}
		}
	}
	GList* __temp596 = NULL;
	self->priv->temp_vars = (__temp596 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp596);
	if (self->priv->temp_ref_vars == NULL) {
		return;
	}
	/* nothing to do without temporary variables */
	ValaVariableDeclarator* full_expr_decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (expr));
	ValaVariableDeclarator* __temp597 = NULL;
	expr->temp_vars = g_list_append (expr->temp_vars, (__temp597 = full_expr_decl, (__temp597 == NULL ? __temp597 : g_object_ref (__temp597))));
	ValaCCodeCommaExpression* expr_list = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
	ValaCCodeAssignment* __temp599 = NULL;
	ValaCCodeIdentifier* __temp598 = NULL;
	vala_ccode_comma_expression_append_expression (expr_list, VALA_CCODE_EXPRESSION ((__temp599 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp598 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (full_expr_decl), NULL)), "right", vala_code_node_get_ccodenode (VALA_CODE_NODE (expr)), NULL))));
	(__temp599 == NULL ? NULL : (__temp599 = (g_object_unref (__temp599), NULL)));
	(__temp598 == NULL ? NULL : (__temp598 = (g_object_unref (__temp598), NULL)));
	{
		GList * decl_it;
		for (decl_it = self->priv->temp_ref_vars; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				ValaCCodeExpression* __temp601 = NULL;
				ValaCCodeIdentifier* __temp600 = NULL;
				vala_ccode_comma_expression_append_expression (expr_list, (__temp601 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp600 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (decl), NULL))), decl->type_reference)));
				(__temp601 == NULL ? NULL : (__temp601 = (g_object_unref (__temp601), NULL)));
				(__temp600 == NULL ? NULL : (__temp600 = (g_object_unref (__temp600), NULL)));
			}
		}
	}
	ValaCCodeIdentifier* __temp602 = NULL;
	vala_ccode_comma_expression_append_expression (expr_list, VALA_CCODE_EXPRESSION ((__temp602 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (full_expr_decl), NULL))));
	(__temp602 == NULL ? NULL : (__temp602 = (g_object_unref (__temp602), NULL)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (expr_list));
	GList* __temp603 = NULL;
	self->priv->temp_ref_vars = (__temp603 = NULL, (self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL))), __temp603);
	(full_expr_decl == NULL ? NULL : (full_expr_decl = (g_object_unref (full_expr_decl), NULL)));
	(expr_list == NULL ? NULL : (expr_list = (g_object_unref (expr_list), NULL)));
}


static void vala_code_generator_append_temp_decl (ValaCodeGenerator* self, ValaCCodeFragment* cfrag, GList* temp_vars)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CCODE_FRAGMENT (cfrag));
	{
		GList * decl_it;
		for (decl_it = temp_vars; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				char* __temp604 = NULL;
				ValaCCodeDeclaration* __temp605 = NULL;
				ValaCCodeDeclaration* cdecl = (__temp605 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp604 = vala_type_reference_get_cname (decl->type_reference, TRUE)), NULL), (__temp604 == NULL ? NULL : (__temp604 = (g_free (__temp604), NULL))), __temp605);
				ValaCCodeVariableDeclarator* vardecl = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", vala_variable_declarator_get_name (decl), NULL);
				vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR (vardecl));
				if (decl->type_reference->type != NULL && vala_data_type_is_reference_type (decl->type_reference->type)) {
					ValaCCodeConstant* __temp606 = NULL;
					vala_ccode_variable_declarator_set_initializer (vardecl, VALA_CCODE_EXPRESSION ((__temp606 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL))));
					(__temp606 == NULL ? NULL : (__temp606 = (g_object_unref (__temp606), NULL)));
				}
				vala_ccode_fragment_append (cfrag, VALA_CCODE_NODE (cdecl));
				(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
				(vardecl == NULL ? NULL : (vardecl = (g_object_unref (vardecl), NULL)));
			}
		}
	}
}


static void vala_code_generator_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_EXPRESSION_STATEMENT (stmt));
	ValaCCodeExpressionStatement* __temp607 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp607 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_expression_statement_get_expression (stmt)))), NULL))));
	(__temp607 == NULL ? NULL : (__temp607 = (g_object_unref (__temp607), NULL)));
	/* free temporary objects */
	if (!vala_code_generator_get_memory_management (self)) {
		GList* __temp608 = NULL;
		self->priv->temp_vars = (__temp608 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp608);
		GList* __temp609 = NULL;
		self->priv->temp_ref_vars = (__temp609 = NULL, (self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL))), __temp609);
		return;
	}
	if (self->priv->temp_vars == NULL) {
		return;
	}
	/* nothing to do without temporary variables */
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	vala_code_generator_append_temp_decl (self, cfrag, self->priv->temp_vars);
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)));
	{
		GList * decl_it;
		for (decl_it = self->priv->temp_ref_vars; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				ValaCCodeExpressionStatement* __temp612 = NULL;
				ValaCCodeExpression* __temp611 = NULL;
				ValaCCodeIdentifier* __temp610 = NULL;
				vala_ccode_fragment_append (cfrag, VALA_CCODE_NODE ((__temp612 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp611 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp610 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (decl), NULL))), decl->type_reference)), NULL))));
				(__temp612 == NULL ? NULL : (__temp612 = (g_object_unref (__temp612), NULL)));
				(__temp611 == NULL ? NULL : (__temp611 = (g_object_unref (__temp611), NULL)));
				(__temp610 == NULL ? NULL : (__temp610 = (g_object_unref (__temp610), NULL)));
			}
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfrag));
	GList* __temp613 = NULL;
	self->priv->temp_vars = (__temp613 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp613);
	GList* __temp614 = NULL;
	self->priv->temp_ref_vars = (__temp614 = NULL, (self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL))), __temp614);
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static void vala_code_generator_create_temp_decl (ValaCodeGenerator* self, ValaStatement* stmt, GList* temp_vars)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_STATEMENT (stmt));
	/* declare temporary variables */
	if (temp_vars == NULL) {
		return;
	}
	/* nothing to do without temporary variables */
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	vala_code_generator_append_temp_decl (self, cfrag, temp_vars);
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfrag));
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static void vala_code_generator_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_IF_STATEMENT (stmt));
	if (vala_if_statement_get_false_statement (stmt) != NULL) {
		ValaCCodeIfStatement* __temp615 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp615 = g_object_new (VALA_TYPE_CCODE_IF_STATEMENT, "condition", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_condition (stmt)))), "true_statement", VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_true_statement (stmt)))), "false_statement", VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_false_statement (stmt)))), NULL))));
		(__temp615 == NULL ? NULL : (__temp615 = (g_object_unref (__temp615), NULL)));
	} else {
		ValaCCodeIfStatement* __temp616 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp616 = g_object_new (VALA_TYPE_CCODE_IF_STATEMENT, "condition", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_condition (stmt)))), "true_statement", VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_true_statement (stmt)))), NULL))));
		(__temp616 == NULL ? NULL : (__temp616 = (g_object_unref (__temp616), NULL)));
	}
	vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_if_statement_get_condition (stmt)->temp_vars);
}


static void vala_code_generator_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_WHILE_STATEMENT (stmt));
	ValaCCodeWhileStatement* __temp617 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp617 = g_object_new (VALA_TYPE_CCODE_WHILE_STATEMENT, "condition", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_while_statement_get_condition (stmt)))), "body", VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_while_statement_get_body (stmt)))), NULL))));
	(__temp617 == NULL ? NULL : (__temp617 = (g_object_unref (__temp617), NULL)));
	vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_while_statement_get_condition (stmt)->temp_vars);
}


static void vala_code_generator_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_FOR_STATEMENT (stmt));
	ValaCCodeForStatement* cfor = g_object_new (VALA_TYPE_CCODE_FOR_STATEMENT, "condition", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_for_statement_get_condition (stmt)))), "body", VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_for_statement_get_body (stmt)))), NULL);
	{
		GList * init_expr_it;
		for (init_expr_it = vala_for_statement_get_initializer (stmt); init_expr_it != NULL; init_expr_it = init_expr_it->next) {
			ValaExpression* init_expr = init_expr_it->data;
			{
				vala_ccode_for_statement_add_initializer (cfor, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (init_expr))));
			}
		}
	}
	{
		GList * it_expr_it;
		for (it_expr_it = vala_for_statement_get_iterator (stmt); it_expr_it != NULL; it_expr_it = it_expr_it->next) {
			ValaExpression* it_expr = it_expr_it->data;
			{
				vala_ccode_for_statement_add_iterator (cfor, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (it_expr))));
			}
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfor));
	vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_for_statement_get_condition (stmt)->temp_vars);
	(cfor == NULL ? NULL : (cfor = (g_object_unref (cfor), NULL)));
}


static void vala_code_generator_real_visit_end_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (stmt));
	ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	if (VALA_IS_ARRAY (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt))->type)) {
		char* it_name = g_strdup_printf ("%s_it", vala_foreach_statement_get_variable_name (stmt));
		char* __temp618 = NULL;
		ValaCCodeDeclaration* __temp619 = NULL;
		ValaCCodeDeclaration* citdecl = (__temp619 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp618 = vala_type_reference_get_cname (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt)), FALSE)), NULL), (__temp618 == NULL ? NULL : (__temp618 = (g_free (__temp618), NULL))), __temp619);
		ValaCCodeVariableDeclarator* __temp620 = NULL;
		vala_ccode_declaration_add_declarator (citdecl, VALA_CCODE_DECLARATOR ((__temp620 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", it_name, NULL))));
		(__temp620 == NULL ? NULL : (__temp620 = (g_object_unref (__temp620), NULL)));
		vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (citdecl));
		ValaCCodeBlock* cbody = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
		char* __temp621 = NULL;
		ValaCCodeDeclaration* __temp622 = NULL;
		ValaCCodeDeclaration* cdecl = (__temp622 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp621 = vala_type_reference_get_cname (vala_foreach_statement_get_type_reference (stmt), FALSE)), NULL), (__temp621 == NULL ? NULL : (__temp621 = (g_free (__temp621), NULL))), __temp622);
		ValaCCodeVariableDeclarator* __temp625 = NULL;
		ValaCCodeIdentifier* __temp624 = NULL;
		char* __temp623 = NULL;
		vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp625 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", vala_foreach_statement_get_variable_name (stmt), "initializer", (__temp624 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp623 = g_strdup_printf ("*%s", it_name)), NULL)), NULL))));
		(__temp625 == NULL ? NULL : (__temp625 = (g_object_unref (__temp625), NULL)));
		(__temp624 == NULL ? NULL : (__temp624 = (g_object_unref (__temp624), NULL)));
		(__temp623 == NULL ? NULL : (__temp623 = (g_free (__temp623), NULL)));
		vala_ccode_block_add_statement (cbody, VALA_CCODE_NODE (cdecl));
		vala_ccode_block_add_statement (cbody, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_foreach_statement_get_body (stmt))));
		ValaCCodeConstant* __temp628 = NULL;
		ValaCCodeIdentifier* __temp627 = NULL;
		char* __temp626 = NULL;
		ValaCCodeBinaryExpression* __temp629 = NULL;
		ValaCCodeBinaryExpression* ccond = (__temp629 = g_object_new (VALA_TYPE_CCODE_BINARY_EXPRESSION, "operator", VALA_CCODE_BINARY_OPERATOR_INEQUALITY, "left", (__temp627 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp626 = g_strdup_printf ("*%s", it_name)), NULL)), "right", (__temp628 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL)), NULL), (__temp628 == NULL ? NULL : (__temp628 = (g_object_unref (__temp628), NULL))), (__temp627 == NULL ? NULL : (__temp627 = (g_object_unref (__temp627), NULL))), (__temp626 == NULL ? NULL : (__temp626 = (g_free (__temp626), NULL))), __temp629);
		ValaCCodeForStatement* cfor = g_object_new (VALA_TYPE_CCODE_FOR_STATEMENT, "condition", ccond, "body", cbody, NULL);
		ValaCCodeAssignment* __temp631 = NULL;
		ValaCCodeIdentifier* __temp630 = NULL;
		vala_ccode_for_statement_add_initializer (cfor, VALA_CCODE_EXPRESSION ((__temp631 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp630 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", it_name, NULL)), "right", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_foreach_statement_get_collection (stmt)))), NULL))));
		(__temp631 == NULL ? NULL : (__temp631 = (g_object_unref (__temp631), NULL)));
		(__temp630 == NULL ? NULL : (__temp630 = (g_object_unref (__temp630), NULL)));
		ValaCCodeAssignment* __temp636 = NULL;
		ValaCCodeBinaryExpression* __temp635 = NULL;
		ValaCCodeConstant* __temp634 = NULL;
		ValaCCodeIdentifier* __temp633 = NULL;
		ValaCCodeIdentifier* __temp632 = NULL;
		vala_ccode_for_statement_add_iterator (cfor, VALA_CCODE_EXPRESSION ((__temp636 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp632 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", it_name, NULL)), "right", (__temp635 = g_object_new (VALA_TYPE_CCODE_BINARY_EXPRESSION, "operator", VALA_CCODE_BINARY_OPERATOR_PLUS, "left", (__temp633 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", it_name, NULL)), "right", (__temp634 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "1", NULL)), NULL)), NULL))));
		(__temp636 == NULL ? NULL : (__temp636 = (g_object_unref (__temp636), NULL)));
		(__temp635 == NULL ? NULL : (__temp635 = (g_object_unref (__temp635), NULL)));
		(__temp634 == NULL ? NULL : (__temp634 = (g_object_unref (__temp634), NULL)));
		(__temp633 == NULL ? NULL : (__temp633 = (g_object_unref (__temp633), NULL)));
		(__temp632 == NULL ? NULL : (__temp632 = (g_object_unref (__temp632), NULL)));
		vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cfor));
		(it_name == NULL ? NULL : (it_name = (g_free (it_name), NULL)));
		(citdecl == NULL ? NULL : (citdecl = (g_object_unref (citdecl), NULL)));
		(cbody == NULL ? NULL : (cbody = (g_object_unref (cbody), NULL)));
		(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
		(ccond == NULL ? NULL : (ccond = (g_object_unref (ccond), NULL)));
		(cfor == NULL ? NULL : (cfor = (g_object_unref (cfor), NULL)));
	} else if (g_utf8_collate (vala_data_type_get_name (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt))->type), "List") == 0) {
		char* it_name = g_strdup_printf ("%s_it", vala_foreach_statement_get_variable_name (stmt));
		ValaCCodeDeclaration* citdecl = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", "GList *", NULL);
		ValaCCodeVariableDeclarator* __temp637 = NULL;
		vala_ccode_declaration_add_declarator (citdecl, VALA_CCODE_DECLARATOR ((__temp637 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", it_name, NULL))));
		(__temp637 == NULL ? NULL : (__temp637 = (g_object_unref (__temp637), NULL)));
		vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (citdecl));
		ValaCCodeBlock* cbody = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
		char* __temp638 = NULL;
		ValaCCodeDeclaration* __temp639 = NULL;
		ValaCCodeDeclaration* cdecl = (__temp639 = g_object_new (VALA_TYPE_CCODE_DECLARATION, "type_name", (__temp638 = vala_type_reference_get_cname (vala_foreach_statement_get_type_reference (stmt), FALSE)), NULL), (__temp638 == NULL ? NULL : (__temp638 = (g_free (__temp638), NULL))), __temp639);
		ValaCCodeVariableDeclarator* __temp642 = NULL;
		ValaCCodeMemberAccess* __temp641 = NULL;
		ValaCCodeIdentifier* __temp640 = NULL;
		vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp642 = g_object_new (VALA_TYPE_CCODE_VARIABLE_DECLARATOR, "name", vala_foreach_statement_get_variable_name (stmt), "initializer", (__temp641 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", (__temp640 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", it_name, NULL)), "member_name", "data", "is_pointer", TRUE, NULL)), NULL))));
		(__temp642 == NULL ? NULL : (__temp642 = (g_object_unref (__temp642), NULL)));
		(__temp641 == NULL ? NULL : (__temp641 = (g_object_unref (__temp641), NULL)));
		(__temp640 == NULL ? NULL : (__temp640 = (g_object_unref (__temp640), NULL)));
		vala_ccode_block_add_statement (cbody, VALA_CCODE_NODE (cdecl));
		vala_ccode_block_add_statement (cbody, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_foreach_statement_get_body (stmt))));
		ValaCCodeConstant* __temp644 = NULL;
		ValaCCodeIdentifier* __temp643 = NULL;
		ValaCCodeBinaryExpression* __temp645 = NULL;
		ValaCCodeBinaryExpression* ccond = (__temp645 = g_object_new (VALA_TYPE_CCODE_BINARY_EXPRESSION, "operator", VALA_CCODE_BINARY_OPERATOR_INEQUALITY, "left", (__temp643 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", it_name, NULL)), "right", (__temp644 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL)), NULL), (__temp644 == NULL ? NULL : (__temp644 = (g_object_unref (__temp644), NULL))), (__temp643 == NULL ? NULL : (__temp643 = (g_object_unref (__temp643), NULL))), __temp645);
		ValaCCodeForStatement* cfor = g_object_new (VALA_TYPE_CCODE_FOR_STATEMENT, "condition", ccond, "body", cbody, NULL);
		ValaCCodeAssignment* __temp647 = NULL;
		ValaCCodeIdentifier* __temp646 = NULL;
		vala_ccode_for_statement_add_initializer (cfor, VALA_CCODE_EXPRESSION ((__temp647 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp646 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", it_name, NULL)), "right", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_foreach_statement_get_collection (stmt)))), NULL))));
		(__temp647 == NULL ? NULL : (__temp647 = (g_object_unref (__temp647), NULL)));
		(__temp646 == NULL ? NULL : (__temp646 = (g_object_unref (__temp646), NULL)));
		ValaCCodeAssignment* __temp651 = NULL;
		ValaCCodeMemberAccess* __temp650 = NULL;
		ValaCCodeIdentifier* __temp649 = NULL;
		ValaCCodeIdentifier* __temp648 = NULL;
		vala_ccode_for_statement_add_iterator (cfor, VALA_CCODE_EXPRESSION ((__temp651 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp648 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", it_name, NULL)), "right", (__temp650 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", (__temp649 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", it_name, NULL)), "member_name", "next", "is_pointer", TRUE, NULL)), NULL))));
		(__temp651 == NULL ? NULL : (__temp651 = (g_object_unref (__temp651), NULL)));
		(__temp650 == NULL ? NULL : (__temp650 = (g_object_unref (__temp650), NULL)));
		(__temp649 == NULL ? NULL : (__temp649 = (g_object_unref (__temp649), NULL)));
		(__temp648 == NULL ? NULL : (__temp648 = (g_object_unref (__temp648), NULL)));
		vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cfor));
		(it_name == NULL ? NULL : (it_name = (g_free (it_name), NULL)));
		(citdecl == NULL ? NULL : (citdecl = (g_object_unref (citdecl), NULL)));
		(cbody == NULL ? NULL : (cbody = (g_object_unref (cbody), NULL)));
		(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
		(ccond == NULL ? NULL : (ccond = (g_object_unref (ccond), NULL)));
		(cfor == NULL ? NULL : (cfor = (g_object_unref (cfor), NULL)));
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cblock));
	(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
}


static void vala_code_generator_real_visit_break_statement (ValaCodeVisitor* base, ValaBreakStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BREAK_STATEMENT (stmt));
	ValaCCodeBreakStatement* __temp652 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp652 = g_object_new (VALA_TYPE_CCODE_BREAK_STATEMENT, NULL))));
	(__temp652 == NULL ? NULL : (__temp652 = (g_object_unref (__temp652), NULL)));
}


static void vala_code_generator_real_visit_continue_statement (ValaCodeVisitor* base, ValaContinueStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CONTINUE_STATEMENT (stmt));
	ValaCCodeContinueStatement* __temp653 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp653 = g_object_new (VALA_TYPE_CCODE_CONTINUE_STATEMENT, NULL))));
	(__temp653 == NULL ? NULL : (__temp653 = (g_object_unref (__temp653), NULL)));
}


static void vala_code_generator_append_local_free (ValaCodeGenerator* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (sym == NULL || VALA_IS_SYMBOL (sym));
	g_return_if_fail (cfrag == NULL || VALA_IS_CCODE_FRAGMENT (cfrag));
	ValaBlock* b = VALA_BLOCK (vala_symbol_get_node (sym));
	GList* local_vars = vala_block_get_local_variables (b);
	{
		GList * decl_it;
		for (decl_it = local_vars; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				if (vala_code_node_get_symbol (VALA_CODE_NODE (decl))->active && vala_data_type_is_reference_type (decl->type_reference->type) && vala_type_reference_get_is_lvalue_ref (decl->type_reference)) {
					ValaCCodeExpressionStatement* __temp656 = NULL;
					ValaCCodeExpression* __temp655 = NULL;
					ValaCCodeIdentifier* __temp654 = NULL;
					vala_ccode_fragment_append (cfrag, VALA_CCODE_NODE ((__temp656 = g_object_new (VALA_TYPE_CCODE_EXPRESSION_STATEMENT, "expression", (__temp655 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp654 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (decl), NULL))), decl->type_reference)), NULL))));
					(__temp656 == NULL ? NULL : (__temp656 = (g_object_unref (__temp656), NULL)));
					(__temp655 == NULL ? NULL : (__temp655 = (g_object_unref (__temp655), NULL)));
					(__temp654 == NULL ? NULL : (__temp654 = (g_object_unref (__temp654), NULL)));
				}
			}
		}
	}
	if (VALA_IS_BLOCK (vala_symbol_get_node (sym->parent_symbol))) {
		vala_code_generator_append_local_free (self, sym->parent_symbol, cfrag, stop_at_loop);
	}
	(local_vars == NULL ? NULL : (local_vars = (g_list_free (local_vars), NULL)));
}


static void vala_code_generator_create_local_free (ValaCodeGenerator* self, ValaStatement* stmt)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (stmt == NULL || VALA_IS_STATEMENT (stmt));
	if (!vala_code_generator_get_memory_management (self)) {
		return;
	}
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	vala_code_generator_append_local_free (self, self->priv->current_symbol, cfrag, FALSE);
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfrag));
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static gboolean vala_code_generator_append_local_free_expr (ValaCodeGenerator* self, ValaSymbol* sym, ValaCCodeCommaExpression* ccomma, gboolean stop_at_loop)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), FALSE);
	g_return_val_if_fail (sym == NULL || VALA_IS_SYMBOL (sym), FALSE);
	g_return_val_if_fail (ccomma == NULL || VALA_IS_CCODE_COMMA_EXPRESSION (ccomma), FALSE);
	gboolean found = FALSE;
	ValaBlock* b = VALA_BLOCK (vala_symbol_get_node (sym));
	GList* local_vars = vala_block_get_local_variables (b);
	{
		GList * decl_it;
		for (decl_it = local_vars; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				if (vala_code_node_get_symbol (VALA_CODE_NODE (decl))->active && vala_data_type_is_reference_type (decl->type_reference->type) && vala_type_reference_get_is_lvalue_ref (decl->type_reference)) {
					found = TRUE;
					ValaCCodeExpression* __temp658 = NULL;
					ValaCCodeIdentifier* __temp657 = NULL;
					vala_ccode_comma_expression_append_expression (ccomma, (__temp658 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp657 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (decl), NULL))), decl->type_reference)));
					(__temp658 == NULL ? NULL : (__temp658 = (g_object_unref (__temp658), NULL)));
					(__temp657 == NULL ? NULL : (__temp657 = (g_object_unref (__temp657), NULL)));
				}
			}
		}
	}
	if (VALA_IS_BLOCK (vala_symbol_get_node (sym->parent_symbol))) {
		found = found || vala_code_generator_append_local_free_expr (self, sym->parent_symbol, ccomma, stop_at_loop);
	}
	gboolean __temp659;
	return (__temp659 = found, (local_vars == NULL ? NULL : (local_vars = (g_list_free (local_vars), NULL))), __temp659);
	(local_vars == NULL ? NULL : (local_vars = (g_list_free (local_vars), NULL)));
}


static void vala_code_generator_create_local_free_expr (ValaCodeGenerator* self, ValaExpression* expr)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (expr == NULL || VALA_IS_EXPRESSION (expr));
	if (!vala_code_generator_get_memory_management (self)) {
		return;
	}
	ValaVariableDeclarator* return_expr_decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (expr));
	ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
	ValaCCodeAssignment* __temp661 = NULL;
	ValaCCodeIdentifier* __temp660 = NULL;
	vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp661 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp660 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (return_expr_decl), NULL)), "right", vala_code_node_get_ccodenode (VALA_CODE_NODE (expr)), NULL))));
	(__temp661 == NULL ? NULL : (__temp661 = (g_object_unref (__temp661), NULL)));
	(__temp660 == NULL ? NULL : (__temp660 = (g_object_unref (__temp660), NULL)));
	if (!vala_code_generator_append_local_free_expr (self, self->priv->current_symbol, ccomma, FALSE)) {
		(return_expr_decl == NULL ? NULL : (return_expr_decl = (g_object_unref (return_expr_decl), NULL)));
		(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
		return;
	}
	/* no local variables need to be freed */
	ValaCCodeIdentifier* __temp662 = NULL;
	vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp662 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (return_expr_decl), NULL))));
	(__temp662 == NULL ? NULL : (__temp662 = (g_object_unref (__temp662), NULL)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccomma));
	ValaVariableDeclarator* __temp663 = NULL;
	expr->temp_vars = g_list_append (expr->temp_vars, (__temp663 = return_expr_decl, (__temp663 == NULL ? __temp663 : g_object_ref (__temp663))));
	(return_expr_decl == NULL ? NULL : (return_expr_decl = (g_object_unref (return_expr_decl), NULL)));
	(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
}


static void vala_code_generator_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (stmt));
	if (vala_return_statement_get_return_expression (stmt) == NULL) {
		ValaCCodeReturnStatement* __temp664 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp664 = g_object_new (VALA_TYPE_CCODE_RETURN_STATEMENT, NULL))));
		(__temp664 == NULL ? NULL : (__temp664 = (g_object_unref (__temp664), NULL)));
		vala_code_generator_create_local_free (self, VALA_STATEMENT (stmt));
	} else {
		vala_code_generator_create_local_free_expr (self, vala_return_statement_get_return_expression (stmt));
		ValaCCodeReturnStatement* __temp665 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp665 = g_object_new (VALA_TYPE_CCODE_RETURN_STATEMENT, "return_expression", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_return_statement_get_return_expression (stmt)))), NULL))));
		(__temp665 == NULL ? NULL : (__temp665 = (g_object_unref (__temp665), NULL)));
		vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_return_statement_get_return_expression (stmt)->temp_vars);
	}
}


static void vala_code_generator_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BOOLEAN_LITERAL (expr));
	if (vala_boolean_literal_get_value (expr)) {
		ValaCCodeConstant* __temp666 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp666 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "TRUE", NULL))));
		(__temp666 == NULL ? NULL : (__temp666 = (g_object_unref (__temp666), NULL)));
	} else {
		ValaCCodeConstant* __temp667 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp667 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "FALSE", NULL))));
		(__temp667 == NULL ? NULL : (__temp667 = (g_object_unref (__temp667), NULL)));
	}
}


static void vala_code_generator_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CHARACTER_LITERAL (expr));
	ValaCCodeConstant* __temp668 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp668 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", vala_character_literal_get_value (expr), NULL))));
	(__temp668 == NULL ? NULL : (__temp668 = (g_object_unref (__temp668), NULL)));
}


static void vala_code_generator_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INTEGER_LITERAL (expr));
	ValaCCodeConstant* __temp669 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp669 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", vala_integer_literal_get_value (expr), NULL))));
	(__temp669 == NULL ? NULL : (__temp669 = (g_object_unref (__temp669), NULL)));
}


static void vala_code_generator_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_REAL_LITERAL (expr));
	ValaCCodeConstant* __temp670 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp670 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", vala_real_literal_get_value (expr), NULL))));
	(__temp670 == NULL ? NULL : (__temp670 = (g_object_unref (__temp670), NULL)));
}


static void vala_code_generator_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_STRING_LITERAL (expr));
	ValaCCodeConstant* __temp671 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp671 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", vala_string_literal_get_value (expr), NULL))));
	(__temp671 == NULL ? NULL : (__temp671 = (g_object_unref (__temp671), NULL)));
}


static void vala_code_generator_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_NULL_LITERAL (expr));
	ValaCCodeConstant* __temp672 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp672 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL))));
	(__temp672 == NULL ? NULL : (__temp672 = (g_object_unref (__temp672), NULL)));
}


static void vala_code_generator_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_LITERAL_EXPRESSION (expr));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_literal_expression_get_literal (expr))));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_process_cmember (ValaCodeGenerator* self, ValaMemberAccess* expr, ValaCCodeExpression* pub_inst, ValaDataType* base_type)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (expr));
	g_return_if_fail (pub_inst == NULL || VALA_IS_CCODE_EXPRESSION (pub_inst));
	g_return_if_fail (base_type == NULL || VALA_IS_DATA_TYPE (base_type));
	if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		if (!vala_method_get_overrides (m)) {
			ValaCCodeIdentifier* __temp673 = NULL;
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp673 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_method_get_cname (m), NULL))));
			(__temp673 == NULL ? NULL : (__temp673 = (g_object_unref (__temp673), NULL)));
		} else {
			ValaCCodeIdentifier* __temp674 = NULL;
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp674 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_method_get_cname (vala_method_get_base_method (m)), NULL))));
			(__temp674 == NULL ? NULL : (__temp674 = (g_object_unref (__temp674), NULL)));
		}
	} else if (VALA_IS_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
		ValaField* f = VALA_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		if (vala_field_get_instance (f)) {
			ValaCCodeExpression* typed_inst = NULL;
			if (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (f))->parent_symbol) != base_type) {
				ValaCCodeExpression* __temp677 = NULL;
				ValaCCodeIdentifier* __temp676 = NULL;
				char* __temp675 = NULL;
				typed_inst = (__temp677 = VALA_CCODE_EXPRESSION (g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp676 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp675 = vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (f))->parent_symbol))), NULL)), NULL)), NULL)), (typed_inst == NULL ? NULL : (typed_inst = (g_object_unref (typed_inst), NULL))), __temp677);
				(__temp676 == NULL ? NULL : (__temp676 = (g_object_unref (__temp676), NULL)));
				(__temp675 == NULL ? NULL : (__temp675 = (g_free (__temp675), NULL)));
				vala_ccode_function_call_add_argument ((VALA_CCODE_FUNCTION_CALL (typed_inst)), pub_inst);
			} else {
				ValaCCodeExpression* __temp679 = NULL;
				ValaCCodeExpression* __temp678 = NULL;
				typed_inst = (__temp679 = (__temp678 = pub_inst, (__temp678 == NULL ? __temp678 : g_object_ref (__temp678))), (typed_inst == NULL ? NULL : (typed_inst = (g_object_unref (typed_inst), NULL))), __temp679);
			}
			ValaCCodeExpression* inst = NULL;
			if (f->access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
				ValaCCodeExpression* __temp680 = NULL;
				inst = (__temp680 = VALA_CCODE_EXPRESSION (g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", typed_inst, "member_name", "priv", "is_pointer", TRUE, NULL)), (inst == NULL ? NULL : (inst = (g_object_unref (inst), NULL))), __temp680);
			} else {
				ValaCCodeExpression* __temp682 = NULL;
				ValaCCodeExpression* __temp681 = NULL;
				inst = (__temp682 = (__temp681 = typed_inst, (__temp681 == NULL ? __temp681 : g_object_ref (__temp681))), (inst == NULL ? NULL : (inst = (g_object_unref (inst), NULL))), __temp682);
			}
			ValaCCodeMemberAccess* __temp683 = NULL;
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp683 = g_object_new (VALA_TYPE_CCODE_MEMBER_ACCESS, "inner", inst, "member_name", vala_field_get_cname (f), "is_pointer", vala_data_type_is_reference_type ((VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (f))->parent_symbol)))), NULL))));
			(__temp683 == NULL ? NULL : (__temp683 = (g_object_unref (__temp683), NULL)));
			(typed_inst == NULL ? NULL : (typed_inst = (g_object_unref (typed_inst), NULL)));
			(inst == NULL ? NULL : (inst = (g_object_unref (inst), NULL)));
		} else {
			if (VALA_IS_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (f))->parent_symbol))) {
				ValaDataType* t = VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (f))->parent_symbol));
				ValaCCodeIdentifier* __temp686 = NULL;
				char* __temp685 = NULL;
				char* __temp684 = NULL;
				vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp686 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp685 = g_strdup_printf ("%s_%s", (__temp684 = vala_data_type_get_lower_case_cname (t, NULL)), vala_field_get_cname (f))), NULL))));
				(__temp686 == NULL ? NULL : (__temp686 = (g_object_unref (__temp686), NULL)));
				(__temp685 == NULL ? NULL : (__temp685 = (g_free (__temp685), NULL)));
				(__temp684 == NULL ? NULL : (__temp684 = (g_free (__temp684), NULL)));
			} else {
				ValaCCodeIdentifier* __temp687 = NULL;
				vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp687 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_field_get_cname (f), NULL))));
				(__temp687 == NULL ? NULL : (__temp687 = (g_object_unref (__temp687), NULL)));
			}
		}
	} else if (VALA_IS_CONSTANT (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
		ValaConstant* c = VALA_CONSTANT (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		ValaCCodeIdentifier* __temp688 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp688 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_constant_get_cname (c), NULL))));
		(__temp688 == NULL ? NULL : (__temp688 = (g_object_unref (__temp688), NULL)));
	} else if (VALA_IS_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
		ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (prop))->parent_symbol));
		ValaCCodeIdentifier* __temp691 = NULL;
		char* __temp690 = NULL;
		char* __temp689 = NULL;
		ValaCCodeFunctionCall* __temp692 = NULL;
		ValaCCodeFunctionCall* ccall = (__temp692 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp691 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp690 = g_strdup_printf ("%s_get_%s", (__temp689 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop))), NULL)), NULL), (__temp691 == NULL ? NULL : (__temp691 = (g_object_unref (__temp691), NULL))), (__temp690 == NULL ? NULL : (__temp690 = (g_free (__temp690), NULL))), (__temp689 == NULL ? NULL : (__temp689 = (g_free (__temp689), NULL))), __temp692);
		/* explicitly use strong reference as ccast
		 * gets unrefed at the end of the inner block
		 */
		ValaCCodeExpression* __temp693 = NULL;
		ValaCCodeExpression* typed_pub_inst = (__temp693 = pub_inst, (__temp693 == NULL ? __temp693 : g_object_ref (__temp693)));
		/* cast if necessary */
		if (prop->no_accessor_method) {
			ValaCCodeIdentifier* __temp694 = NULL;
			ValaCCodeFunctionCall* __temp695 = NULL;
			ValaCCodeFunctionCall* ccast = (__temp695 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp694 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "G_OBJECT", NULL)), NULL), (__temp694 == NULL ? NULL : (__temp694 = (g_object_unref (__temp694), NULL))), __temp695);
			vala_ccode_function_call_add_argument (ccast, pub_inst);
			ValaCCodeExpression* __temp697 = NULL;
			ValaCCodeFunctionCall* __temp696 = NULL;
			typed_pub_inst = (__temp697 = VALA_CCODE_EXPRESSION ((__temp696 = ccast, (__temp696 == NULL ? __temp696 : g_object_ref (__temp696)))), (typed_pub_inst == NULL ? NULL : (typed_pub_inst = (g_object_unref (typed_pub_inst), NULL))), __temp697);
			(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
		} else if (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (prop))->parent_symbol) != base_type) {
			ValaCCodeIdentifier* __temp699 = NULL;
			char* __temp698 = NULL;
			ValaCCodeFunctionCall* __temp700 = NULL;
			ValaCCodeFunctionCall* ccast = (__temp700 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp699 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp698 = vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (prop))->parent_symbol))), NULL)), NULL)), NULL), (__temp699 == NULL ? NULL : (__temp699 = (g_object_unref (__temp699), NULL))), (__temp698 == NULL ? NULL : (__temp698 = (g_free (__temp698), NULL))), __temp700);
			vala_ccode_function_call_add_argument (ccast, pub_inst);
			ValaCCodeExpression* __temp702 = NULL;
			ValaCCodeFunctionCall* __temp701 = NULL;
			typed_pub_inst = (__temp702 = VALA_CCODE_EXPRESSION ((__temp701 = ccast, (__temp701 == NULL ? __temp701 : g_object_ref (__temp701)))), (typed_pub_inst == NULL ? NULL : (typed_pub_inst = (g_object_unref (typed_pub_inst), NULL))), __temp702);
			(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
		}
		vala_ccode_function_call_add_argument (ccall, typed_pub_inst);
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
		(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		(typed_pub_inst == NULL ? NULL : (typed_pub_inst = (g_object_unref (typed_pub_inst), NULL)));
	} else if (VALA_IS_ENUM_VALUE (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
		ValaEnumValue* ev = VALA_ENUM_VALUE (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		ValaCCodeConstant* __temp703 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp703 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", vala_enum_value_get_cname (ev), NULL))));
		(__temp703 == NULL ? NULL : (__temp703 = (g_object_unref (__temp703), NULL)));
	} else if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
		ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		ValaCCodeIdentifier* __temp704 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp704 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (decl), NULL))));
		(__temp704 == NULL ? NULL : (__temp704 = (g_object_unref (__temp704), NULL)));
	} else if (VALA_IS_FORMAL_PARAMETER (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
		ValaFormalParameter* p = VALA_FORMAL_PARAMETER (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		if (g_utf8_collate (vala_formal_parameter_get_name (p), "this") == 0) {
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (pub_inst));
		} else {
			if (vala_type_reference_get_is_out (vala_formal_parameter_get_type_reference (p)) || vala_type_reference_get_is_ref (vala_formal_parameter_get_type_reference (p))) {
				ValaCCodeIdentifier* __temp706 = NULL;
				char* __temp705 = NULL;
				vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp706 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp705 = g_strdup_printf ("*%s", vala_formal_parameter_get_name (p))), NULL))));
				(__temp706 == NULL ? NULL : (__temp706 = (g_object_unref (__temp706), NULL)));
				(__temp705 == NULL ? NULL : (__temp705 = (g_free (__temp705), NULL)));
			} else {
				ValaCCodeIdentifier* __temp707 = NULL;
				vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp707 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_formal_parameter_get_name (p), NULL))));
				(__temp707 == NULL ? NULL : (__temp707 = (g_object_unref (__temp707), NULL)));
			}
		}
	} else if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
		ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		ValaCCodeIdentifier* __temp708 = NULL;
		ValaCCodeFunctionCall* __temp709 = NULL;
		ValaCCodeFunctionCall* ccall = (__temp709 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp708 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_signal_emit_by_name", NULL)), NULL), (__temp708 == NULL ? NULL : (__temp708 = (g_object_unref (__temp708), NULL))), __temp709);
		ValaCCodeIdentifier* __temp710 = NULL;
		ValaCCodeFunctionCall* __temp711 = NULL;
		ValaCCodeFunctionCall* ccast = (__temp711 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp710 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "G_OBJECT", NULL)), NULL), (__temp710 == NULL ? NULL : (__temp710 = (g_object_unref (__temp710), NULL))), __temp711);
		vala_ccode_function_call_add_argument (ccast, pub_inst);
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (ccast));
		ValaCCodeConstant* __temp713 = NULL;
		char* __temp712 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp713 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", (__temp712 = g_strdup_printf ("\"%s\"", vala_signal_get_name (sig))), NULL))));
		(__temp713 == NULL ? NULL : (__temp713 = (g_object_unref (__temp713), NULL)));
		(__temp712 == NULL ? NULL : (__temp712 = (g_free (__temp712), NULL)));
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
		(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
	}
}


static void vala_code_generator_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_PARENTHESIZED_EXPRESSION (expr));
	ValaCCodeParenthesizedExpression* __temp714 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp714 = g_object_new (VALA_TYPE_CCODE_PARENTHESIZED_EXPRESSION, "inner", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_parenthesized_expression_get_inner (expr)))), NULL))));
	(__temp714 == NULL ? NULL : (__temp714 = (g_object_unref (__temp714), NULL)));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (expr));
	ValaCCodeExpression* pub_inst = NULL;
	ValaDataType* base_type = NULL;
	if (vala_member_access_get_inner (expr) == NULL) {
		ValaCCodeExpression* __temp715 = NULL;
		pub_inst = (__temp715 = VALA_CCODE_EXPRESSION (g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL)), (pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL))), __temp715);
		if (self->priv->current_type_symbol != NULL) {
			/* base type is available if this is a type method */
			base_type = VALA_DATA_TYPE (vala_symbol_get_node (self->priv->current_type_symbol));
		}
	} else {
		ValaCCodeExpression* __temp717 = NULL;
		ValaCCodeExpression* __temp716 = NULL;
		pub_inst = (__temp717 = (__temp716 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (expr)))), (__temp716 == NULL ? __temp716 : g_object_ref (__temp716))), (pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL))), __temp717);
		if (vala_expression_get_static_type (vala_member_access_get_inner (expr)) != NULL) {
			base_type = vala_expression_get_static_type (vala_member_access_get_inner (expr))->type;
		}
	}
	vala_code_generator_process_cmember (self, expr, pub_inst, base_type);
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
	(pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL)));
}


static void vala_code_generator_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INVOCATION_EXPRESSION (expr));
	ValaCCodeFunctionCall* ccall = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_invocation_expression_get_call (expr)))), NULL);
	ValaMethod* m = NULL;
	GList* params = NULL;
	if (!(VALA_IS_MEMBER_ACCESS (vala_invocation_expression_get_call (expr)))) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "unsupported method invocation");
		(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
		return;
	}
	ValaMemberAccess* ma = VALA_MEMBER_ACCESS (vala_invocation_expression_get_call (expr));
	if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))))) {
		ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))));
		ValaCallback* cb = VALA_CALLBACK (decl->type_reference->type);
		GList* __temp718 = NULL;
		params = (__temp718 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp718);
	} else if (VALA_IS_FORMAL_PARAMETER (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))))) {
		ValaFormalParameter* param = VALA_FORMAL_PARAMETER (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))));
		ValaCallback* cb = VALA_CALLBACK (vala_formal_parameter_get_type_reference (param)->type);
		GList* __temp719 = NULL;
		params = (__temp719 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp719);
	} else if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))))) {
		m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))));
		GList* __temp720 = NULL;
		params = (__temp720 = vala_method_get_parameters (m), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp720);
	} else {
		ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))));
		GList* __temp721 = NULL;
		params = (__temp721 = vala_signal_get_parameters (sig), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp721);
		ValaCCodeFunctionCall* __temp723 = NULL;
		ValaCCodeFunctionCall* __temp722 = NULL;
		ccall = (__temp723 = (__temp722 = VALA_CCODE_FUNCTION_CALL (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_invocation_expression_get_call (expr)))), (__temp722 == NULL ? __temp722 : g_object_ref (__temp722))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp723);
	}
	/* explicitly use strong reference as ccall gets unrefed
	 * at end of inner block
	 */
	ValaCCodeExpression* instance = NULL;
	if (m != NULL && vala_method_get_instance (m)) {
		ValaMethod* base_method = m;
		if (vala_method_get_overrides (m)) {
			base_method = vala_method_get_base_method (m);
		}
		gboolean req_cast = FALSE;
		if (vala_member_access_get_inner (ma) == NULL) {
			ValaCCodeExpression* __temp724 = NULL;
			instance = (__temp724 = VALA_CCODE_EXPRESSION (g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL)), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp724);
			/* require casts for overriden and inherited methods */
			req_cast = vala_method_get_overrides (m) || (vala_code_node_get_symbol (VALA_CODE_NODE (m))->parent_symbol != self->priv->current_type_symbol);
		} else {
			ValaCCodeExpression* __temp726 = NULL;
			ValaCCodeExpression* __temp725 = NULL;
			instance = (__temp726 = (__temp725 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (ma)))), (__temp725 == NULL ? __temp725 : g_object_ref (__temp725))), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp726);
			/* reqiure casts if the type of the used instance is
			 * different than the type which declared the method */
			req_cast = vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (base_method))->parent_symbol) != vala_expression_get_static_type (vala_member_access_get_inner (ma))->type;
		}
		if (req_cast && vala_data_type_is_reference_type ((VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (m))->parent_symbol))))) {
			ValaCCodeIdentifier* __temp728 = NULL;
			char* __temp727 = NULL;
			ValaCCodeFunctionCall* __temp729 = NULL;
			ValaCCodeFunctionCall* ccall = (__temp729 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp728 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp727 = vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (base_method))->parent_symbol))), NULL)), NULL)), NULL), (__temp728 == NULL ? NULL : (__temp728 = (g_object_unref (__temp728), NULL))), (__temp727 == NULL ? NULL : (__temp727 = (g_free (__temp727), NULL))), __temp729);
			vala_ccode_function_call_add_argument (ccall, instance);
			ValaCCodeExpression* __temp731 = NULL;
			ValaCCodeFunctionCall* __temp730 = NULL;
			instance = (__temp731 = VALA_CCODE_EXPRESSION ((__temp730 = ccall, (__temp730 == NULL ? __temp730 : g_object_ref (__temp730)))), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp731);
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		}
		if (!vala_method_get_instance_last (m)) {
			vala_ccode_function_call_add_argument (ccall, instance);
		}
	}
	int i = 1;
	{
		GList * arg_it;
		for (arg_it = vala_invocation_expression_get_argument_list (expr); arg_it != NULL; arg_it = arg_it->next) {
			ValaExpression* arg = arg_it->data;
			{
				/* explicitly use strong reference as ccall gets
				 * unrefed at end of inner block
				 */
				ValaCCodeExpression* __temp732 = NULL;
				ValaCCodeExpression* cexpr = (__temp732 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (arg))), (__temp732 == NULL ? __temp732 : g_object_ref (__temp732)));
				if (params != NULL) {
					ValaFormalParameter* param = VALA_FORMAL_PARAMETER (params->data);
					if (!vala_formal_parameter_get_ellipsis (param) && vala_formal_parameter_get_type_reference (param)->type != NULL && vala_data_type_is_reference_type (vala_formal_parameter_get_type_reference (param)->type) && vala_expression_get_static_type (arg)->type != NULL && vala_formal_parameter_get_type_reference (param)->type != vala_expression_get_static_type (arg)->type) {
						ValaCCodeIdentifier* __temp734 = NULL;
						char* __temp733 = NULL;
						ValaCCodeFunctionCall* __temp735 = NULL;
						ValaCCodeFunctionCall* ccall = (__temp735 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp734 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp733 = vala_data_type_get_upper_case_cname (vala_formal_parameter_get_type_reference (param)->type, NULL)), NULL)), NULL), (__temp734 == NULL ? NULL : (__temp734 = (g_object_unref (__temp734), NULL))), (__temp733 == NULL ? NULL : (__temp733 = (g_free (__temp733), NULL))), __temp735);
						vala_ccode_function_call_add_argument (ccall, cexpr);
						ValaCCodeExpression* __temp737 = NULL;
						ValaCCodeFunctionCall* __temp736 = NULL;
						cexpr = (__temp737 = VALA_CCODE_EXPRESSION ((__temp736 = ccall, (__temp736 == NULL ? __temp736 : g_object_ref (__temp736)))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp737);
						(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
					}
				}
				vala_ccode_function_call_add_argument (ccall, cexpr);
				i++;
				if (params != NULL) {
					GList* __temp739 = NULL;
					GList* __temp738 = NULL;
					params = (__temp739 = (__temp738 = params->next, (__temp738 == NULL ? __temp738 : g_list_copy (__temp738))), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp739);
				}
				(cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL)));
			}
		}
	}
	while (params != NULL) {
		ValaFormalParameter* param = VALA_FORMAL_PARAMETER (params->data);
		if (vala_formal_parameter_get_ellipsis (param)) {
			break;
		}
		if (vala_formal_parameter_get_default_expression (param) == NULL) {
			char* __temp740 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp740 = g_strdup_printf ("no default expression for argument %d", i)));
			(__temp740 == NULL ? NULL : (__temp740 = (g_free (__temp740), NULL)));
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
			(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
			(instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL)));
			return;
		}
		/* evaluate default expression here as the code
		 * generator might not have visited the formal
		 * parameter yet */
		vala_code_node_accept (VALA_CODE_NODE (vala_formal_parameter_get_default_expression (param)), VALA_CODE_VISITOR (self));
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_formal_parameter_get_default_expression (param)))));
		i++;
		GList* __temp742 = NULL;
		GList* __temp741 = NULL;
		params = (__temp742 = (__temp741 = params->next, (__temp741 == NULL ? __temp741 : g_list_copy (__temp741))), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp742);
	}
	if (m != NULL && vala_method_get_instance (m) && vala_method_get_instance_last (m)) {
		vala_ccode_function_call_add_argument (ccall, instance);
	}
	if (m != NULL && vala_method_get_instance (m) && vala_method_get_returns_modified_pointer (m)) {
		ValaCCodeAssignment* __temp743 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp743 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", instance, "right", ccall, NULL))));
		(__temp743 == NULL ? NULL : (__temp743 = (g_object_unref (__temp743), NULL)));
	} else {
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
		vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
	}
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
	(instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL)));
}


static void vala_code_generator_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (expr));
	ValaCCodeElementAccess* __temp744 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp744 = g_object_new (VALA_TYPE_CCODE_ELEMENT_ACCESS, "container", vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_element_access_get_container (expr))), "index", vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_element_access_get_index (expr))), NULL))));
	(__temp744 == NULL ? NULL : (__temp744 = (g_object_unref (__temp744), NULL)));
}


static void vala_code_generator_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_POSTFIX_EXPRESSION (expr));
	if (vala_postfix_expression_get_increment (expr)) {
		ValaCCodeUnaryExpression* __temp745 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp745 = g_object_new (VALA_TYPE_CCODE_UNARY_EXPRESSION, "operator", VALA_CCODE_UNARY_OPERATOR_POSTFIX_INCREMENT, "inner", vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_postfix_expression_get_inner (expr))), NULL))));
		(__temp745 == NULL ? NULL : (__temp745 = (g_object_unref (__temp745), NULL)));
	} else {
		ValaCCodeUnaryExpression* __temp746 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp746 = g_object_new (VALA_TYPE_CCODE_UNARY_EXPRESSION, "operator", VALA_CCODE_UNARY_OPERATOR_POSTFIX_DECREMENT, "inner", vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_postfix_expression_get_inner (expr))), NULL))));
		(__temp746 == NULL ? NULL : (__temp746 = (g_object_unref (__temp746), NULL)));
	}
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static ValaCCodeExpression* vala_code_generator_get_ref_expression (ValaCodeGenerator* self, ValaExpression* expr)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_EXPRESSION (expr), NULL);
	/* (temp = expr, temp == NULL ? temp : ref (temp))
	 *
	 * can be simplified to
	 * ref (expr)
	 * if static type of expr is non-null
	 */
	char* ref_function = NULL;
	if (vala_data_type_is_reference_counting (vala_expression_get_static_type (expr)->type)) {
		ref_function = vala_data_type_get_ref_function (vala_expression_get_static_type (expr)->type);
	} else {
		ref_function = vala_data_type_get_dup_function (vala_expression_get_static_type (expr)->type);
	}
	ValaCCodeIdentifier* __temp747 = NULL;
	ValaCCodeFunctionCall* __temp748 = NULL;
	ValaCCodeFunctionCall* ccall = (__temp748 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp747 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", ref_function, NULL)), NULL), (__temp747 == NULL ? NULL : (__temp747 = (g_object_unref (__temp747), NULL))), __temp748);
	if (vala_type_reference_get_non_null (vala_expression_get_static_type (expr))) {
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))));
		ValaCCodeFunctionCall* __temp749 = NULL;
		ValaCCodeFunctionCall* __temp750 = NULL;
		return (__temp750 = (__temp749 = ccall, (__temp749 == NULL ? __temp749 : g_object_ref (__temp749))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp750);
	} else {
		ValaVariableDeclarator* decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (expr));
		ValaVariableDeclarator* __temp751 = NULL;
		self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp751 = decl, (__temp751 == NULL ? __temp751 : g_object_ref (__temp751))));
		ValaCCodeIdentifier* ctemp = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (decl), NULL);
		ValaCCodeConstant* __temp752 = NULL;
		ValaCCodeBinaryExpression* __temp753 = NULL;
		ValaCCodeBinaryExpression* cisnull = (__temp753 = g_object_new (VALA_TYPE_CCODE_BINARY_EXPRESSION, "operator", VALA_CCODE_BINARY_OPERATOR_EQUALITY, "left", ctemp, "right", (__temp752 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL)), NULL), (__temp752 == NULL ? NULL : (__temp752 = (g_object_unref (__temp752), NULL))), __temp753);
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (ctemp));
		ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
		ValaCCodeAssignment* __temp754 = NULL;
		vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp754 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", ctemp, "right", vala_code_node_get_ccodenode (VALA_CODE_NODE (expr)), NULL))));
		(__temp754 == NULL ? NULL : (__temp754 = (g_object_unref (__temp754), NULL)));
		ValaCCodeConditionalExpression* __temp755 = NULL;
		vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp755 = g_object_new (VALA_TYPE_CCODE_CONDITIONAL_EXPRESSION, "condition", cisnull, "true_expression", ctemp, "false_expression", ccall, NULL))));
		(__temp755 == NULL ? NULL : (__temp755 = (g_object_unref (__temp755), NULL)));
		ValaCCodeCommaExpression* __temp756 = NULL;
		ValaCCodeCommaExpression* __temp757 = NULL;
		return (__temp757 = (__temp756 = ccomma, (__temp756 == NULL ? __temp756 : g_object_ref (__temp756))), (decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL))), (ctemp == NULL ? NULL : (ctemp = (g_object_unref (ctemp), NULL))), (cisnull == NULL ? NULL : (cisnull = (g_object_unref (cisnull), NULL))), (ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL))), __temp757);
		(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
		(ctemp == NULL ? NULL : (ctemp = (g_object_unref (ctemp), NULL)));
		(cisnull == NULL ? NULL : (cisnull = (g_object_unref (cisnull), NULL)));
		(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
	}
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
}


static void vala_code_generator_visit_expression (ValaCodeGenerator* self, ValaExpression* expr)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_EXPRESSION (expr));
	if (vala_expression_get_static_type (expr) != NULL && vala_type_reference_get_is_ref (vala_expression_get_static_type (expr)) && vala_type_reference_get_floating_reference (vala_expression_get_static_type (expr))) {
		/* constructor of GInitiallyUnowned subtype
		 * returns floating reference, sink it
		 */
		ValaCCodeIdentifier* __temp758 = NULL;
		ValaCCodeFunctionCall* __temp759 = NULL;
		ValaCCodeFunctionCall* csink = (__temp759 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp758 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_object_ref_sink", NULL)), NULL), (__temp758 == NULL ? NULL : (__temp758 = (g_object_unref (__temp758), NULL))), __temp759);
		vala_ccode_function_call_add_argument (csink, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))));
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (csink));
		(csink == NULL ? NULL : (csink = (g_object_unref (csink), NULL)));
	}
	if (vala_expression_get_ref_leaked (expr)) {
		ValaVariableDeclarator* decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (expr));
		ValaVariableDeclarator* __temp760 = NULL;
		self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp760 = decl, (__temp760 == NULL ? __temp760 : g_object_ref (__temp760))));
		ValaVariableDeclarator* __temp761 = NULL;
		self->priv->temp_ref_vars = g_list_prepend (self->priv->temp_ref_vars, (__temp761 = decl, (__temp761 == NULL ? __temp761 : g_object_ref (__temp761))));
		ValaCCodeParenthesizedExpression* __temp764 = NULL;
		ValaCCodeAssignment* __temp763 = NULL;
		ValaCCodeIdentifier* __temp762 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp764 = g_object_new (VALA_TYPE_CCODE_PARENTHESIZED_EXPRESSION, "inner", (__temp763 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp762 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (decl), NULL)), "right", vala_code_node_get_ccodenode (VALA_CODE_NODE (expr)), NULL)), NULL))));
		(__temp764 == NULL ? NULL : (__temp764 = (g_object_unref (__temp764), NULL)));
		(__temp763 == NULL ? NULL : (__temp763 = (g_object_unref (__temp763), NULL)));
		(__temp762 == NULL ? NULL : (__temp762 = (g_object_unref (__temp762), NULL)));
		(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
	} else if (vala_expression_get_ref_missing (expr)) {
		ValaCCodeExpression* __temp765 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp765 = vala_code_generator_get_ref_expression (self, expr))));
		(__temp765 == NULL ? NULL : (__temp765 = (g_object_unref (__temp765), NULL)));
	}
}


static void vala_code_generator_real_visit_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_OBJECT_CREATION_EXPRESSION (expr));
	ValaCCodeIdentifier* __temp766 = NULL;
	ValaCCodeFunctionCall* __temp767 = NULL;
	ValaCCodeFunctionCall* ccall = (__temp767 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp766 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "g_object_new", NULL)), NULL), (__temp766 == NULL ? NULL : (__temp766 = (g_object_unref (__temp766), NULL))), __temp767);
	ValaCCodeConstant* __temp769 = NULL;
	char* __temp768 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp769 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", (__temp768 = vala_type_reference_get_upper_case_cname (vala_object_creation_expression_get_type_reference (expr), "TYPE_")), NULL))));
	(__temp769 == NULL ? NULL : (__temp769 = (g_object_unref (__temp769), NULL)));
	(__temp768 == NULL ? NULL : (__temp768 = (g_free (__temp768), NULL)));
	{
		GList * arg_it;
		for (arg_it = vala_object_creation_expression_get_named_argument_list (expr); arg_it != NULL; arg_it = arg_it->next) {
			ValaNamedArgument* arg = arg_it->data;
			{
				ValaCCodeConstant* __temp771 = NULL;
				char* __temp770 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp771 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", (__temp770 = g_strdup_printf ("\"%s\"", vala_named_argument_get_name (arg))), NULL))));
				(__temp771 == NULL ? NULL : (__temp771 = (g_object_unref (__temp771), NULL)));
				(__temp770 == NULL ? NULL : (__temp770 = (g_free (__temp770), NULL)));
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_named_argument_get_argument (arg)))));
			}
		}
	}
	ValaCCodeConstant* __temp772 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp772 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL))));
	(__temp772 == NULL ? NULL : (__temp772 = (g_object_unref (__temp772), NULL)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
}


static void vala_code_generator_real_visit_typeof_expression (ValaCodeVisitor* base, ValaTypeofExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_TYPEOF_EXPRESSION (expr));
	ValaCCodeIdentifier* __temp773 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp773 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_data_type_get_type_id (vala_typeof_expression_get_type_reference (expr)->type), NULL))));
	(__temp773 == NULL ? NULL : (__temp773 = (g_object_unref (__temp773), NULL)));
}


static void vala_code_generator_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_UNARY_EXPRESSION (expr));
	ValaCCodeUnaryOperator op;
	if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_PLUS) {
		op = VALA_CCODE_UNARY_OPERATOR_PLUS;
	} else if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_MINUS) {
		op = VALA_CCODE_UNARY_OPERATOR_MINUS;
	} else if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_LOGICAL_NEGATION) {
		op = VALA_CCODE_UNARY_OPERATOR_LOGICAL_NEGATION;
	} else if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_BITWISE_COMPLEMENT) {
		op = VALA_CCODE_UNARY_OPERATOR_BITWISE_COMPLEMENT;
	} else if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_REF) {
		op = VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF;
	} else if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_OUT) {
		op = VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF;
	}
	ValaCCodeUnaryExpression* __temp774 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp774 = g_object_new (VALA_TYPE_CCODE_UNARY_EXPRESSION, "operator", op, "inner", vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_unary_expression_get_inner (expr))), NULL))));
	(__temp774 == NULL ? NULL : (__temp774 = (g_object_unref (__temp774), NULL)));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CAST_EXPRESSION (expr));
	if (VALA_IS_STRUCT (vala_cast_expression_get_type_reference (expr)->type) || VALA_IS_ENUM (vala_cast_expression_get_type_reference (expr)->type) || VALA_IS_FLAGS (vala_cast_expression_get_type_reference (expr)->type)) {
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_cast_expression_get_inner (expr))));
	} else {
		ValaInstanceCast* __temp775 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp775 = g_object_new (VALA_TYPE_INSTANCE_CAST, "type_reference", vala_cast_expression_get_type_reference (expr)->type, "inner", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_cast_expression_get_inner (expr)))), NULL))));
		(__temp775 == NULL ? NULL : (__temp775 = (g_object_unref (__temp775), NULL)));
	}
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BINARY_EXPRESSION (expr));
	ValaCCodeBinaryExpression* __temp776 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp776 = g_object_new (VALA_TYPE_CCODE_BINARY_EXPRESSION, "operator", vala_binary_expression_get_operator (expr), "left", vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_binary_expression_get_left (expr))), "right", vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_binary_expression_get_right (expr))), NULL))));
	(__temp776 == NULL ? NULL : (__temp776 = (g_object_unref (__temp776), NULL)));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_TYPE_CHECK (expr));
	ValaCCodeIdentifier* __temp778 = NULL;
	char* __temp777 = NULL;
	ValaCCodeFunctionCall* __temp779 = NULL;
	ValaCCodeFunctionCall* ccheck = (__temp779 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp778 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp777 = vala_data_type_get_upper_case_cname (vala_type_check_get_type_reference (expr)->type, "IS_")), NULL)), NULL), (__temp778 == NULL ? NULL : (__temp778 = (g_object_unref (__temp778), NULL))), (__temp777 == NULL ? NULL : (__temp777 = (g_free (__temp777), NULL))), __temp779);
	vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_type_check_get_expression (expr)))));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccheck));
	(ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL)));
}


static void vala_code_generator_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CONDITIONAL_EXPRESSION (expr));
	ValaCCodeConditionalExpression* __temp780 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp780 = g_object_new (VALA_TYPE_CCODE_CONDITIONAL_EXPRESSION, "condition", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_conditional_expression_get_condition (expr)))), "true_expression", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_conditional_expression_get_true_expression (expr)))), "false_expression", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_conditional_expression_get_false_expression (expr)))), NULL))));
	(__temp780 == NULL ? NULL : (__temp780 = (g_object_unref (__temp780), NULL)));
}


static void vala_code_generator_real_visit_end_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_LAMBDA_EXPRESSION (l));
	ValaCCodeIdentifier* __temp781 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (l), VALA_CCODE_NODE ((__temp781 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_method_get_cname (vala_lambda_expression_get_method (l)), NULL))));
	(__temp781 == NULL ? NULL : (__temp781 = (g_object_unref (__temp781), NULL)));
}


static void vala_code_generator_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ASSIGNMENT (a));
	ValaMemberAccess* ma = VALA_MEMBER_ACCESS (vala_assignment_get_left (a));
	if (VALA_IS_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))))) {
		ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))));
		ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (prop))->parent_symbol));
		char* set_func = g_strdup ("g_object_set");
		if (!prop->no_accessor_method) {
			char* __temp783 = NULL;
			char* __temp782 = NULL;
			set_func = (__temp783 = g_strdup_printf ("%s_set_%s", (__temp782 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop)), (set_func == NULL ? NULL : (set_func = (g_free (set_func), NULL))), __temp783);
			(__temp782 == NULL ? NULL : (__temp782 = (g_free (__temp782), NULL)));
		}
		ValaCCodeIdentifier* __temp784 = NULL;
		ValaCCodeFunctionCall* __temp785 = NULL;
		ValaCCodeFunctionCall* ccall = (__temp785 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp784 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", set_func, NULL)), NULL), (__temp784 == NULL ? NULL : (__temp784 = (g_object_unref (__temp784), NULL))), __temp785);
		/* target instance is first argument */
		ValaCCodeExpression* instance = NULL;
		gboolean req_cast = FALSE;
		if (vala_member_access_get_inner (ma) == NULL) {
			ValaCCodeExpression* __temp786 = NULL;
			instance = (__temp786 = VALA_CCODE_EXPRESSION (g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL)), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp786);
			/* require casts for inherited properties */
			req_cast = (vala_code_node_get_symbol (VALA_CODE_NODE (prop))->parent_symbol != self->priv->current_type_symbol);
		} else {
			ValaCCodeExpression* __temp788 = NULL;
			ValaCCodeExpression* __temp787 = NULL;
			instance = (__temp788 = (__temp787 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (ma)))), (__temp787 == NULL ? __temp787 : g_object_ref (__temp787))), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp788);
			/* require casts if the type of the used instance is
			 * different than the type which declared the property */
			req_cast = vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (prop))->parent_symbol) != vala_expression_get_static_type (vala_member_access_get_inner (ma))->type;
		}
		if (req_cast && vala_data_type_is_reference_type ((VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (prop))->parent_symbol))))) {
			ValaCCodeIdentifier* __temp790 = NULL;
			char* __temp789 = NULL;
			ValaCCodeFunctionCall* __temp791 = NULL;
			ValaCCodeFunctionCall* ccast = (__temp791 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp790 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp789 = vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (prop))->parent_symbol))), NULL)), NULL)), NULL), (__temp790 == NULL ? NULL : (__temp790 = (g_object_unref (__temp790), NULL))), (__temp789 == NULL ? NULL : (__temp789 = (g_free (__temp789), NULL))), __temp791);
			vala_ccode_function_call_add_argument (ccast, instance);
			ValaCCodeExpression* __temp793 = NULL;
			ValaCCodeFunctionCall* __temp792 = NULL;
			instance = (__temp793 = VALA_CCODE_EXPRESSION ((__temp792 = ccast, (__temp792 == NULL ? __temp792 : g_object_ref (__temp792)))), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp793);
			(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
		}
		vala_ccode_function_call_add_argument (ccall, instance);
		ValaCCodeExpression* __temp794 = NULL;
		ValaCCodeExpression* cexpr = (__temp794 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_right (a)))), (__temp794 == NULL ? __temp794 : g_object_ref (__temp794)));
		if (prop->no_accessor_method) {
			/* property name is second argument of g_object_set */
			ValaCCodeConstant* __temp795 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp795 = vala_property_get_canonical_cconstant (prop))));
			(__temp795 == NULL ? NULL : (__temp795 = (g_object_unref (__temp795), NULL)));
		} else if (vala_property_get_type_reference (prop)->type != NULL && vala_data_type_is_reference_type (vala_property_get_type_reference (prop)->type) && vala_expression_get_static_type (vala_assignment_get_right (a))->type != NULL && vala_property_get_type_reference (prop)->type != vala_expression_get_static_type (vala_assignment_get_right (a))->type) {
			/* cast is necessary */
			ValaCCodeIdentifier* __temp797 = NULL;
			char* __temp796 = NULL;
			ValaCCodeFunctionCall* __temp798 = NULL;
			ValaCCodeFunctionCall* ccast = (__temp798 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp797 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp796 = vala_data_type_get_upper_case_cname (vala_property_get_type_reference (prop)->type, NULL)), NULL)), NULL), (__temp797 == NULL ? NULL : (__temp797 = (g_object_unref (__temp797), NULL))), (__temp796 == NULL ? NULL : (__temp796 = (g_free (__temp796), NULL))), __temp798);
			vala_ccode_function_call_add_argument (ccast, cexpr);
			ValaCCodeExpression* __temp800 = NULL;
			ValaCCodeFunctionCall* __temp799 = NULL;
			cexpr = (__temp800 = VALA_CCODE_EXPRESSION ((__temp799 = ccast, (__temp799 == NULL ? __temp799 : g_object_ref (__temp799)))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp800);
			(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
		}
		vala_ccode_function_call_add_argument (ccall, cexpr);
		if (prop->no_accessor_method) {
			ValaCCodeConstant* __temp801 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp801 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL))));
			(__temp801 == NULL ? NULL : (__temp801 = (g_object_unref (__temp801), NULL)));
		}
		vala_code_node_set_ccodenode (VALA_CODE_NODE (a), VALA_CCODE_NODE (ccall));
		(set_func == NULL ? NULL : (set_func = (g_free (set_func), NULL)));
		(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		(instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL)));
		(cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL)));
	} else if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))))) {
		ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))));
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_right (a))));
		char* connect_func = "g_signal_connect_object";
		if (!vala_method_get_instance (m)) {
			connect_func = "g_signal_connect";
		}
		ValaCCodeIdentifier* __temp802 = NULL;
		ValaCCodeFunctionCall* __temp803 = NULL;
		ValaCCodeFunctionCall* ccall = (__temp803 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp802 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", connect_func, NULL)), NULL), (__temp802 == NULL ? NULL : (__temp802 = (g_object_unref (__temp802), NULL))), __temp803);
		if (vala_member_access_get_inner (ma) != NULL) {
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (ma)))));
		} else {
			ValaCCodeIdentifier* __temp804 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp804 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL))));
			(__temp804 == NULL ? NULL : (__temp804 = (g_object_unref (__temp804), NULL)));
		}
		ValaCCodeConstant* __temp806 = NULL;
		char* __temp805 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp806 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", (__temp805 = g_strdup_printf ("\"%s\"", vala_signal_get_name (sig))), NULL))));
		(__temp806 == NULL ? NULL : (__temp806 = (g_object_unref (__temp806), NULL)));
		(__temp805 == NULL ? NULL : (__temp805 = (g_free (__temp805), NULL)));
		ValaCCodeCastExpression* __temp808 = NULL;
		ValaCCodeIdentifier* __temp807 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp808 = g_object_new (VALA_TYPE_CCODE_CAST_EXPRESSION, "inner", (__temp807 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_method_get_cname (m), NULL)), "type_name", "GCallback", NULL))));
		(__temp808 == NULL ? NULL : (__temp808 = (g_object_unref (__temp808), NULL)));
		(__temp807 == NULL ? NULL : (__temp807 = (g_object_unref (__temp807), NULL)));
		if (vala_method_get_instance (m)) {
			if (VALA_IS_MEMBER_ACCESS (vala_assignment_get_right (a))) {
				ValaMemberAccess* right_ma = VALA_MEMBER_ACCESS (vala_assignment_get_right (a));
				if (vala_member_access_get_inner (right_ma) != NULL) {
					vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (right_ma)))));
				} else {
					ValaCCodeIdentifier* __temp809 = NULL;
					vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp809 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL))));
					(__temp809 == NULL ? NULL : (__temp809 = (g_object_unref (__temp809), NULL)));
				}
			} else if (VALA_IS_LAMBDA_EXPRESSION (vala_assignment_get_right (a))) {
				ValaCCodeIdentifier* __temp810 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp810 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", "self", NULL))));
				(__temp810 == NULL ? NULL : (__temp810 = (g_object_unref (__temp810), NULL)));
			}
			ValaCCodeConstant* __temp811 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp811 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "0", NULL))));
			(__temp811 == NULL ? NULL : (__temp811 = (g_object_unref (__temp811), NULL)));
		} else {
			ValaCCodeConstant* __temp812 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp812 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", "NULL", NULL))));
			(__temp812 == NULL ? NULL : (__temp812 = (g_object_unref (__temp812), NULL)));
		}
		vala_code_node_set_ccodenode (VALA_CODE_NODE (a), VALA_CCODE_NODE (ccall));
		(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	} else {
		/* explicitly use strong reference as ccast gets
		 * unrefed at end of inner block
		 */
		ValaCCodeExpression* __temp813 = NULL;
		ValaCCodeExpression* rhs = (__temp813 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_right (a)))), (__temp813 == NULL ? __temp813 : g_object_ref (__temp813)));
		if (vala_expression_get_static_type (vala_assignment_get_left (a))->type != NULL && vala_expression_get_static_type (vala_assignment_get_right (a))->type != NULL && vala_data_type_is_reference_type (vala_expression_get_static_type (vala_assignment_get_left (a))->type) && vala_expression_get_static_type (vala_assignment_get_right (a))->type != vala_expression_get_static_type (vala_assignment_get_left (a))->type) {
			ValaCCodeIdentifier* __temp815 = NULL;
			char* __temp814 = NULL;
			ValaCCodeFunctionCall* __temp816 = NULL;
			ValaCCodeFunctionCall* ccast = (__temp816 = g_object_new (VALA_TYPE_CCODE_FUNCTION_CALL, "call", (__temp815 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", (__temp814 = vala_data_type_get_upper_case_cname (vala_expression_get_static_type (vala_assignment_get_left (a))->type, NULL)), NULL)), NULL), (__temp815 == NULL ? NULL : (__temp815 = (g_object_unref (__temp815), NULL))), (__temp814 == NULL ? NULL : (__temp814 = (g_free (__temp814), NULL))), __temp816);
			vala_ccode_function_call_add_argument (ccast, rhs);
			ValaCCodeExpression* __temp818 = NULL;
			ValaCCodeFunctionCall* __temp817 = NULL;
			rhs = (__temp818 = VALA_CCODE_EXPRESSION ((__temp817 = ccast, (__temp817 == NULL ? __temp817 : g_object_ref (__temp817)))), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp818);
			(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
		}
		if (vala_code_generator_get_memory_management (self) && vala_type_reference_get_is_lvalue_ref (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
			/* unref old value */
			ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
			ValaVariableDeclarator* temp_decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (vala_assignment_get_left (a)));
			ValaVariableDeclarator* __temp819 = NULL;
			self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp819 = temp_decl, (__temp819 == NULL ? __temp819 : g_object_ref (__temp819))));
			ValaCCodeAssignment* __temp821 = NULL;
			ValaCCodeIdentifier* __temp820 = NULL;
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp821 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", (__temp820 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (temp_decl), NULL)), "right", rhs, NULL))));
			(__temp821 == NULL ? NULL : (__temp821 = (g_object_unref (__temp821), NULL)));
			(__temp820 == NULL ? NULL : (__temp820 = (g_object_unref (__temp820), NULL)));
			ValaCCodeExpression* __temp822 = NULL;
			vala_ccode_comma_expression_append_expression (ccomma, (__temp822 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_left (a)))), vala_expression_get_static_type (vala_assignment_get_left (a)))));
			(__temp822 == NULL ? NULL : (__temp822 = (g_object_unref (__temp822), NULL)));
			ValaCCodeIdentifier* __temp823 = NULL;
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp823 = g_object_new (VALA_TYPE_CCODE_IDENTIFIER, "name", vala_variable_declarator_get_name (temp_decl), NULL))));
			(__temp823 == NULL ? NULL : (__temp823 = (g_object_unref (__temp823), NULL)));
			ValaCCodeExpression* __temp825 = NULL;
			ValaCCodeCommaExpression* __temp824 = NULL;
			rhs = (__temp825 = VALA_CCODE_EXPRESSION ((__temp824 = ccomma, (__temp824 == NULL ? __temp824 : g_object_ref (__temp824)))), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp825);
			(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
			(temp_decl == NULL ? NULL : (temp_decl = (g_object_unref (temp_decl), NULL)));
		}
		ValaCCodeAssignmentOperator cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE;
		if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_OR) {
			cop = VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_OR;
		} else if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_AND) {
			cop = VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_AND;
		} else if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_XOR) {
			cop = VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_XOR;
		} else if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_ADD) {
			cop = VALA_CCODE_ASSIGNMENT_OPERATOR_ADD;
		} else if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SUB) {
			cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SUB;
		} else if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_MUL) {
			cop = VALA_CCODE_ASSIGNMENT_OPERATOR_MUL;
		} else if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_DIV) {
			cop = VALA_CCODE_ASSIGNMENT_OPERATOR_DIV;
		} else if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_PERCENT) {
			cop = VALA_CCODE_ASSIGNMENT_OPERATOR_PERCENT;
		} else if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SHIFT_LEFT) {
			cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SHIFT_LEFT;
		} else if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SHIFT_RIGHT) {
			cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SHIFT_RIGHT;
		}
		ValaCCodeAssignment* __temp826 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (a), VALA_CCODE_NODE ((__temp826 = g_object_new (VALA_TYPE_CCODE_ASSIGNMENT, "left", VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_left (a)))), "right", rhs, "operator", cop, NULL))));
		(__temp826 == NULL ? NULL : (__temp826 = (g_object_unref (__temp826), NULL)));
		(rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL)));
	}
}


gboolean vala_code_generator_get_memory_management (ValaCodeGenerator* self)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), FALSE);
	return self->priv->_memory_management;
}


void vala_code_generator_set_memory_management (ValaCodeGenerator* self, gboolean value)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	self->priv->_memory_management = value;
}


void vala_code_generator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (object);
	switch (property_id) {
		case VALA_CODE_GENERATOR_MEMORY_MANAGEMENT:
		g_value_set_boolean (value, vala_code_generator_get_memory_management (self));
		break;
	}
}


void vala_code_generator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (object);
	switch (property_id) {
		case VALA_CODE_GENERATOR_MEMORY_MANAGEMENT:
		vala_code_generator_set_memory_management (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_code_generator_class_init (ValaCodeGeneratorClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaCodeGeneratorPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_code_generator_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_code_generator_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_code_generator_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_source_file = vala_code_generator_real_visit_begin_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_source_file = vala_code_generator_real_visit_end_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_class = vala_code_generator_real_visit_begin_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_class = vala_code_generator_real_visit_end_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_struct = vala_code_generator_real_visit_begin_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_interface = vala_code_generator_real_visit_begin_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_interface = vala_code_generator_real_visit_end_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_enum = vala_code_generator_real_visit_begin_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum_value = vala_code_generator_real_visit_enum_value;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_callback = vala_code_generator_real_visit_end_callback;
	VALA_CODE_VISITOR_CLASS (klass)->visit_constant = vala_code_generator_real_visit_constant;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vala_code_generator_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_method = vala_code_generator_real_visit_begin_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_method = vala_code_generator_real_visit_end_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_formal_parameter = vala_code_generator_real_visit_formal_parameter;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property = vala_code_generator_real_visit_end_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property_accessor = vala_code_generator_real_visit_end_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_constructor = vala_code_generator_real_visit_end_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_block = vala_code_generator_real_visit_begin_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_block = vala_code_generator_real_visit_end_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_empty_statement = vala_code_generator_real_visit_empty_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_declaration_statement = vala_code_generator_real_visit_declaration_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_variable_declarator = vala_code_generator_real_visit_variable_declarator;
	VALA_CODE_VISITOR_CLASS (klass)->visit_initializer_list = vala_code_generator_real_visit_initializer_list;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_full_expression = vala_code_generator_real_visit_end_full_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_expression_statement = vala_code_generator_real_visit_expression_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_if_statement = vala_code_generator_real_visit_if_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_while_statement = vala_code_generator_real_visit_while_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_for_statement = vala_code_generator_real_visit_for_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_foreach_statement = vala_code_generator_real_visit_end_foreach_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_break_statement = vala_code_generator_real_visit_break_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_continue_statement = vala_code_generator_real_visit_continue_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_return_statement = vala_code_generator_real_visit_return_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_boolean_literal = vala_code_generator_real_visit_boolean_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_character_literal = vala_code_generator_real_visit_character_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_integer_literal = vala_code_generator_real_visit_integer_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_real_literal = vala_code_generator_real_visit_real_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_string_literal = vala_code_generator_real_visit_string_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_null_literal = vala_code_generator_real_visit_null_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_literal_expression = vala_code_generator_real_visit_literal_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_parenthesized_expression = vala_code_generator_real_visit_parenthesized_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_member_access = vala_code_generator_real_visit_member_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_invocation_expression = vala_code_generator_real_visit_end_invocation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_element_access = vala_code_generator_real_visit_element_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_postfix_expression = vala_code_generator_real_visit_postfix_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_object_creation_expression = vala_code_generator_real_visit_object_creation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_typeof_expression = vala_code_generator_real_visit_typeof_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_unary_expression = vala_code_generator_real_visit_unary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_cast_expression = vala_code_generator_real_visit_cast_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_binary_expression = vala_code_generator_real_visit_binary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_type_check = vala_code_generator_real_visit_type_check;
	VALA_CODE_VISITOR_CLASS (klass)->visit_conditional_expression = vala_code_generator_real_visit_conditional_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_lambda_expression = vala_code_generator_real_visit_end_lambda_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_assignment = vala_code_generator_real_visit_end_assignment;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CODE_GENERATOR_MEMORY_MANAGEMENT, g_param_spec_boolean ("memory-management", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_code_generator_init (ValaCodeGenerator * self)
{
	self->priv = VALA_CODE_GENERATOR_GET_PRIVATE (self);
	self->priv->next_temp_var_id = 0;
}


static void vala_code_generator_dispose (GObject * obj)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (obj);
	(self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL)));
	(self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL)));
	(self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL)));
	(self->priv->header_begin == NULL ? NULL : (self->priv->header_begin = (g_object_unref (self->priv->header_begin), NULL)));
	(self->priv->header_type_declaration == NULL ? NULL : (self->priv->header_type_declaration = (g_object_unref (self->priv->header_type_declaration), NULL)));
	(self->priv->header_type_definition == NULL ? NULL : (self->priv->header_type_definition = (g_object_unref (self->priv->header_type_definition), NULL)));
	(self->priv->header_type_member_declaration == NULL ? NULL : (self->priv->header_type_member_declaration = (g_object_unref (self->priv->header_type_member_declaration), NULL)));
	(self->priv->source_begin == NULL ? NULL : (self->priv->source_begin = (g_object_unref (self->priv->source_begin), NULL)));
	(self->priv->source_include_directives == NULL ? NULL : (self->priv->source_include_directives = (g_object_unref (self->priv->source_include_directives), NULL)));
	(self->priv->source_type_member_declaration == NULL ? NULL : (self->priv->source_type_member_declaration = (g_object_unref (self->priv->source_type_member_declaration), NULL)));
	(self->priv->source_type_member_definition == NULL ? NULL : (self->priv->source_type_member_definition = (g_object_unref (self->priv->source_type_member_definition), NULL)));
	(self->priv->instance_struct == NULL ? NULL : (self->priv->instance_struct = (g_object_unref (self->priv->instance_struct), NULL)));
	(self->priv->type_struct == NULL ? NULL : (self->priv->type_struct = (g_object_unref (self->priv->type_struct), NULL)));
	(self->priv->instance_priv_struct == NULL ? NULL : (self->priv->instance_priv_struct = (g_object_unref (self->priv->instance_priv_struct), NULL)));
	(self->priv->prop_enum == NULL ? NULL : (self->priv->prop_enum = (g_object_unref (self->priv->prop_enum), NULL)));
	(self->priv->cenum == NULL ? NULL : (self->priv->cenum = (g_object_unref (self->priv->cenum), NULL)));
	(self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL)));
	(self->priv->block == NULL ? NULL : (self->priv->block = (g_object_unref (self->priv->block), NULL)));
	(self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL)));
	(self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL)));
	(self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL)));
	(self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL)));
	ValaCodeGeneratorClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CODE_GENERATOR_CLASS (g_type_class_peek (VALA_TYPE_CODE_GENERATOR));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_code_generator_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCodeGeneratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_code_generator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCodeGenerator), 0, (GInstanceInitFunc) vala_code_generator_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaCodeGenerator", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



